/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: Dshowmanual.cpp  * Status: stable * Version: 2.00 * last changes in version: 0.45 	*/
/************************************************************************************************/
/*  This file contains functions of ShowManualDialog class                                  	*/
/*												*/
/*   The class are defined in Dshowmanual.h							*/
/*												*/
/*   It's called by MyMainWindow class defined in mainwindow.h, mainwindow.cpp, Dmainwindow.ui  */
/*												*/
/*   It's based in ShowSolutionDialog class defined in Dshowsolution.h, Dshowsolution.cpp 	*/
/*												*/
/************************************************************************************************/

#include <QtWidgets>
#include "Dshowmanual.h"
#include <QProcess>

#include "dmem.h"

/***************/
/* Constructor */
/***************/
ShowManualDialog::ShowManualDialog(bool English , QWidget *parent)
		: QDialog(parent),
                  English1OrSpanish0 (English)  // Set Languange variable
{

	QString path = ("file:///");
	path.append( QDir::currentPath() );

	/* Set Typical Attributes */
	setAttribute(Qt::WA_QuitOnClose,false);
	setAttribute(Qt::WA_DeleteOnClose,false);
	setWindowFlags(Qt::WindowShadeButtonHint);



	/* Set TextBrowser */	
	TextBrowser = DM_NEW ( new QTextBrowser() );
	
	

	/* Set Manual file. English or Spanish */
	if (English1OrSpanish0)
	{
		path.append("/doc/Manual_en.html");
//		path.append("/doc/Manual_es.html");
	}
	else 
	{
		path.append("/doc/Manual_es.html");
	}

	TextBrowser->setHtml( path );
	TextBrowser->setSource( path );
	TextBrowser->setOpenExternalLinks(true);

	//Button Close
	botClose = DM_NEW ( new QPushButton(tr("&Close")) );
	botIndex = DM_NEW ( new QPushButton(tr("&Index")) );
	botNext  = DM_NEW ( new QPushButton(tr(">>")) );
	botBack  = DM_NEW ( new QPushButton(tr("<<")) );

	/* CONNECTIONS */

	connect(botClose, SIGNAL(clicked()), this, SLOT(close()));
	connect(botIndex, SIGNAL(clicked()), TextBrowser, SLOT(home ()));
	connect(botBack , SIGNAL(clicked()), this, SLOT(mBack ()));
	connect(botNext , SIGNAL(clicked()), this, SLOT(mNext ()));

	/********************/

	// Put buttons on layouts

	// Layout of Buttons
	LayoutButtons = DM_NEW ( new QHBoxLayout );
	LayoutButtons->addWidget(botIndex);
	LayoutButtons->addStretch();
	LayoutButtons->addWidget(botBack);
	LayoutButtons->addWidget(botNext);
	LayoutButtons->addStretch();
	LayoutButtons->addWidget(botClose);

	// Final Layout 
	MainLayout = DM_NEW ( new QVBoxLayout );
	MainLayout->addWidget(TextBrowser);
	MainLayout->addLayout(LayoutButtons);

	// See the Final Layout:
	setLayout(MainLayout);

	setWindowTitle(tr("Manual"));

	//Ajust size
	//TODO: Review and change if is necessary
	
	setMinimumSize(450,450);
	resize(500,500);

}
/**************/
/* Destructor */
/**************/
ShowManualDialog::~ShowManualDialog()
{
	/* Delete All */
	delete DM_DEL (TextBrowser  );
	delete DM_DEL (botClose     );

	delete DM_DEL (LayoutButtons	);
	delete DM_DEL (MainLayout	);
}

/*********/
/* mNext */
/*****************************************/
/* This function load the next help file */
/*****************************************/
void ShowManualDialog::mNext()
{

	QString path = ("file:///");
	QString url  = TextBrowser->source().path();;
	path.append( QDir::currentPath() );

	if (English1OrSpanish0)
	{
		path.append("/doc/manual_en/");
	}
	else 
	{
		path.append("/doc/manual_es/");
	}
	
	
	//						path.append("1Instalacion.html");
	if (url.contains("1Instalacion.html"))		path.append("2Introduccion.html");
	else if (url.contains("2Introduccion.html"))	path.append("3SistemaJuego.html");
	else if (url.contains("3SistemaJuego.html"))	path.append("4descripcion.html");
	else if (url.contains("4descripcion.html"))	path.append("5Glosario.html");
	else if (url.contains("5Glosario.html"))	path.append("6ReporteErrores.html");
	else if (url.contains("6ReporteErrores.html"))	path.append("7LicenseGPL.html");
	else return;
	
	TextBrowser->setSource( path );
}

/*********/
/* mBack */
/********************************************/
/* This function load the previus help file */
/********************************************/
void ShowManualDialog::mBack()
{


	QString path = ("file:///");
	QString url  = TextBrowser->source().path();
	path.append( QDir::currentPath() );

	if (English1OrSpanish0)
	{
		path.append("/doc/manual_en/");
	}
	else 
	{
		path.append("/doc/manual_es/");
	}
	
	
	if (url.contains("2Introduccion.html"))		path.append("1Instalacion.html");
	else if (url.contains("3SistemaJuego.html"))	path.append("2Introduccion.html");
	else if (url.contains("4descripcion.html"))	path.append("3SistemaJuego.html");
	else if (url.contains("5Glosario.html"))	path.append("4descripcion.html");
	else if (url.contains("6ReporteErrores.html"))	path.append("5Glosario.html");
	else if (url.contains("7LicenseGPL.html"))	path.append("6ReporteErrores.html");
	else return; 

	
	TextBrowser->setSource( path );
}

