/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: SudokuBoard.h  * Status: stable * Version: 2.00 * last changes in version: 0.46 	*/
/************************************************************************************************/
/*  This file contains the definition of SudokuBoard, a wrapper class for t_board		*/
/*	All methods are defined inline                                                      */
/*												*/
/************************************************************************************************/

#ifndef SUDOKUBOARD_H
#define SUDOKUBOARD_H

#include "sudoku-pub.h"
#include "graphics.h"
#include "dmem.h"
#include "myassert.h"

class SudokuView;

class SudokuBoard
{
  private:

    SudokuView * pView;
    t_board_p pBoard;
    GraphCell * pGraphCells;

    int nSy;
    int nCl;
    int nGr;
    
  public:

    typedef enum { basic, samurai, custom } boardtype;

    SudokuBoard (SudokuView * pv,
                 boardtype t,
                 bool diag=false,
                 int wb=3, int hb=-1,
                 int nGrids=-1,
                 int * xg=NULL, int * yg=NULL)
      : pView(pv),
        pBoard(NULL),
        pGraphCells(NULL)
    {
      if (hb<0)
        hb = wb;

      pBoard = t==basic   ? ConstructBasicBoard   (wb, diag)     :
               t==samurai ? ConstructSamuraiBoard (wb, hb, diag) :
               t==custom  ? ConstructCustomBoard  (wb, hb, diag,
                                                   nGrids,
                                                   xg, yg)       :
                            NULL;
      if (pBoard)
      {
        nSy = GetNumSymbols (pBoard);
        nCl = GetNumCells (pBoard);
        nGr = GetNumGroups (pBoard);

        if (pView)
        {
          pGraphCells = DM_NEW ( new GraphCell[nCl] );

          for (int i=0; i<nCl; i++)
            pGraphCells[i].board = this;
        }
      }
    }

    ~SudokuBoard ()
    {
      if (pGraphCells)
        delete [] ( DM_DEL (pGraphCells) );
        
      if (pBoard)
        DestroyBoard (pBoard);
    }

    void Clear ()
    {
      if (pBoard)
        CleanBoard (pBoard);
        
      if (pGraphCells)
        for (int i=0; i<nCl; i++)
          pGraphCells[i].ClearCell ();
    }

    GraphCell * GCell (int n)
    {
      MY_ASSERT (pGraphCells!=NULL);
      MY_ASSERT (nCl>0);
      MY_ASSERT (n>=0 && n<nCl);
      return pGraphCells + n;
    }

    int CellNumber (GraphCell * pCell)
    {
      MY_ASSERT (pGraphCells!=NULL);
      MY_ASSERT (nCl>0);
      MY_ASSERT (pCell!=NULL);
      int n = pCell - pGraphCells;
      MY_ASSERT (n>=0 && n<nCl);
      return n;
    }

    t_board_p Board () { return pBoard; }
    
    int NumCells () const { return nCl; }
    int NumSymbols () const { return nSy; }
    int NumGroups () const { return nGr; }
    
    SudokuView * View () { return pView; }
};

#endif
