/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: SudokuView.cpp  * Status: stable * Version: 2.00 * last changes in version: 0.50	*/
/************************************************************************************************/
/*  This file contains the functions of SudokuView class					*/
/*												*/
/*   The header	 of the class is in SudokuView.h						*/
/*												*/
/************************************************************************************************/
#include "SudokuView.h"
#include "SudokuBoard.h"
#include "sudoku-pub.h"
#include "mainwindow.h"
#include "mask.h"
#include <math.h>

#include <QtWidgets>
//#include <QtOpenGL>
#include <QPainter>
#include <QColor>
#include <QMessageBox>
#include <QWidget>
#include <QMainWindow>
#include <QGraphicsRectItem>
#include <QFile>

#include "dmem.h"
#include "myassert.h"

//#define _DEBUG

//#include <QDialog>

/*
  SDK_FORMAT_VERSION
  
  This macro controls the output format for saving games.
  Possible values are:
  
    0:     Initial format without preamble and with the magic
           number 0x100CAFE5 after the board description.

    0x100: Initial format with an additional preamble.
  
    0x200: Reduced format with preamble.
    
  The program will read and understand the three formats,
  but it will always write using the format specified by
  the macro defined below. This setting can be overriden
  at compilation time by a -D... parameter.

  The program will also be able to understand some formats
  derived from the previous ones. Whenever the preamble is
  detected, the version field will be interpreted as
  follows:

    0..0xFF:      Invalid. These formats don't have any
                  preamble. Show error message.

    0x101..0x1FF: Same as 0x100, but with additional data at
                  the end (which will be ignored).
  
    0x201..0x2FF: Same as 0x200, but with additional data at
                  the end (which will be ignored).

    0x300..0xFFFFFFFF: Unsupported. Show error message.

  In any case, this program will only use the grids
  configuration and the masks of possible values for the
  cells.
*/

#define DEFAULT_SDK_FORMAT_VERSION  0x100

#ifndef SDK_FORMAT_VERSION
#define SDK_FORMAT_VERSION  DEFAULT_SDK_FORMAT_VERSION
#endif

#if SDK_FORMAT_VERSION != 0
#if SDK_FORMAT_VERSION != 0x100
#if SDK_FORMAT_VERSION != 0x200
#error "Unknown SDK format version"
#endif
#endif
#endif

/************************/
/* DEFAULT CONSTRUCTOR  */
/************************/
SudokuView::SudokuView( QWidget *parent) : 
	      QFrame(parent),
	    isPlaying (true),
	 sudoku_board (NULL),
	 check_board  (NULL),
	 thread_board (NULL),
	   pMainWindow(NULL),
	 mScene       (NULL),
	 graphicsView (NULL),
       pMaskCellsArray(NULL),
      isActiveHelpBox(false),
   ThisSudokuHasErrors(false),
flagSudokuWithErrorsinAllMoment(false),
SudokuSolutionwasfound(false),
	  mRulesDialog(NULL),
	mEnlargeWindow(NULL),
	 flagEnlarge (true ),
	 ppGroupsRect (NULL),
 	      nmGroups( 0  ),
	 ppGridsRect  (NULL),
	       nmGrids( 0  )
{
#ifdef _DEBUG
	printf("SudokuView::SudokuView 1\n");
#endif	
	//setPalette(QPalette(QColor(250, 250, 200)));
	//    setAutoFillBackground(true);
	

	NoPlayerMovement = true;
	/* Setting Variables */

	ListLastColorModifed.clear();
	ListLastRulesModifed.clear();

	RestartUndo();
	
//	colorCell  = QColor(0,0,200);
//	colorBlock = QColor(0,100,100);
//	colorGrid  = QColor(200,0,0);

	for (int i=0;i<15;i++)
	{
	  GridsX[i]=0;
	  GridsY[i]=0;
	}
	

	/* Setting Style of Frame */
	setFrameStyle ( Sunken | StyledPanel);

	setFrameRect(parent->rect());	
	
	
	/* Create and Setting Threads */
	mSenseiThread = DM_NEW (new SenseiThread());
	mCheckThread  = DM_NEW (new SenseiThread());

	connect (mCheckThread, SIGNAL(IntegrityChecked (int)),this,SLOT(CheckIntegrity(int )));

	/* Create and Setting graphicsView */
	graphicsView = DM_NEW (new QGraphicsView(this) );
	//graphicsView->setRenderHint (QPainter::Antialiasing, false); // TODO
	//graphicsView->setDragMode (QGraphicsView::RubberBandDrag);
	graphicsView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	graphicsView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

//	graphicsView->setAutoFillBackground(true);

      /* Create mScene and Setting Size of graphicsView and mScene */
	mScene = DM_NEW (new QGraphicsScene ) ;	

	/* This Size will change whe I resize */
	/*graphicsView->setSceneRect(0 , 0 ,0 , 0);
	mScene->setSceneRect( 0, 0, 0, 0 ) ;
	graphicsView->setSceneRect(0 , 0 ,0 , 0);
	mScene->setSceneRect( 0, 0, 0, 0 ) ;*/


	//setRect( QRectF(0,0, parent->width()  , parent->height()) );
	//resize( (int) parent->width()   , (int) parent->height() );
	graphicsView->setSceneRect( QRectF(0,0, (int) parent->width()  ,(int)  parent->height()) );
	mScene->setSceneRect( QRectF(0,0, (int) parent->width()  , (int)  parent->height()) );
	graphicsView->resize( (int) parent->width()   , (int) parent->height() );

	
	//ensureVisible(QRectF(0,0,50,50));


	
	graphicsView->setScene(mScene); //TODO Segmention Fault ?? if Sudoki es Central Widget
	//resize (parent->size());
	graphicsView->show();

	/* Set Fonts */
	mFont = QFont("Courier",5);
	mBigFont = QFont("Courier",10);

	/* About Rules Window */
	mRulesDialog = DM_NEW (new RulesDialog(this));

	connect (mSenseiThread, SIGNAL(NewRule ()),mRulesDialog,SLOT(AddNewRule()));
//	connect (mSenseiThread, SIGNAL(Aborted ()),mRulesDialog,SLOT(ClearRules()));
	/***/
#ifdef _DEBUG
	printf("SudokuView::SudokuView END\n");
#endif	
}


/****************/
/*  DESTRUCTOR  */
/****************/

SudokuView::~SudokuView()
{
#ifdef _DEBUG
	printf("SudokuView::~SudokuView 1\n");
#endif	
	int i=0;

	ListLastColorModifed.clear();
	ListLastRulesModifed.clear();
	if (sudoku_board)
	{
		/* Free Memory of Block and Grids */
		for (i=0; i < nmGroups; i++)
		{
			mScene->removeItem(ppGroupsRect[i]);
			if (ppGroupsRect[i]) delete DM_DEL (ppGroupsRect[i]);
		}
		free (DM_DEL (ppGroupsRect) );
		for (i=0; i < nmGrids; i++)
		{
			mScene->removeItem(ppGridsRect[i]);
			if (ppGridsRect[i]) delete DM_DEL (ppGridsRect[i]);
		}
		free (DM_DEL (ppGridsRect) );

	
		for (i=0; i < sudoku_board->NumCells(); i++)
			mScene->removeItem(sudoku_board->GCell(i));
	}
	delete DM_DEL (sudoku_board);
	delete DM_DEL (check_board);
	delete DM_DEL (thread_board);

	delete [] ( DM_DEL (pMaskCellsArray) );

	delete DM_DEL (graphicsView );
	delete DM_DEL (mSenseiThread);
	delete DM_DEL (mCheckThread );

        if (mRulesDialog  ) 
	delete DM_DEL (mRulesDialog  );

	if (mEnlargeWindow)
	 delete DM_DEL (mEnlargeWindow);

	delete DM_DEL (mScene       );

	
#ifdef _DEBUG
	printf("SudokuView::~SudokuView END\n");
#endif	
}

void SudokuView::paintEvent(QPaintEvent *event)
{
#ifdef _DEBUG
	printf("SudokuView::paintEvent 1\n");
#endif
	// setAutoFillBackground(true);
	QFrame::paintEvent (event);
}

/****************/
/* resizeEvent  */
/****************************************************************/
/* Called when a user resize MainWindow; it repaint the Sudoku  */
/****************************************************************/

void SudokuView::resizeEvent (QResizeEvent *event)
{
#ifdef _DEBUG
	printf("SudokuView::resizeEvent 1\n");
#endif	
	float width_window=0;
	float height_window=0;
	float w_block=0, h_block=0;
	int i=0;
	
	QApplication::setOverrideCursor (Qt::WaitCursor);
	width_window  = width();
	height_window = height();
	graphicsView->setSceneRect( QRectF(0,0,(int) width_window  , (int) height_window) );
	graphicsView->resize( (int) width_window  , (int) height_window );


	// Redrawing the Cells 


//	QMatrix matrix;
//	matrix.scale(  width_window / mScene->width()  , height_window / mScene->height()  );

//	graphicsView->setMatrix( matrix);

	if (sudoku_board)
	{	
	        int nSy = sudoku_board->NumSymbols ();
	        
		/* Calculate size */
		t_board_p pBoard = sudoku_board->Board ();
                int w = GetBoardWidth (pBoard);
                int h = GetBoardHeight (pBoard);
		w_block = ( width_window - (WIDTH_GRID_BORDER * (w + 1))- _2WIDTH_W_BORDER) 
                          / w;
		h_block = ( height_window - (WIDTH_GRID_BORDER * (h+ 1))- _2WIDTH_H_BORDER) 
                          / h;
		

		// Resize CELLs
		for (i=0; i < sudoku_board->NumCells() ; i++)
		{
		        int x = GetCellX (pBoard, i);
		        int y = GetCellY (pBoard, i);
			sudoku_board->GCell(i)->setRect(
					QRectF(
					x * (w_block + WIDTH_GRID_BORDER) +
					WIDTH_W_BORDER,
					y * (h_block + WIDTH_GRID_BORDER) +
					WIDTH_H_BORDER,
						w_block,
						h_block) );
			//mScene->addItem( sudoku_board->GCell(i));
		}
		
		// Resize BLOCKs
		int k=0;
		for (i =0; i < GetNumGroups(pBoard); i++)
		{
			if (GetGroupType(pBoard,i) == GR_BLOCK)
			{
			        int cell = GetCellOfGroup (pBoard, 0, i);
		                int x = GetCellX (pBoard, cell);
		                int y = GetCellY (pBoard, cell);
		                int wb = GetBlockWidth (pBoard);
		                int hb = GetBlockHeight (pBoard);
	
				ppGroupsRect[k]->setRect (
					QRectF(
					x * (w_block + WIDTH_GRID_BORDER) +
					- HALF_WIDTH_BLOCK_BORDER + WIDTH_W_BORDER,
					y * (h_block + WIDTH_GRID_BORDER) +
					- HALF_WIDTH_BLOCK_BORDER + WIDTH_H_BORDER,
					wb * (w_block + WIDTH_GRID_BORDER),
					hb * (h_block + WIDTH_GRID_BORDER)
					) );
				k++;
			}	
		}
		
		k=0;	
		// Resize GRIDs
		for (i =0; i < GetNumGroups(pBoard); i++)
		{
			if (i == k*(3* nSy)) //sudoku_board->pGr[i].type == GR_DIAG)
			{
			        int cell = GetCellOfGroup (pBoard, 0, i);
		                int x = GetCellX (pBoard, cell);
		                int y = GetCellY (pBoard, cell);
		                int wb = GetBlockWidth (pBoard);
		                int hb = GetBlockHeight (pBoard);
		                
				ppGridsRect[k]->setRect (
					QRectF(
					x * (w_block  + WIDTH_GRID_BORDER) +
					-HALF_WIDTH_GRID_BORDER + WIDTH_W_BORDER,
					y * (h_block  + WIDTH_GRID_BORDER) +
					-HALF_WIDTH_GRID_BORDER + WIDTH_H_BORDER,
					
					wb * hb * (w_block + WIDTH_GRID_BORDER),
					hb * wb * (h_block +WIDTH_GRID_BORDER))) ;
				k++; 
			}	
		}
		 ResizeCellFont(); // TODO OPTIMIZE
	}
	mScene->setSceneRect( QRectF(0,0, width_window  , height_window));
	mScene->update(QRectF(0,0,mScene->width(),mScene->height()));

	if (mEnlargeWindow)
		if (mEnlargeWindow->isVisible())
		{
			mEnlargeWindow->centerWindow(sudoku_board->CellNumber(mEnlargeWindow->mCellFocus));
			
		}
	//QWidget::resizeEvent (event);
	QApplication::restoreOverrideCursor ();
}


/***********************/
/* CreateSimpleSudoku  */
/****************************************************************/
/* Slot which create a Sudoku (3x3)  and call PutSudokuInScene  */
/****************************************************************/

SudokuBoard * SudokuView::CreateSimpleSudoku(int order , char diag)
{
#ifdef _DEBUG
	printf("SudokuView::CreateSimpleSudoku 1\n");
#endif

	clearBoard();
	sudoku_board = DM_NEW ( new SudokuBoard (this,
	                                         SudokuBoard::basic,
	                                         diag, order)       );
 	if (!sudoku_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Board"), tr("The board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
	}


	check_board = DM_NEW ( new SudokuBoard (NULL,
	                                        SudokuBoard::basic,
	                                        diag, order)       );
	if (!check_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Check Board"), tr("The check board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
	}
	
	thread_board = DM_NEW ( new SudokuBoard (NULL,
	                                         SudokuBoard::basic,
	                                         diag, order)       );
	if (!thread_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Thread Board"), tr("The thread board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
	}

	/* Put Sudoku in the Scene */
	else PutSudokuInScene();
	return	 sudoku_board;
}

/**********************/
/*   CreateCustom     */
/****************************************************************/
/* Slot which create a Custom Sudoku and call PutSudokuInScene  */
/****************************************************************/

SudokuBoard * SudokuView::CreateCustom(
                             int nGrids,  /* Num. composed grids */
                             int * xg,    /* Coordinates of */
                             int * yg,
                             char wb ,     /* Block width */
                             char hb ,     /* Block height */
                             char diag   /* TRUE == diagonals */
                             )
{
#ifdef _DEBUG
	printf("SudokuView::CreateCustom 1\n");
#endif	
	clearBoard();

	sudoku_board = DM_NEW ( new SudokuBoard (this,
	                                         SudokuBoard::custom,
	                                         diag, wb, hb,
	                                         nGrids, xg, yg) );
	if (!sudoku_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Board"), tr("The board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
	}

	check_board = DM_NEW ( new SudokuBoard (NULL,
	                                        SudokuBoard::custom,
	                                        diag, wb, hb,
	                                        nGrids, xg, yg) );
	if (!check_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Check Board"), tr("The check board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
	}

	thread_board = DM_NEW ( new SudokuBoard (NULL,
	                                         SudokuBoard::custom,
	                                         diag, wb, hb,
	                                         nGrids, xg, yg) );
	if (!thread_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Thread Board"), tr("The thread board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
	}

	else PutSudokuInScene();
	return	 sudoku_board;
}


/********************/
/* CreateSamurai    */
/****************************************************************/
/* Slot which create a Samurai Sudoku and call PutSudokuInScene */
/****************************************************************/


SudokuBoard * SudokuView::CreateSamurai (  char wb ,     /* Block width */
                             char hb ,     /* Block height */
                             char diag )  /* TRUE == diagonals */
{
#ifdef _DEBUG
	printf("SudokuView::CreateSamurai 1\n");
#endif
	clearBoard();
	
	sudoku_board = DM_NEW ( new SudokuBoard (this,
	                                         SudokuBoard::samurai,
	                                         diag, wb, hb)        );
	if (!sudoku_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Board"), tr("The board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
	}


	check_board = DM_NEW ( new SudokuBoard (NULL,
	                                        SudokuBoard::samurai,
	                                        diag, wb, hb)        );
	if (!check_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Check Board"), tr("The check board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
	}

	thread_board = DM_NEW ( new SudokuBoard (NULL,
	                                         SudokuBoard::samurai,
	                                         diag, wb, hb)        );
	if (!thread_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Thread Board"), tr("The thread board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
	}


	else PutSudokuInScene();
	return	 sudoku_board;
}


/********************/
/* PutSudokuInScene */
/***********************************************************/
/* Used to setup and paint the first time the sudoku_board */
/***********************************************************/

void SudokuView::PutSudokuInScene()
{
#ifdef _DEBUG
	printf("SudokuView::PutSudokuInScene 1\n");
#endif
	QPen myPen;
	float width_window=0;
	float height_window=0;
	float w_block=0, h_block=0;
	//float size_block;
	int i=0;

	/* Set flags */
	ThisSudokuHasErrors=false;
	flagSudokuWithErrorsinAllMoment=false;
	SudokuSolutionwasfound=false;
	//RestartUndo();
	QApplication::setOverrideCursor (Qt::WaitCursor);

	if (sudoku_board)
	{
	        t_board_p pBoard = sudoku_board->Board ();
                int w = GetBoardWidth (pBoard);
                int h = GetBoardHeight (pBoard);
                int wb = GetBlockWidth (pBoard);
                int hb = GetBlockHeight (pBoard);
                
		/* Calculate the cell size */
		
		width_window = mScene->width();
		height_window = mScene->height();
		
		
		/* Calculate size */
		//TODO: CHECK VALID SIZE FIRST
	//	setMinimumSize( 2 * (WIDTH_GRID_BORDER * (sudoku_board->w + 1))+ _2WIDTH_W_BORDER , 2 * (WIDTH_GRID_BORDER * (sudoku_board->h + 1)) + _2WIDTH_H_BORDER );

		/*emit MinimunSize (2 * (WIDTH_GRID_BORDER * (sudoku_board->w + 1))+ _2WIDTH_W_BORDER , 2 * (WIDTH_GRID_BORDER * (sudoku_board->h + 1)) + _2WIDTH_H_BORDER );*/

		w_block = ( width_window - (WIDTH_GRID_BORDER * (w + 1))- _2WIDTH_W_BORDER) 
                          / w;
		h_block = ( height_window - (WIDTH_GRID_BORDER * (h+ 1))- _2WIDTH_H_BORDER) 
                          / h;

		/* Calculate Text Order */
		Table_TextCell_Size = 0;
		float WxH= (wb * hb);
		int k,p;

		/* Calculate Proportion  Table */
		for (i=0; i <= int(ceil ( WxH / 2)) ; i++)
		{
			if ( i + 1 > int(ceil ( WxH / (i +1 )) ))
			{
				break;
			}
			else
			{
				Table_TextCell_Width[i] = i + 1;
				Table_TextCell_Height[i] = int( ceil ( WxH / (i +1 )) );
				Table_TextCell_Proportion[i] = float (Table_TextCell_Width[i]) / Table_TextCell_Height[i];
				Table_TextCell_Size++;
			}
		}
		p = i;
		for (k=1 ; k <= p; k++)
		{
				Table_TextCell_Width[i]  = Table_TextCell_Height[p -k];
				Table_TextCell_Height[i] = Table_TextCell_Width[p - k];  
				Table_TextCell_Proportion[i] = float (Table_TextCell_Width[i]) / Table_TextCell_Height[i];
				Table_TextCell_Size++;
				i++;
		}


		/* Printing Proportion Table */
		/*
#ifdef _DEBUG
		printf ( "Index \t Width \t Height \t Proportion\n");
		for (i=0; i < Table_TextCell_Size ; i++)
		{
			printf ( " %d \t %d \t %d \t %f\n",
				 i, Table_TextCell_Width[i], Table_TextCell_Height[i], Table_TextCell_Proportion[i]);
		}	
#endif
		*/


#ifdef _DEBUG
	printf("SudokuView::PutSudokuInScene 2\n");
#endif
		
		FreeSudokuInScene();

		/*
		myPen.setColor(Qt::blue);//TODO;
		myPen.setWidth(10);
		QGraphicsRectItem * a;
		a= mScene->addRect ( QRectF(
					0,
					0,
					width_window ,
					height_window
					) );
		a->setPen(myPen);
		a->setBrush(Qt::NoBrush);
		a->setZValue(50);
		*/


		/* Setup Pen of Cells */
		myPen.setColor(colorCell);
		myPen.setWidth(1);

		/* Painting Cells */
		for (i=0; i < sudoku_board->NumCells() ; i++)
		{
		        int x = GetCellX (pBoard, i);
		        int y = GetCellY (pBoard, i);
		        
			sudoku_board->GCell(i)->setPen(myPen);
			sudoku_board->GCell(i)->setBrush(Qt::NoBrush);
			sudoku_board->GCell(i)->setRect(
				QRectF(
				x * (w_block + WIDTH_GRID_BORDER) +
				WIDTH_W_BORDER,
				y * (h_block + WIDTH_GRID_BORDER) +
				WIDTH_H_BORDER,
					w_block,
					h_block) );
		
		
			mScene->addItem(sudoku_board->GCell(i));

			
		}

		/** Painting the BLOCKs **/

		/* Setup Pen of Blocks */
		myPen.setColor(colorBlock);
		myPen.setWidth(3);
		nmGroups=0;

		ppGroupsRect= ( QGraphicsRectItem **) DM_NEW (malloc ( sudoku_board->NumGroups() * sizeof (QGraphicsRectItem*)) );
		if (!ppGroupsRect )
		{
			QMessageBox A(tr("Memory Error creating Blocks of Board"), tr("The board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
			A.exec();
			return;
		}

		for (i =0; i < sudoku_board->NumGroups(); i++)
		{
			if ( GetGroupType(pBoard,i) == GR_BLOCK )
			{
			        int cell = GetCellOfGroup (pBoard, 0, i);
		                int x = GetCellX (pBoard, cell);
		                int y = GetCellY (pBoard, cell);
		                int wb = GetBlockWidth (pBoard);
		                int hb = GetBlockHeight (pBoard);

				ppGroupsRect[nmGroups]= mScene->addRect (
					QRectF(
					x * (w_block + WIDTH_GRID_BORDER) +
					- HALF_WIDTH_BLOCK_BORDER + WIDTH_W_BORDER,
					y * (h_block + WIDTH_GRID_BORDER) +
					- HALF_WIDTH_BLOCK_BORDER + WIDTH_H_BORDER,
					wb * (w_block + WIDTH_GRID_BORDER),
					hb * (h_block + WIDTH_GRID_BORDER)
					) );
				ppGroupsRect[nmGroups]->setPen(myPen);
				ppGroupsRect[nmGroups]->setBrush(Qt::NoBrush);
				ppGroupsRect[nmGroups]->setZValue(2);
				nmGroups++;
			}	
		}
		ppGroupsRect= ( QGraphicsRectItem **) realloc ( ppGroupsRect, nmGroups * sizeof (QGraphicsRectItem*));
		
		if (!ppGroupsRect )
		{
			QMessageBox A(tr("Memory Error creating Blocks of Board"), tr("The board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
			A.exec();
			return;
		}

		/** Painting the GRIDs **/

#ifdef _DEBUG
	printf("SudokuView::PutSudokuInScene 3\n");
#endif
		/* Setup Pen of Grids */
		myPen.setColor(colorGrid);
		myPen.setWidth(6);
		
		nmGrids=0;

		ppGridsRect= ( QGraphicsRectItem **) DM_NEW (malloc ( sudoku_board->NumGroups() * sizeof (QGraphicsRectItem*)) );
		if (!ppGridsRect )
		{
			QMessageBox A(tr("Memory Error creating Grids of Board"), tr("The board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
			A.exec();
			return;
		}
	
	        int nSy = sudoku_board->NumSymbols ();
	
		for (i =0; i < sudoku_board->NumGroups(); i++)
		{
			if (i == nmGrids*(3* nSy)) //sudoku_board->pGr[i].type == GR_DIAG)
			{
			        int cell = GetCellOfGroup (pBoard, 0, i);
		                int x = GetCellX (pBoard, cell);
		                int y = GetCellY (pBoard, cell);
		                int wb = GetBlockWidth (pBoard);
		                int hb = GetBlockHeight (pBoard);

				ppGridsRect[nmGrids]= mScene->addRect (
					QRectF(
					x * (w_block  + WIDTH_GRID_BORDER) +
					-HALF_WIDTH_GRID_BORDER + WIDTH_W_BORDER,
					y * (h_block  + WIDTH_GRID_BORDER) +
					-HALF_WIDTH_GRID_BORDER + WIDTH_H_BORDER,
					
					wb * hb * (w_block + WIDTH_GRID_BORDER),
					hb * wb * (h_block +WIDTH_GRID_BORDER))) ;
				ppGridsRect[nmGrids]->setPen(myPen);
				ppGridsRect[nmGrids]->setBrush(Qt::NoBrush);
				ppGridsRect[nmGrids]->setZValue(4);

				/* Added in version 0.10 */
			  	GridsX[nmGrids]=x;
	  		   	GridsY[nmGrids]=y;
				/*************************/
				nmGrids++; 

			}
		}
		ppGridsRect= ( QGraphicsRectItem **) realloc ( ppGridsRect, nmGrids * sizeof (QGraphicsRectItem*));
		
		if (!ppGridsRect )
		{
			QMessageBox A(tr("Memory Error creating Grids of Board"), tr("The board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
			A.exec();
			return;
		}

		/** Setup Text Cells  added in Project 7 **/
		SetupTextCell();	

#ifdef _DEBUG
	printf("SudokuView::PutSudokuInScene 4\n");
#endif

	mScene->setSceneRect( QRectF(0,0, width_window  , height_window));
	mScene->update(QRectF(0,0,mScene->width(),mScene->height()) );

		/** EnlargeWindow  added in Project 24, this function 30 **/

		/** EnlargeWindow  added in Project 24, changed 30**/
		if (mEnlargeWindow == NULL)
			CreateEnlargeWindow();
		
		mEnlargeWindow->ChangeSudoku();
		/**                                    **/
		if (flagEnlarge)
			ShowEnlargeWindow();
		else    HideEnlargeWindow();

		hide();
		show();
		
		
	}// end if(board)
	QApplication::restoreOverrideCursor ();

#ifdef _DEBUG
	printf ("SudokuView::PutSudokuInScene END\n");
#endif
}


/*******************/
/* ResizeCellFont  */
/********************************************************************/
/* Used to resize font cells and put it in the best visual position */
/********************************************************************/

void SudokuView::ResizeCellFont()
{
#ifdef _DEBUG
	printf("SudokuView::ResizeCellFont 1\n");
#endif

	int i=0, j=0;// k=0;

	//float w_box = GetBlockWidth (sudoku_board->Board());
	//float h_box = GetBlockHeight (sudoku_board->Board());
	//float size_w = (int) sudoku_board->GCell(0)->rect().width() /w_box;
	//float size_h = (int) sudoku_board->GCell(0)->rect().height()/h_box;
	float h_block= sudoku_board->GCell(0)->rect().height();
	float w_block= sudoku_board->GCell(0)->rect().width();
	//float size_w = w_block /w_box;
	//float size_h = h_block /h_box;
	
	
	int n_lines=0;
	int n_char=0;


	//QString * mstring =NULL;

	int littlesize=0, bigsize=0;
	
	
	/* Calculate Text Width and Character Width */
	/* Using Proportion Table */

	
	float Proportion = float (w_block) / h_block;
	float MIN=0,min=0;
	int select=0;
	MIN = fabs( Proportion - Table_TextCell_Proportion[0] ); 

	/* Looking for the minimun distance table Proportion */
	for (i=1; i < Table_TextCell_Size; i++)
	{
		min = fabs( Proportion - Table_TextCell_Proportion[i] );
		if ( min < MIN)
		{
			MIN = min;
			select=i;	
		}
	}
	/* Set n_lines and n_char to optimal values*/
	n_char  = Table_TextCell_Width[select];
    	n_lines = Table_TextCell_Height[select];


/* Calculate Text Size */ //TODO: Optimize

	littlesize = (uint) h_block / n_lines;
	bigsize =  (uint) h_block;

	if (  littlesize < 1  ) littlesize = 1;
	if (  bigsize < 1  ) bigsize = 1;
	
	mFont.setPixelSize( littlesize );
	mBigFont.setPixelSize( bigsize  );



	/* Special Case when we must change number of lines off cell */
	if (n_lines > sudoku_board->GCell(0)->nTextPoss )
	{
		for (i=0 ;  i < sudoku_board->NumCells() ; i++)
		{
				/* Expand pTextPoss */
				QGraphicsSimpleTextItem ** paux = DM_NEW ( new QGraphicsSimpleTextItem*[n_lines] );
				if (!paux )
				{
					QMessageBox A(tr("Memory Error Doing Resize Operation"), tr("You will have problems playing the game \n Please restart\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
					A.exec();
					return;
				}
				/* Copy array of pointers */
				for ( j=0; j < sudoku_board->GCell(i)->nTextPoss; j++)
				{
					paux[j]=sudoku_board->GCell(i)->pTextPoss[j];
				}
				/* Adding more lines */
				for (; j < n_lines; j++)
				{
					paux[j] = DM_NEW ( new QGraphicsSimpleTextItem );
					if (!paux[j] )
					{
						QMessageBox A(tr("Memory Error Doing Resize Operation"), tr("You will have problems playing the game \n Please restart\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
						A.exec();
						return;
					}
					paux[j]->setParentItem(sudoku_board->GCell(i));
					//sudoku_board->GCell(i)->pTextPoss.setPlainText(mstring);
					paux[j]->setZValue (7);
					paux[j]->setPen(Qt::NoPen);
					paux[j]->setBrush(colorSmallText);

					if ( qVersion()[0]<='4' && qVersion()[1]=='.'&& qVersion()[2]< '3')
						mScene->addItem(  paux[j] );
			
					if ( sudoku_board->GCell(i)->FvActivate || sudoku_board->GCell(i)->canChange > 0)
					{
						paux[j]->hide();
						//sudoku_board->GCell(i)->pTextPoss[h]->show();
					}
				}
				delete DM_DEL (sudoku_board->GCell(i)->pTextPoss );
				sudoku_board->GCell(i)->pTextPoss = paux;
				sudoku_board->GCell(i)->nTextPoss = n_lines;
			
		}
	}
	else if (n_lines < sudoku_board->GCell(0)->nTextPoss )
	{/* Special Case when we must change number of lines off cell */
		for (i=0 ;  i < sudoku_board->NumCells() ; i++)
		{
			for (j = n_lines; j < sudoku_board->GCell(i)->nTextPoss ; j++)
			{
				sudoku_board->GCell(i)->pTextPoss[j]->hide();
				mScene->removeItem(sudoku_board->GCell(i)->pTextPoss[j] );
				delete DM_DEL (sudoku_board->GCell(i)->pTextPoss[j] );
			}
			sudoku_board->GCell(i)->nTextPoss =  n_lines;
		}
	}

	int var_ajust=0;
	if  ( int (w_block / 2) - int (bigsize/2)  >  0)
	{
		var_ajust = int (w_block / 2) - int (bigsize/2);
	}
	for (i=0 ;  i < sudoku_board->NumCells() ; i++)
	{
		
		sudoku_board->GCell(i)->TextFinalValue.setFont( mBigFont );
		
		int x = GetCellX (sudoku_board->Board(), i);
		int y = GetCellY (sudoku_board->Board(), i);
		
		sudoku_board->GCell(i)->TextFinalValue.setPos (
					(x  * w_block ) +
					(x) * WIDTH_GRID_BORDER +
					WIDTH_W_BORDER + var_ajust,
					(y  * h_block ) +
					(y ) * WIDTH_GRID_BORDER +
					WIDTH_H_BORDER ) ;
		
		/* */
		for ( int h=0; h < n_lines ; h++ )
		{
			sudoku_board->GCell(i)->pTextPoss[h]->setFont(mFont);
		
		//	sudoku_board->GCell(i)->pTextPoss[h]->setText(mstring[h]);
	//		sudoku_board->GCell(i)->pTextPoss[h]->setTextWidth(w_block);

			sudoku_board->GCell(i)->pTextPoss[h]->setPos (
					(x  * w_block ) +
					(x) * WIDTH_GRID_BORDER +
					WIDTH_W_BORDER,
					(y  * h_block ) +
					(y ) * WIDTH_GRID_BORDER +
					WIDTH_H_BORDER + h * littlesize  ) ; //TODO Calibrate


			/*if ( sudoku_board->GCell(i)->FvActivate == FALSE)
			{
				sudoku_board->GCell(i)->pTextPoss[h]->hide();
				sudoku_board->GCell(i)->pTextPoss[h]->show();
			}*/
		}
		sudoku_board->GCell(i)->UpdatePossUserText( littlesize, n_lines);
	}
#ifdef _DEBUG
	printf ("SudokuView::ResizeCellFont END\n");
#endif
}

/*******************/
/* SetupTextCell   */
/********************************************************************/
/* Used to setup font and text cells. Called by PutSudokuInScene    */
/********************************************************************/

void SudokuView::SetupTextCell()
{
#ifdef _DEBUG
	printf("SudokuView::SetupTextCell 1\n");
#endif
	//TODO: Delete all unnecesary actions. Remember we call ResizeCellFont at the end.


	//float w_box = GetBlockWidth (sudoku_board->Board());
	//float h_box = GetBlockHeight (sudoku_board->Board());
	float h_block= sudoku_board->GCell(0)->rect().height();
	float w_block= sudoku_board->GCell(0)->rect().width();
	//float size_w = w_block /w_box;
	//float size_h = h_block /h_box;
	int i=0, n_char=0, n_lines=0; //k=0;
	//if ( sudoku_board->GCell(0)->rect().width() < sudoku_board->GCell(0)->rect().height() ) 
		
	//else 
	//	size_block= sudoku_board->GCell(0)->rect().width();
	//QString * mstring=NULL;


	int littlesize=0, bigsize=0;
	


	
	/* Calculate Text Width and Character Width */
	/* Using Proportion Table */

	
	float Proportion = float (w_block) / h_block;
	float MIN=0,min=0;
	int select=0;
	MIN = fabs( Proportion - Table_TextCell_Proportion[0] ); 

	/* Looking for the minimun distance table Proportion */
	for (i=1; i < Table_TextCell_Size; i++)
	{
		min = fabs( Proportion - Table_TextCell_Proportion[i] );
		if ( min < MIN)
		{
			MIN = min;
			select=i;	
		}
	}
	/* Set n_lines and n_char to optimal values*/
	n_char  = Table_TextCell_Width[select];
    	n_lines = Table_TextCell_Height[select];


	/* Calculate Text Size */ //TODO: Optimize

	littlesize = (int) h_block / n_lines;
	bigsize =  (int) h_block;

	if (  littlesize < 1  ) littlesize = 1;
	if (  bigsize < 1  ) bigsize = 1;
	
	mFont.setPixelSize( littlesize );
	mBigFont.setPixelSize( bigsize  );

	
	/********************************************/
	/* Asign memory to pTextPoss                */	
	//mstring = DM_NEW (new QString[n_lines] );
	int h=0;
	/*
	for ( h=0; h < n_lines ; h++ )
	{
		
		for (int w=0 ; w < n_char ; w++)
		{
			if ( k < sudoku_board->NumSymbols())
			{
			 mstring[h].append(simbols(k));
			}
			 k++;
		}
	}*/
	/*
	int var_ajust=0;
	if  ( int (w_block / 2) - int (bigsize/2)  >  0)
	{
		var_ajust = int (w_block / 2) - int (bigsize/2);
	}*/
	for (i=0 ;  i < sudoku_board->NumCells() ; i++)
	{
		//TODO:
		sudoku_board->GCell(i)->pTextPoss = DM_NEW ( new QGraphicsSimpleTextItem*[n_lines] );
		if (!sudoku_board->GCell(i)->pTextPoss )
		{
			QMessageBox A(tr("Memory Error creating Text of Cells"), tr("The board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
			A.exec();
			return;
		}
		
		
		sudoku_board->GCell(i)->canChange = true;
		sudoku_board->GCell(i)->FvActivate = false;
		
		sudoku_board->GCell(i)->TextFinalValue.setZValue (10);
		sudoku_board->GCell(i)->TextFinalValue.setParentItem(sudoku_board->GCell(i));
		//sudoku_board->GCell(i)->TextFinalValue.setFont( mBigFont );
		
		//sudoku_board->GCell(i)->TextFinalValue.setPos ( sudoku_board->GCell(i)->scenePos() );
		sudoku_board->GCell(i)->TextFinalValue.setPen(Qt::NoPen);
		sudoku_board->GCell(i)->TextFinalValue.setBrush(colorBigText);
/*
		int x = GetCellX (sudoku_board->Board(), i);
		int y = GetCellY (sudoku_board->Board(), i);

		sudoku_board->GCell(i)->TextFinalValue.setPos (
					(x  * w_block ) +
					(x) * WIDTH_GRID_BORDER +
					WIDTH_W_BORDER + var_ajust,
					(y  * h_block ) +
					(y ) * WIDTH_GRID_BORDER +
					WIDTH_H_BORDER ) ;*/

		/* This line is not necesary in QT 4.3.0 */
		if ( qVersion()[0]<='4' && qVersion()[1]=='.'&& qVersion()[2]< '3')
			mScene->addItem( &(sudoku_board->GCell(i)->TextFinalValue) );


		/** **/
		sudoku_board->GCell(i)->nTextPoss = n_lines;
		for  ( h=0; h < n_lines ; h++ )
		{
			sudoku_board->GCell(i)->pTextPoss[h] = DM_NEW ( new QGraphicsSimpleTextItem );
			if (!sudoku_board->GCell(i)->pTextPoss[h] )
			{
				QMessageBox A(tr("Memory Error creating Text of Cells"), tr("The board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
				A.exec();
				return;
			}

			sudoku_board->GCell(i)->pTextPoss[h]->setParentItem(sudoku_board->GCell(i));
			//sudoku_board->GCell(i)->pTextPoss.setPlainText(mstring);
			sudoku_board->GCell(i)->pTextPoss[h]->setZValue (7);
			//sudoku_board->GCell(i)->pTextPoss[h]->setFont(mFont);
			
			//sudoku_board->GCell(i)->pTextPoss.setPos ( sudoku_board->GCell(i)->scenePos() );
			/*sudoku_board->GCell(i)->pTextPoss[h]->setPos (
					(x  * w_block ) +
					(x) * WIDTH_GRID_BORDER +
					WIDTH_W_BORDER,
					(y  * h_block ) +
					(y ) * WIDTH_GRID_BORDER +
					WIDTH_H_BORDER + h * littlesize  ) ; // TODO Calibrate
			*/
			sudoku_board->GCell(i)->pTextPoss[h]->setPen(Qt::NoPen);
			sudoku_board->GCell(i)->pTextPoss[h]->setBrush(colorSmallText);

			// Change in proyect27
			//sudoku_board->GCell(i)->pTextPoss[h]->setText(mstring[h]);

			/* This line is not necesary in QT 4.3.0 */
			if ( qVersion()[0]<='4' && qVersion()[1]=='.'&& qVersion()[2]< '3')
				mScene->addItem(  (sudoku_board->GCell(i)->pTextPoss[h] ) );
			//sudoku_board->GCell(i)->pTextPoss[h]->show();
		}
		


		/* TODO ¿ set Pos ?*/
		/* TODO : Move this code to other file */
	
		if ( ! (sudoku_board->GCell(i)->mTextPoss) && !( sudoku_board->GCell(i)->FvActivate) )
		{
			/* Setup of mTextPoss */
			/*
			for (int j=0; j < sudoku_board->NumSymbols(); j++ )
			{
				sudoku_board->GCell(i)->mTextPoss = sudoku_board->GCell(i)->mTextPoss << 1;
				sudoku_board->GCell(i)->mTextPoss ++;
			}*/
			
			sudoku_board->GCell(i)->mTextPoss =
			    GetPossValuesOfCell (sudoku_board->Board(), i);
			
		}
		/**************************/
	}

	//delete [] DM_DEL (mstring );

	ResizeCellFont();

}

/**************/
/* clearBoard */
/********************************************************************/
/*  DestroyBoard cells and free sudoku_board                        */
/* TODO: Need more tests.                                           */
/********************************************************************/
void SudokuView::clearBoard()
{
	#ifdef _DEBUG
		printf("SudokuView::clearBoard 1\n");
	#endif
	ClearListLastColorModifed();

	delete [] ( DM_DEL (pMaskCellsArray) );
	pMaskCellsArray = NULL;
	if (sudoku_board ) 
	{
		mRulesDialog->ClearRules();
		sudoku_board->Clear ();
		for (int i=0; i < sudoku_board->NumCells(); i++)
			mScene->removeItem(sudoku_board->GCell(i));
		delete DM_DEL (sudoku_board); 
		sudoku_board=NULL; 
	}
	if (check_board )
	{
		check_board->Clear ();
		delete DM_DEL (check_board); 
		check_board=NULL; 
	}
	if (thread_board )
	{
		thread_board->Clear ();
		delete DM_DEL (thread_board); 
		thread_board=NULL; 
	}

	ThisSudokuHasErrors=false;
	flagSudokuWithErrorsinAllMoment=false;
}


/*****************/
/* cleamAllCells */
/********************************************************************/
/*  Empty all cells. Finally you have an Empty Board                */
/********************************************************************/
void SudokuView::cleanAllCells()
{
#ifdef _DEBUG
	printf("SudokuView::cleanAllCells 1\n");
#endif
	if (sudoku_board)
	{
		mRulesDialog->ClearRules();
		sudoku_board->Clear ();
		for (int i=0; i< sudoku_board->NumCells() ; i++)
			sudoku_board->GCell(i)->UpdateCell();

		ResizeCellFont();
	}
	ThisSudokuHasErrors=false;
	flagSudokuWithErrorsinAllMoment=false;
}

/*********************/
/* RepaintBackGround */
/********************************************************************/
/*  Repaint the background with a different color (mcolor)          */
/********************************************************************/
void SudokuView::RepaintBackGround( QColor mcolor)
{
#ifdef _DEBUG
	printf("SudokuView::RepaintBackGround 1\n");
#endif
	mScene->setBackgroundBrush( mcolor );
	hide();
	show();
}


/*********************/
/* RepaintGrid       */
/********************************************************/
/*  Repaint the grids  with the color (colorGrid).      */
/* It usually uses when colorGrid has just changed.     */
/********************************************************/
void SudokuView::RepaintGrid ()
{
#ifdef _DEBUG
	printf("SudokuView::RepaintGrid 1\n");
#endif
	if (sudoku_board)
	{
		QPen myPen;
		myPen.setColor(colorGrid);
		myPen.setWidth(6);

		for (int i =0; i < nmGrids; i++)
		{
			ppGridsRect[i]->setPen(myPen);
		}
	}
}


/*********************/
/* RepaintBlock      */
/********************************************************/
/*  Repaint the blocks  with the color (colorBlock).    */
/* It usually uses when colorBlock has just changed.    */
/********************************************************/
void SudokuView::RepaintBlock ()
{
#ifdef _DEBUG
	printf("SudokuView::RepaintBlock 1\n");
#endif
	if (sudoku_board)
	{
		QPen myPen;
		myPen.setColor(colorBlock);
		myPen.setWidth(3);
		

		for (int i =0; i < nmGroups ; i++)
		{
			ppGroupsRect[i]->setPen(myPen);
		}
	}
}

/********************/
/* RepaintCell      */
/********************************************************/
/*  Repaint the cells  with the color (colorCell).      */
/* It usually uses when colorCell has just changed.     */
/********************************************************/
void SudokuView::RepaintCell ()
{
#ifdef _DEBUG
	printf("SudokuView::RepaintCell 1\n");
#endif
	if (sudoku_board)
	{
		QPen myPen;
		myPen.setColor(colorCell);
		myPen.setWidth(1);
		for (int i=0; i < sudoku_board->NumCells() ; i++)
		{
			sudoku_board->GCell(i)->setPen(myPen);
		}
	}
}

/********************/
/* RepaintBigText   */
/*************************************************************************/
/*  Repaint the Final Value Text of cells  with the color (colorBigText).*/
/* It usually uses when colorBigText has just changed.                   */
/*************************************************************************/
void SudokuView::RepaintBigText ()
{
#ifdef _DEBUG
	printf("SudokuView::RepaintBigText 1\n");
#endif
	if (sudoku_board)
	{

		
		for ( int i=0 ;  i < sudoku_board->NumCells() ; i++)
		{
				sudoku_board->GCell(i)->TextFinalValue.setPen(Qt::NoPen);
				sudoku_board->GCell(i)->TextFinalValue.setBrush(colorBigText);
				//sudoku_board->GCell(i)->TextFinalValue.hide();
				//sudoku_board->GCell(i)->TextFinalValue.show();
		}
	}
}

/**********************/
/* RepaintSmallText   */
/******************************************************************************/
/*  Repaint the Possible Value Text of cells  with the color (colorSmallText).*/
/* It usually uses when colorSmallText has just changed.                      */
/******************************************************************************/
void SudokuView::RepaintSmallText ()
{
#ifdef _DEBUG
	printf("SudokuView::RepaintSmallText 1\n");
#endif
	if (sudoku_board)
	{

		
		for ( int i=0 ;  i < sudoku_board->NumCells() ; i++)
		{
			for  ( int h=0; h < sudoku_board->GCell(i)->nTextPoss ; h++ )
			{
				sudoku_board->GCell(i)->pTextPoss[h]->setPen(Qt::NoPen);
				sudoku_board->GCell(i)->pTextPoss[h]->setBrush(colorSmallText);
				sudoku_board->GCell(i)->UpdatePossUserText(sudoku_board->GCell(i)->pTextPoss[h]->font().pixelSize(),sudoku_board->GCell(i)->nTextPoss);
			}
			
		}
	}
}

/*******************/
/* RepaintCursor   */
/*******************/
void SudokuView::RepaintCursor ()
{
}

/****************/
/* writeFile    */
/************************************************************/
/*  Save the sudoku board in a file named fileName.         */
/*                                                          */
/*  NOTE: Be careful. writeFile and readFile are symetrics  */
/*  All changes in this function must be made in readFile    */
/************************************************************/
bool SudokuView::writeFile (const QString &fileName)
{
#ifdef _DEBUG
	printf("SudokuView::writeFile 1\n");
#endif
	int i=0;
	t_board_p pBoard = sudoku_board->Board ();
	int wb = GetBlockWidth (pBoard);
	int hb = GetBlockHeight (pBoard);

	QFile file (fileName);

	if (!file.open(QIODevice::WriteOnly))
	{
		QMessageBox::warning(this, tr("Sudoku View"), tr("Cannot write file %1:\n%2.").arg(file.fileName()).arg(file.errorString()));
		file.close();
		return false;
	}

	QDataStream out(&file);
                                             // INITIAL SETTINGS:
	out.setVersion (QDataStream::Qt_4_2);      // Serializer version for preamble
        out.setByteOrder (QDataStream::BigEndian); // Default setting, though...

#if SDK_FORMAT_VERSION > 0

             // At this point, nothing has been written. The lines above
             // should be preserved across versions FOREVER. They establish
             // a fixed format for the preamble.

                                             // PREAMBLE:
	out << quint32 (QDataStream::Qt_4_2);      // Serializer version for data
	out << quint32 (MAGICNUMBER);              // Magic number
	out << quint32 (SDK_FORMAT_VERSION); // Format version
	out << quint32 (MAGICNUMBER);              // Magic number again (for alignment)

             // The reader shold check the magic number and the version
             // specifications at this point, after reading the preamble.
             
#endif // SDK_FORMAT_VERSION > 0
                                             // DATA:

#if SDK_FORMAT_VERSION == 0x200

	QApplication::setOverrideCursor (Qt::WaitCursor);

	out << quint8 (wb);
	out << quint8 (hb);

	out << quint16 (nmGrids);

	for (i=0; i < nmGrids; i++)
	{
		out << quint16 (GridsX[i]);
		out << quint16 (GridsY[i]);
	}

        int bytes = GetNumBytesAllPVMasks (pBoard);
        char * buff = DM_NEW ( new char[bytes] );
        
        GetAllPVMasks (pBoard, buff);
        
        for (i=0; i<bytes; i++)
          out << quint8 (buff[i]);
          
        delete [] DM_DEL (buff);

	file.close();
	QApplication::restoreOverrideCursor();
	return true;

#endif // SDK_FORMAT_VERSION == 0x200

#if SDK_FORMAT_VERSION <= 0x100

	int w = GetBoardWidth (pBoard);
	int h = GetBoardHeight (pBoard);

	QApplication::setOverrideCursor (Qt::WaitCursor);

	out << quint16 (nmGrids);
	
	for (i=0; i < nmGrids; i++)
	{
		out << quint32 (i);
		out << quint32 (GridsX[i]);
		out << quint32 (GridsY[i]);
	}

	out << quint8 (wb);
	out << quint8 (hb);

#if SDK_FORMAT_VERSION == 0

	out << quint32 (MAGICNUMBER);          // Magic number
	out << quint32 (QDataStream::Qt_4_2);  // Serializer version for data

#endif // SDK_FORMAT_VERSION == 0

	// Variables of t_board sudoku_board.
	out << quint16 (sudoku_board->NumCells());
	out << quint16 (sudoku_board->NumGroups());
	out << quint8  (sudoku_board->NumSymbols());
	out << quint8  (0);       // pBoard->maxIC; // trivial
	out << quint16 (w);
	out << quint16 (h);
	out << quint8  (wb);
	out << quint8  (hb);
	out << quint8  (GetState          (pBoard));
	out << quint16 (GetNumSolvedCells (pBoard));
	out << quint16 (GetErrSymbolMask  (pBoard));  // Should be 32
	out << quint16 (GetErrCellMask    (pBoard));  // Should be 32
	out << quint16 (GetErrGroup       (pBoard));
	
	// Save t_list
	//TODO: Or not to do.
	//Right the other way around: remove unnecesary data

	out << quint8 (0); // sudoku_board->maxLevel); // trivial
	out << quint32 (GetNumGuessedCells          (pBoard));
	out << quint32 (GetTotalGuessings           (pBoard));
	out << quint8  (GetLastRuleLevel            (pBoard));
	out << quint8  (GetLastRuleType             (pBoard));
	out << quint16 (GetLastRuleGroup            (pBoard));
	out << quint16 (GetLastRuleIntersectedGroup (pBoard));
	out << quint32 (GetLastRuleSymbolsMask      (pBoard));
	out << quint32 (GetLastRuleCellsMask        (pBoard));
	out << quint16 (GetNumRulesN1               (pBoard));
	out << quint16 (GetNumRulesN2               (pBoard));
	out << quint16 (GetNumRulesNGt2             (pBoard));

	// Variables of array of cells
	for (i = 0; i < sudoku_board->NumCells(); i ++ )
	{
		out << quint32 (i);
		out << quint8  (GetCellX(pBoard,i));   // Should be 16
		out << quint8  (GetCellY(pBoard,i));   // Should be 16
		out << quint8  (GetNumGroupsOfCell(pBoard,i));
		out << quint32 (GetPossValuesOfCell(pBoard,i));
		out << quint8  (GetNumPossValuesOfCell(pBoard,i));
		out << quint8  (GetSymbolOfCell(pBoard,i));
		out << quint16 (GetSolPositionOfCell(pBoard,i));
		out << quint16 (GetGuessPositionOfCell(pBoard,i));
		out << quint8  (0); // (sudoku_board->pCl[i].flags); // irrelevant
		out << quint32 (GetLastRemovedPossValuesOfCell(pBoard,i));

		// Variables of Graphcells

		out << quint16 (sudoku_board->GCell(i)->nTextPoss);
		out << quint32 (sudoku_board->GCell(i)->mTextPoss);
		out << quint8  (sudoku_board->GCell(i)->canChange?1:0);
		out << quint8  (sudoku_board->GCell(i)->FvActivate?1:0);
	}
	
	file.close();
	QApplication::restoreOverrideCursor();
	return true;

#endif // SDK_FORMAT_VERSION <= 0x100
}

/***************/
/* readFile    */
/************************************************************/
/*  Loade the sudoku board saved in a file named fileName.  */
/*                                                          */
/*  NOTE: Be careful. writeFile and readFile are symetrics  */
/*  All changes in this function must be made in writeFile   */
/************************************************************/
bool SudokuView::readFile (const QString &fileName)
{
#ifdef _DEBUG
	printf("SudokuView::readFile 1\n");
#endif
	int i=0;
	short aux_nmGrids=0;
	int aux_GridsX[15];
	int aux_GridsY[15];
	//int block_x,block_y;

	for (i=0; i < 15 ; i++)
	{
		aux_GridsX[i]=0;
		aux_GridsY[i]=0;
	}
	i=0;

	QFile file1 (fileName);  // Yes: same fileName
	QFile file2 (fileName);  // file2 is our second chance

	if (!file1.open(QIODevice::ReadOnly))
	{
		QMessageBox::warning(this, tr("Sudoku View"), tr("Cannot read file %1:\n%2.").arg(file1.fileName()).arg(file1.errorString()));
		file1.close();
		return false;
	}

        quint8 aux8;
        quint16 aux16;
        quint32 aux32;
        int wb, hb;

	QDataStream in1(&file1);
	QDataStream in2;         // No IO device (by now)
	
	in1.setVersion (QDataStream::Qt_4_2);      // Serializer version for preamble
        in1.setByteOrder (QDataStream::BigEndian); // Default setting, though...

             // At this point, nothing has been read. The lines above
             // should be preserved across versions FOREVER. They establish
             // a fixed format for the preamble.
             
        quint32 serVer, magic, magic2, formatVer;
        
                                             // PREAMBLE:
	in1 >> serVer;                            // Serializer version for data
	in1 >> magic;                             // Magic number
	in1 >> formatVer;                         // Format version
	in1 >> magic2;                            // Magic number again (for alignment)

        if (in1.status()!=QDataStream::Ok)
	{
		QMessageBox::warning(this, tr("Sudoku View"), tr(" MAGICNUMBER data corrupt"));
		file1.close();
		return false;
	}

             // Check the magic number and the version specifications at
             // this point, after reading the preamble.

        if (magic==MAGICNUMBER && magic2==MAGICNUMBER)
        {
                if (formatVer<0x100 || formatVer>0x2FF)
                {
	                QMessageBox::warning(this, tr("Sudoku View"), tr(" MAGICNUMBER data corrupt"));
		        file1.close();
		        return false;
                }
        }
        else
        {
                formatVer = 0;      // No preamble ==> Initial format
		file1.close();
		if (!file2.open(QIODevice::ReadOnly))
        	{
	        	QMessageBox::warning(this, tr("Sudoku View"), tr("Cannot read file %1:\n%2.").arg(file2.fileName()).arg(file2.errorString()));
		        file2.close();
		        return false;
	        }
	        in2.setDevice (&file2);
        }

        QFile & file = formatVer==0 ? file2 : file1;
        QDataStream & in = formatVer==0 ? in2 : in1;

        if (formatVer>=0x100)
	        in.setVersion (serVer);      // Serializer version for data

        if (formatVer>=0x200 && formatVer<0x300)
        {
                QApplication::setOverrideCursor (Qt::WaitCursor);
                
	        in >> aux8;  wb = aux8;
	        in >> aux8;  hb = aux8;

	        in >> aux16; aux_nmGrids = aux16;
	
	        if (aux_nmGrids > 15 || aux_nmGrids < 1 ||
	            wb < 2 || hb < 2 || wb*hb > 30 ||
	            in.status()!=QDataStream::Ok)
	        {
		        QMessageBox::warning(this, tr("Sudoku View"), tr("Error nmGrids data corrupt"));
		        file.close();
	                QApplication::restoreOverrideCursor();
		        return false;
	        }
	
	        for (i=0; i <aux_nmGrids; i++)
	        {
		        in >> aux16;  aux_GridsX[i] = aux16;
		        in >> aux16;  aux_GridsY[i] = aux16;
		        
	                if (aux_GridsX[i] % wb != 0 ||
	                    aux_GridsY[i] % hb != 0 ||
		            aux_GridsX[i]<0 || aux_GridsX[i]+wb*hb>127 ||
		            aux_GridsY[i]<0 || aux_GridsY[i]+wb*hb>127)
		                break;

                        /* Conversion units cell -> units block */
		        aux_GridsX[i] /= wb;
		        aux_GridsY[i] /= hb;
	        }

	        if ( i<aux_nmGrids || in.status()!=QDataStream::Ok)
	        {
		        QMessageBox::warning(this, tr("Sudoku View"), tr("Error GridsX or GridsY data corrupt"));
		        file.close();
	                QApplication::restoreOverrideCursor();
		        return false;
	        }	

        	// Create sudoku_board

	        clearBoard();
	        sudoku_board = CreateCustom (aux_nmGrids,
	                                     aux_GridsX, aux_GridsY,
	                                     wb, hb, 0);
	                             
	        t_board_p pBoard = sudoku_board->Board ();

                int bytes = GetNumBytesAllPVMasks (pBoard);
                char * buff = DM_NEW ( new char[bytes] );
                e_state state;

                for (i=0; i<bytes; i++)
                {
                        in >> aux8;
                        buff[i] = aux8;
                }
        
                state = SetAllPVMasks (pBoard, buff);
        
                delete [] DM_DEL (buff);

                int mPV, symbol, cell;
                int nCl = sudoku_board->NumCells ();
                
                for (cell=0; cell<nCl; cell++)
                {
                	mPV = GetPossValuesOfCell (pBoard, cell);
                	
                	if (mPV==0)
                	        break;
                	
		        sudoku_board->GCell(cell)->mTextPoss = mPV;
                	symbol = GetSymbolOfCell (pBoard, cell);

		        if (symbol==-1)
			        sudoku_board->GCell(cell)->FvActivate = false;
		        else
		        {
			        sudoku_board->GCell(cell)->setFinalUserValue (symbol);
			        sudoku_board->GCell(cell)->canChange = false;
		        }
                }

                if (cell<nCl ||
                    in.status()!=QDataStream::Ok ||
                    state==impossible)
                {
			QMessageBox::warning(this, tr("Sudoku View"), tr("Error Cells data corrupt"));
			QApplication::restoreOverrideCursor();
			file.close();
			return false;
                }

                file.close ();
	        ResizeCellFont();
	        //?? OrderCheckBoard();
	        QApplication::restoreOverrideCursor();
#ifdef _DEBUG
	        printf ("LOAD completado\n");
#endif
                return true;
        }

#ifdef _DEBUG
	printf ("Empezando el LOAD\n");
	printf ("Sizeof Int %ld \n Sizeof Char %ld\n Sizeof short %ld\n", sizeof (int), sizeof (char), sizeof (short));
#endif
	// LOADING VARIABLES

	//Variables of SudokuView

	QApplication::setOverrideCursor (Qt::WaitCursor);

	in >> aux16; aux_nmGrids = aux16;
	
	if (aux_nmGrids > 15 || aux_nmGrids < 1 ||
	    in.status()!=QDataStream::Ok)
	{
		QMessageBox::warning(this, tr("Sudoku View"), tr("Error nmGrids data corrupt"));
		file.close();
	        QApplication::restoreOverrideCursor();
		return false;
	}
	
	for (i=0; i <aux_nmGrids; i++)
	{
		in >> aux32;
		if ( int (aux32) != i )
		  break;
		in >> aux32;  aux_GridsX[i] = aux32;
		in >> aux32;  aux_GridsY[i] = aux32;
	}
	
	if ( i<aux_nmGrids || in.status()!=QDataStream::Ok)
	{
		QMessageBox::warning(this, tr("Sudoku View"), tr("Error GridsX or GridsY data corrupt"));
		file.close();
	        QApplication::restoreOverrideCursor();
		return false;
	}	

	in >> aux8;  wb = aux8;
	in >> aux8;  hb = aux8;

	/* Conversion units cell -> units block */

        if (wb>1 && hb>1 && in.status()==QDataStream::Ok)
	        for (i=0; i <aux_nmGrids; i++)
	        {
	                if (aux_GridsX[i] % wb != 0 ||
	                    aux_GridsY[i] % hb != 0)
        	                break;
	                
		        aux_GridsX[i] = aux_GridsX[i] / wb;
		        aux_GridsY[i] = aux_GridsY[i] / hb;
	        }

	if ( wb<2 || hb<2 ||
	     i<aux_nmGrids ||
	     in.status()!=QDataStream::Ok)
	{
		QMessageBox::warning(this, tr("Sudoku View"), tr("Error GridsX or GridsY data corrupt"));
		file.close();
	        QApplication::restoreOverrideCursor();
		return false;
	}	
	
        if (formatVer==0)
        {
	        in >> magic;          // Magic number
	        in >> serVer;         // Serializer version for data

                if (in.status()!=QDataStream::Ok ||
                    magic!=MAGICNUMBER)
	        {
		        QMessageBox::warning(this, tr("Sudoku View"), tr(" MAGICNUMBER data corrupt"));
		        file.close();
	                QApplication::restoreOverrideCursor();
		        return false;
	        }
        }

	// Create sudoku_board

	clearBoard();
	sudoku_board = CreateCustom (aux_nmGrids,
	                             aux_GridsX, aux_GridsY,
	                             wb, hb, 0);
	                             
	t_board_p pBoard = sudoku_board->Board ();

		      // IGNORE...
	in >> aux16;  //   numCells
	in >> aux16;  //   numGroups
	in >> aux8;   //   numSymbols
	in >> aux8;   //   maxIntersectedCells
	in >> aux16;  //   total width
	in >> aux16;  //   total height
	in >> aux8;   //   block width (again)
	in >> aux8;   //   block height (again)
	in >> aux8;   //   state of solution
	in >> aux16;  //   NumSolvedCells
	in >> aux16;  //   ErrSymbolMask // Should be 32
	in >> aux16;  //   ErrCellMask   // Should be 32
	in >> aux16;  //   ErrGroup      

	// Load t_list
	//TODO: Or not to do.
	//Right the other way around: remove unnecesary data

		      // IGNORE...
	in >> aux8;   //   maxLevel
	in >> aux32;  //   NumGuessedCells
	in >> aux32;  //   TotalGuessings
	in >> aux8;   //   LastRuleLevel
	in >> aux8;   //   LastRuleType
	in >> aux16;  //   LastRuleGroup
	in >> aux16;  //   LastRuleIntersectedGroup
	in >> aux32;  //   LastRuleSymbolsMask
	in >> aux32;  //   LastRuleCellsMask
	in >> aux16;  //   NumRulesN1
	in >> aux16;  //   NumRulesN2
	in >> aux16;  //   NumRulesNGt2

#ifdef _DEBUG
	printf ("SudokuView::readFile 1\n");
#endif

	// Variables of array of cells
	for (i = 0; i < sudoku_board->NumCells() &&
	            in.status()==QDataStream::Ok;    i ++ )
	{
		in >> aux32;  //   Cell number
		
		if ( (int)aux32 != i)
		{
			QMessageBox::warning(this, tr("Sudoku View"), tr("Error Cells data corrupt"));
			QApplication::restoreOverrideCursor();
			file.close();
			return false;
		}

                              // IGNORE...
		in >> aux8;   //   X coord.   // Should be 16
		in >> aux8;   //   Y coord.   // Should be 16
		in >> aux8;   //   NumGroupsOfCell
		in >> aux32;  //   Possible Values of cell (from board)

                              // IGNORE...
		in >> aux8;   //   NumPossValuesOfCell
		in >> aux8;   //   SymbolOfCell
		in >> aux16;  //   SolPositionOfCell
		in >> aux16;  //   GuessPositionOfCell
		in >> aux8;   //   Cell flags
		in >> aux32;  //   LastRemovedPossValuesOfCell

		// Variables of Graphcells

                              // IGNORE...
		in >> aux16;  //   nTextPoss
		
		in >> aux32;  // <<== mTextPoss (true possible values)
		
		RemovePossValuesFromCell (pBoard, ~(int)aux32, i);
		sudoku_board->GCell(i)->mTextPoss = aux32;

		in >> aux8;    bool canChange = (aux8 != 0);
		in >> aux8;    bool FvActivate = (aux8 != 0);
                
		sudoku_board->GCell(i)->canChange = true;  // canChange;
		
		int helpFV = GetSymbolOfCell (pBoard, i);

		if (FvActivate == true)
		{
			//sudoku_board2->GCell(i)->setFinalUserValue ( sudoku_board2->pCl[i].FV );
			sudoku_board->GCell(i)->setFinalUserValue (helpFV );
		}
		
		if (canChange == false)
		{
			sudoku_board->GCell(i)->setFinalUserValue ( helpFV );
			sudoku_board->GCell(i)->canChange = false;
		}

		if (FvActivate == false)
			sudoku_board->GCell(i)->FvActivate = false;
	}

	if (in.status()!=QDataStream::Ok)
	{
		QMessageBox::warning(this, tr("Sudoku View"), tr("Error Cells data corrupt"));
		QApplication::restoreOverrideCursor();
		file.close();
		return false;
	}

	file.close();

	ResizeCellFont();
	//?? OrderCheckBoard();
	QApplication::restoreOverrideCursor();
#ifdef _DEBUG
	printf ("LOAD completado\n");
#endif

	return true;
}

/*********************/
/* FreeSudokuInScene */
/*****************************************************************/
/*  Free memory of all objetcs in scene                          */
/*                                                               */
/*  NOTE: Be careful. This function can cause Segmention Default */
/*  TODO: Testing. Normally use clearBoard.                      */
/*****************************************************************/
void SudokuView::FreeSudokuInScene()
{
#ifdef _DEBUG
	printf("SudokuView::FreeSudokuInScene 1\n");
#endif	
	int i=0;
	/* Free Memory of Block and Grids */
	if (sudoku_board)
	{
		for (i=0; i < nmGroups; i++)
		{
			mScene->removeItem(ppGroupsRect[i]);
			delete DM_DEL (ppGroupsRect[i]);
		}
		free (DM_DEL(ppGroupsRect) );
		for (i=0; i < nmGrids; i++)
		{
			mScene->removeItem(ppGridsRect[i]);
			delete DM_DEL (ppGridsRect[i] );
		}
		free (DM_DEL (ppGridsRect) );
	}
}

/******************************/
/* UNDO / REDO FUNCTIONS      */
/******************************/

/*********************/
/* SaveMovement      */
/*****************************************************************/
/*  This function save the board in a SAVEX.sdk file and enable   */
/*  Undo actions.                                                */
/*                                                               */
/*  TODO: UnStable. Added in proyect27                           */
/*****************************************************************/
bool SudokuView::SaveMovement()
{
#ifdef _DEBUG
	printf ("SudokuView::SaveMovement 1\n");
#endif
	QString file,fileaux;
	int i=0;
	bool isgood=false;

	if (!sudoku_board) return isgood;

//	NoPlayerMovement = flag;

	file.append("SAVE");
	file.append(simbols(nIndexUndo));
	file.append(".sdk");

	fileaux.append("SAVE");
	fileaux.append(simbols(nIndexUndo));
	fileaux.append(".sdk");

	QString path = QDir::currentPath();
	path.append("/system/");

	if ( nIndexUndo > MAX_UNDO) 
	{
		nIndexUndo=MAX_UNDO;
		// Rename all SAVE9.sdk -> SAVE8.sdk; 
		//            SAVE8.sdk -> SAVE7.sdk, 
                //            ......................
                //	      SAVE1.sdk -> SAVE0.sdk 
		QDir dir(path);
		for (i= 0; i < MAX_UNDO+1 ; i++)
		{
			file[4]= QChar (simbols(i));
			fileaux[4]= QChar (simbols(i-1));
			dir.remove(fileaux);
			dir.rename(file,fileaux);
		}
		file[4]= QChar (simbols(MAX_UNDO));
	}
	
	path.append(file);
	// SAVE last movement and check it;
	if (! writeFile(path) )
	{
		QMessageBox::warning(this, tr("Main Menu"), tr("Can not Save Movement"));
		return isgood;
	}
	if (nIndexUndo)
	{	
		//Activate Undo.
		emit CanUndo (true);
		//Disable Redo
		emit CanRedo (false);
	}

	nIndexUndo++;
	nMaxUndo = nIndexUndo;

	isgood = true;
#ifdef _DEBUG
	printf ("n= %d \t Max= %d\n", nIndexUndo, nMaxUndo);
#endif

	/*ONLY CHECK BOARD IF THERE ARE SOME FINAL VALUE IN THE BOARD*/
	for (i=0; i < sudoku_board->NumCells(); i++)
	{	
		if (sudoku_board->GCell(i)->FvActivate )
		{
			/* Call to QThread Solver Process */
			/* Check If a Solution is possible */
			OrderCheckBoard();
			break;
		}
	}
	

	return isgood;
}

/*************/
/* Undo      */
/*****************************************************************/
/*  This function load last board of a SAVEX.sdk file and enable  */
/*  Redo actions.                                                */
/*                                                               */
/*  TODO: UnStable. Added in proyect27                           */
/*****************************************************************/
void SudokuView::Undo()
{
#ifdef _DEBUG
	printf("SudokuView::Undo 1\n");
#endif
	QString file;

	MY_ASSERT (isPlayerPlaying()==true);
	setComputerControl();

	if (!sudoku_board)
	{
	        MY_ASSERT (isPlayerPlaying()==false);
	        setPlayerControl();
	        return;
	}

	cleanAllCells();
	clearBoard();

	file.append("SAVE");
	file.append(simbols(nIndexUndo - 2));// because simbols  return 1  when 0. It adds 1
	file.append(".sdk");

	QString path = QDir::currentPath();
	path.append("/system/");
#ifdef _DEBUG
	printf ("UNDO! n= %d \t Max= %d\n", nIndexUndo, nMaxUndo);
#endif
	path.append(file);
	if (!readFile(path) )
	{
		QMessageBox::warning(this, tr("Main Menu"), tr("Can not Undo"));
	        MY_ASSERT (isPlayerPlaying()==false);
        	setPlayerControl();
		return;
	}
	nIndexUndo--;
	if (nIndexUndo <= 1)
	{
		// Disable Undo
		emit CanUndo (false);
	}
	// Enable Redo
	emit CanRedo (true);
#ifdef _DEBUG
	printf ("UNDO¡ n= %d \t Max= %d\n", nIndexUndo, nMaxUndo);
#endif
	/* Check if this Sudoku has errors */
	if ( ((MyMainWindow *)pMainWindow)->FlagHelpMode 
	   && ((MyMainWindow *)pMainWindow)->FlagExtraHelpMode ) OrderCheckBoard();
	else	OrderOnlyCheckBoard();

	MY_ASSERT (isPlayerPlaying()==false);
	setPlayerControl();

}

/*************/
/* Redo      */
/*****************************************************************/
/*  This function load the next board in a SAVEX.sdk file         */
/*  and enable Undo actions                                      */
/*                                                               */
/*  TODO: UnStable. Added in proyect27                           */
/*****************************************************************/
void SudokuView::Redo()
{
#ifdef _DEBUG
	printf("SudokuView::Redo 1\n");
#endif
	QString file;

	MY_ASSERT (isPlayerPlaying()==true);
	setComputerControl();

	if (!sudoku_board)
	{
	        MY_ASSERT (isPlayerPlaying()==false);
	        setPlayerControl();
	        return;
	}

	cleanAllCells();
	clearBoard();
#ifdef _DEBUG
	printf ("REDO! n= %d \t Max= %d\n", nIndexUndo, nMaxUndo);
#endif
	if (nIndexUndo < nMaxUndo)
	{
		file.append("SAVE");
		file.append(simbols(nIndexUndo));
		file.append(".sdk");
	
		QString path = QDir::currentPath();
		path.append("/system/");

		path.append(file);
		if (!readFile(path) )
		{
			QMessageBox::warning(this, tr("Main Menu"), tr("Can not Redo"));
	                MY_ASSERT (isPlayerPlaying()==false);
	                setPlayerControl();
			return;
		}
		nIndexUndo++;
	}
	if (nIndexUndo == nMaxUndo)
	{
		// Disable Redo
		emit CanRedo (false);
	}
	// Activate Undo
	emit CanUndo (true);
#ifdef _DEBUG
	printf ("REDO¡ n= %d \t Max= %d\n", nIndexUndo, nMaxUndo);
#endif
	MY_ASSERT (isPlayerPlaying()==false);
	setPlayerControl();
	if ( ((MyMainWindow *)pMainWindow)->FlagHelpMode 
	   && ((MyMainWindow *)pMainWindow)->FlagExtraHelpMode) OrderCheckBoard();
	else	OrderOnlyCheckBoard();

}

/*******************/
/* RestartUndo     */
/*****************************************************************/
/*  This function reset and restart all parameters of Undo/Redo   */
/*  actions. Must be called when game changes                    */
/*                                                               */
/*  TODO: UnStable. Added in proyect27                           */
/*****************************************************************/
bool SudokuView::RestartUndo()
{
#ifdef _DEBUG
	printf("SudokuView::RestartUndo 1\n");
#endif
	nIndexUndo = 0;
	nMaxUndo = 0;
	/* Set flags */
	ThisSudokuHasErrors=false;
	SudokuSolutionwasfound=false;
	flagSudokuWithErrorsinAllMoment= false;
	NoPlayerMovement = true;

	emit CanUndo (false);
	emit CanRedo (false);
	SaveMovement ();
#ifdef _DEBUG
	printf("SudokuView::RestartUndo END \n");
#endif
	return true;
}


/*******************/
/* isPlayerPlaying */
/*****************************************************************/
/*  This function returns flags isPlaying.                        */
/*****************************************************************/
bool SudokuView::isPlayerPlaying()
{
	return isPlaying;
}

bool SudokuView::CreateEnlargeWindow()
{
#ifdef _DEBUG
		printf ("SudokuView::CreateEnlargeWindow 1\n");
#endif
		//if (mEnlargeWindow==NULL)
		/*if (sudoku_board == NULL ) 
		{
			return false;
		}*/
		mEnlargeWindow = DM_NEW ( new EnlargeWindow( this) );

		/*else
		{
			delete DM_DEL (mEnlargeWindow);
			mEnlargeWindow = DM_NEW ( new EnlargeWindow( this) ) ;
		}*/
		
		mEnlargeWindow->setFocusPolicy(Qt::StrongFocus);
		
		return true;
}

bool SudokuView::HideEnlargeWindow()
{
#ifdef _DEBUG
	printf("SudokuView::HideEnlargeWindow 1\n");
#endif
		if (mEnlargeWindow!=NULL)
		{
			mEnlargeWindow->hide();
			return true;
		}
		return false;
#ifdef _DEBUG
	printf("SudokuView::HideEnlargeWindow end\n");
#endif
}

bool SudokuView::ShowEnlargeWindow()
{
#ifdef _DEBUG
		printf ("SudokuView::ShowEnlargeWindow 1\n");
#endif
		if (mEnlargeWindow!=NULL)
		{
			mEnlargeWindow->show();
			mEnlargeWindow->activateWindow();
			mEnlargeWindow->raise();
			return true;
		}
		return false;
}

bool SudokuView::ChangeWindowsLanguage()
{
#ifdef _DEBUG
		printf ("SudokuView::ChangeWindowsLanguage 1\n");
#endif
		bool ret=false;

		if (mEnlargeWindow!=NULL)
		{
			mEnlargeWindow->LanguageChange();
			ret=true;
		}
		if (mRulesDialog!=NULL)
		{
			mRulesDialog->LanguageChange();
			ret=true;
		}
		return ret;
}

void SudokuView::ctrlZPress()
{
	emit ctrlZPressed();
}
void SudokuView::ctrlXPress()
{
	emit ctrlXPressed();
}


void SudokuView::ClearListLastColorModifed()
{
#ifdef _DEBUG
	printf("SudokuView::ClearListLastColorModifed 1\n");
#endif
	if (!ListLastColorModifed.isEmpty());
	{
		for (int i = 0; i < ListLastColorModifed.size() ; i++)
		{
			(ListLastColorModifed[i])->setBrush(Qt::NoBrush);
			ListLastColorModifed[i]->UpdatePossUserText(
				 ListLastColorModifed[i]->pTextPoss[0]->font().pixelSize(),
				 ListLastColorModifed[i]->nTextPoss, false,true);
		}
		ListLastColorModifed.clear();
	}

}

void SudokuView::ClearListLastRulesModifed()
{
#ifdef _DEBUG
	printf("SudokuView::ClearListLastRulesModifed 1\n");
#endif
/*	if (!ListLastRulesModifed.isEmpty());
	{
		for (int i = 0; i < ListLastRulesModifed.size() ; i++)
		{
			(ListLastRulesModifed[i])->setBrush(Qt::NoBrush);
			// TODO: Probablemente tendremos, que modificar 
			//la siguiente función y modificar para este caso 
			//o la funcion añadiendo un parametro; o 
			//incrustandola aqui.
			ListLastRulesModifed[i]->UpdatePossUserText(
				 ListLastRulesModifed[i]->pTextPoss[0]->font().pixelSize(),
				 ListLastRulesModifed[i]->nTextPoss);
		}
		ListLastRulesModifed.clear();
	}*/
}

void SudokuView::setIsActiveHelpBox( bool value)
{
#ifdef _DEBUG
	printf("SudokuView::setIsActiveHelpBox 1\n");
#endif
	if ( value == false )
	{
		ClearListLastColorModifed();	
		isActiveHelpBox = false;
	}
	else
	{
		isActiveHelpBox = true;
		ActiveHelpBoxColor();
	}
}

void SudokuView::ActiveHelpBoxColor()
{
#ifdef _DEBUG
	  	printf("SudokuView::ActiveHelpBoxColor 1\n");
#endif

	int i=0, j=0;
	GraphCell * myFocusCell=NULL;
	
	if (!isActiveHelpBox) return;
	if (!sudoku_board) return;

	/* Changes when a cell focus changes */

	ClearListLastColorModifed();	

	myFocusCell= mEnlargeWindow->mCellFocus;

	/* Loop a. Find GraphCells of Block, Column,... and paint in a color */

	ListLastColorModifed.append (myFocusCell );

#ifdef _DEBUG
	  	printf("ActiveHelpBoxColor 2\n");
#endif

        t_board_p pBoard = sudoku_board->Board ();
        int focus_cell = sudoku_board->CellNumber (myFocusCell);
        int nGr = GetNumGroupsOfCell (pBoard, focus_cell);
        int nSy = sudoku_board->NumSymbols ();
        int group, cell;
        
	for (i=0; i < nGr; i++)
	{
#ifdef _DEBUG
	  	printf("valores de i= %d  | j=  %c | myFocusCell= %p | \n",i,j, myFocusCell);
#endif
                group = GetGroupOfCell (pBoard, i, focus_cell);
                
		if (GetGroupType(pBoard,group) == GR_BLOCK)
		{
			/* Paint all cell of the block on a different color */
			for (j=0; j < nSy ; j++)
			{
			        cell = GetCellOfGroup (pBoard, j, group);
				if (cell != focus_cell)
				{
				        sudoku_board->GCell(cell)->setBrush ( colorHelpBox1 );
					ListLastColorModifed.append (sudoku_board->GCell(cell));
				}
			}
		}
		else if (GetGroupType(pBoard,group) == GR_ROW)
		{
			/* Paint all cell of the block on a different color */
			for (j=0; j < nSy ; j++)
			{
			        cell = GetCellOfGroup (pBoard, j, group);
				if (! ListLastColorModifed.contains (sudoku_board->GCell(cell)))
				{
					sudoku_board->GCell(cell)->setBrush ( colorHelpBox2 );

					ListLastColorModifed.append (sudoku_board->GCell(cell) );
				}
			}
		}
		else if (GetGroupType(pBoard,group) == GR_COL)
		{
			/* Paint all cell of the block on a different color */
			for (j=0; j < nSy ; j++)
			{
			        cell = GetCellOfGroup (pBoard, j, group);
				if (! ListLastColorModifed.contains (sudoku_board->GCell(cell)))
				{
					sudoku_board->GCell(cell)->setBrush ( colorHelpBox3 );

					ListLastColorModifed.append (sudoku_board->GCell(cell) );
				}
			}
		}
  
	}

	myFocusCell=NULL;

	#ifdef _DEBUG
	  	printf("ActiveHelpBoxColor end\n");
	#endif
}


bool SudokuView::SudokuHasErrors()
{

#ifdef _DEBUG
  	printf("SudokuView::SudokuHasErrors 1\n");
#endif

	MY_ASSERT (isPlayerPlaying()==true);
	setComputerControl();

	if (!sudoku_board)
	{
	        MY_ASSERT (isPlayerPlaying()==false);
	        setPlayerControl();
	        return false;
	}
	
	if (!ThisSudokuHasErrors)
	{
	/* SAVE THE SUDOKU WITHOUT ERRORS */
		QString path = QDir::currentPath();
		QString fileName = QDir::currentPath();
		
		path.append("/system/NOERRSAV.sdk");

		fileName.append("/system/");
		fileName.append("SAVE");
		
		if (/*nIndexUndo < 2 &&*/ NoPlayerMovement) // Check Special Case when the Loaded or Generated Sudoku has errors.
		{
	//		QMessageBox::warning(this, tr("Sudoku View"), tr("This Sudoku Has Errors. \nIt is not possible find a Movement without errors.\n Probably you have loaded a bad file game.\n Please delete that and restart.\n"));
			ThisSudokuHasErrors=true;
			flagSudokuWithErrorsinAllMoment = true;
	                MY_ASSERT (isPlayerPlaying()==false);
	                setPlayerControl();
			return false;
		}
		fileName.append(simbols(nIndexUndo - 2));// because simbols  return 1  when 0. It adds 1
//		fileName.append(simbols(nIndexUndo - 1));// because simbols  return 1  when 0. It adds 1
		fileName.append(".sdk");
	
		QFile file (fileName);
		QFile file2 (path);
		file2.remove();
		if (!file.open(QIODevice::ReadWrite))
		{
			QMessageBox::warning(this, tr("Sudoku View"), tr("Cannot read file %1:\n%2.").arg(file.fileName()).arg(file.errorString()));
			file.close();
	                MY_ASSERT (isPlayerPlaying()==false);
	                setPlayerControl();
			return false;
		}
	
		// SAVE it;
		if (! 	file.copy(path) )
		{
			
			QMessageBox::warning(this, tr("Main Menu"), tr("Can not Save the game"));
			file.close();
	                MY_ASSERT (isPlayerPlaying()==false);
	                setPlayerControl();
			return false;
		}

		file.close();
	}

	ThisSudokuHasErrors=true;

	MY_ASSERT (isPlayerPlaying()==false);
	setPlayerControl();

	return true;	
}


void SudokuView::OrderCheckBoard()
{
#ifdef _DEBUG
	printf("SudokuView::OrderCheckBoard 1\n");
#endif
	int i;
	/* Create new board similar to sudoku_board*/
	int block_x[15];
	int block_y[15];
	
	t_board_p pBoard = sudoku_board->Board ();
	int wb = GetBlockWidth (pBoard);
	int hb = GetBlockHeight (pBoard);

	mCheckThread->Abort();

	mSenseiThread->Abort();

	for (i=0; i < nmGrids; i++)
	{
			/* Conversion units cell -> units block */
			block_x[i] = ( GridsX[i] -1 + wb) / wb;
			block_y[i] = ( GridsY[i] -1 + hb) / hb;
	}

#ifdef _DEBUG
	printf("SudokuView::OrderCheckBoard -> ClearRules \n");
#endif
	mRulesDialog->ClearRules();

#ifdef _DEBUG
	printf("SudokuView::OrderCheckBoard -> ClearRules END\n");
#endif

	mCheckThread->Wait_for_completion ();
	
	mSenseiThread->Wait_for_completion ();


#ifdef _DEBUG
	printf("SudokuView::OrderCheckBoard 1.5\n");
#endif
	check_board->Clear ();
	thread_board->Clear ();


///	DestroyBoard(check_board);
///	DestroyBoard(thread_board);


	delete [] ( DM_DEL (pMaskCellsArray) );
	pMaskCellsArray =  DM_NEW ( new int[ sudoku_board->NumCells() ] );

#ifdef _DEBUG
	printf("SudokuView::OrderCheckBoard 1.75\n");
#endif
///	check_board = NULL;
///	check_board = ConstructCustomBoard( sudoku_board->wb ,
///                                            sudoku_board->hb ,
///                                            0, nmGrids, block_x, block_y  );
	if (!check_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Check Board"), tr("The check board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
		return;
	}

#ifdef _DEBUG
	printf("SudokuView::OrderCheckBoard 2\n");
#endif
///	thread_board = NULL;
///	thread_board = ConstructCustomBoard( sudoku_board->wb ,
///                                             sudoku_board->hb ,
///nnn                                          0, nmGrids, block_x, block_y  );
	if (!thread_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Check Board"), tr("The check board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
		return;
	}

#ifdef _DEBUG
	printf("SudokuView::OrderCheckBoard 3\n");
#endif

	/* Put all values in new_board */
	for (i=0; i < sudoku_board->NumCells(); i++)
	{
		RemovePossValuesFromCell (check_board->Board(),
		                          ~sudoku_board->GCell(i)->mTextPoss, i);
		                          
		RemovePossValuesFromCell (thread_board->Board(),
		                          ~sudoku_board->GCell(i)->mTextPoss, i);

		// Save all masks for Redundant Algoritm
		pMaskCellsArray[i] = sudoku_board->GCell(i)->mTextPoss;
	}

#ifdef _DEBUG
	printf("SudokuView::OrderCheckBoard START mCheckThread\n");
#endif
//TODO: move Abort to the init.

	mCheckThread->Init (check_board->Board(),1);
	mCheckThread->Start_running ();

	/**/
#ifdef _DEBUG
	printf("SudokuView::OrderCheckBoard START mSenseiThread\n");
#endif
//	usleep(300);


	mSenseiThread->Init (thread_board->Board(),0);
	mSenseiThread->Start_running ();

#ifdef _DEBUG
	printf("SudokuView::OrderCheckBoard END\n");
#endif
}

void SudokuView::OrderOnlyCheckBoard()
{
#ifdef _DEBUG
	printf("SudokuView::OrderCheckBoard 1\n");
#endif
	int i;
	/* Create new board similar to sudoku_board*/
	int block_x[15];
	int block_y[15];

        t_board_p pBoard = sudoku_board->Board ();
        int wb = GetBlockWidth (pBoard);
        int hb = GetBlockHeight (pBoard);

	mCheckThread->Abort();

	for (i=0; i < nmGrids; i++)
	{
			/* Conversion units cell -> units block */
			block_x[i] = ( GridsX[i] -1 + wb) / wb;
			block_y[i] = ( GridsY[i] -1 + hb) / hb;
	}
	mCheckThread->Wait_for_completion ();

#ifdef _DEBUG
	printf("SudokuView::OrderOnlyCheckBoard 1.5\n");
#endif
	check_board->Clear ();

#ifdef _DEBUG
	printf("SudokuView::OrderOnlyCheckBoard 1.75\n");
#endif
///	check_board = NULL;
///	check_board = ConstructCustomBoard( wb , hb ,
///                                            0, nmGrids, block_x, block_y  );
	if (!check_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Check Board"), tr("The check board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
		return;
	}

#ifdef _DEBUG
	printf("SudokuView::OrderOnlyCheckBoard 3\n");
#endif

	/* Put all values in new_board */
	for (i=0; i < sudoku_board->NumCells(); i++)
	{
		RemovePossValuesFromCell (check_board->Board(),  ~sudoku_board->GCell(i)->mTextPoss, i);
	}

#ifdef _DEBUG
	printf("SudokuView::OrderCheckBoard START mCheckThread\n");
#endif
//TODO: move Abort to the init.

	mCheckThread->Init (check_board->Board(),1);
	mCheckThread->Start_running ();


	// TODO: To avoid problems with undo/redo
	mCheckThread->Wait_for_completion ();
#ifdef _DEBUG
	printf("SudokuView::OrderOnlyCheckBoard END\n");
#endif
}


bool SudokuView::CheckIntegrity(int a)
{
#ifdef _DEBUG
	printf("SudokuView::CheckIntegrity 1\n");
#endif
	if (a== -1) /* The sudoku can not be solved */
	{
#ifdef _DEBUG
		printf ("Hay errores. respuesta : %d\n",a);
#endif
		SudokuHasErrors();
		return false;


	}
	else if (a > 0)
	{
#ifdef _DEBUG
		printf ("No tenemos aun solución. PASOS : %d\n",a);
#endif
		ThisSudokuHasErrors = false;
	}
	else if (a == 0)
	{
#ifdef _DEBUG
		printf ("Tenemos Solucion Unica \n");
#endif
		ThisSudokuHasErrors = false;
		flagSudokuWithErrorsinAllMoment = false;

		/* SOLVE THE SUDOKU IF IT IS NOT SOLVED */
		if (! SudokuSolutionwasfound )
		{
				
				FILE * pf = fopen("SOLUTION.htm", "wb");
				if (pf == NULL ) return false;

	/* We can use check_board without mutex or semaphores because: */
	/* 1. This fuction is called when mSenseiThread 
	   has already finished. */
	/* 2. while this fuction is called, nobody can restart
	   mSenseiThread or change check_board because all 
	   SudokuView fuctions are in the same thread */

				PrintHtmlBoard (check_board->Board(),
						pf,
						10,
                                                symbols_string(),
                                                NULL);
				fclose( pf);
#ifdef _DEBUG
				printf ("Impreso en HTML \n");
#endif
				SudokuSolutionwasfound=true;
				
			
		}
		
	}
	return true;
}

