/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: SudokuView.h  * Status: stable * Version: 2.00 * last changes in version: 0.46 	*/
/************************************************************************************************/
/*  This file contains the definitions of SudokuView, mToolButton, EnlargeWindow		*/
/*   RulesDialog classes									*/
/*												*/
/*   These class are defined in SudokuView.cpp							*/
/*				enlargewindow.cpp						*/
/*				diagrules.cpp							*/
/*												*/
/************************************************************************************************/

#ifndef SUDOKUVIEW_H
#define SUDOKUVIEW_H
#include <QColor>
#include <QImage>
#include <QGraphicsView>
#include <QGraphicsScene>
#include <QWidget>
#include <QFrame>
#include <QFont>
#include "sudoku-pub.h"
#include <QDialog>
#include <QTranslator>
#include <QString>
#include <QColor>
#include <QList>
#include <QLabel>
#include <QPushButton>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QListWidget>
#include <QMutex>
#include <QtWidgets>

#include "ui_Denlargewindow.h"
#include "thread-sensei.h"
#include "ui_Drules.h"
//#include "mainwindow.h"
#include "myassert.h"

//#define _DEBUG

//class EnlargeWindow;

#define DATA_GRID_X 0
#define DATA_GRID_Y 1

#define BACK_COLOR 200,200,100
#define SMALL_TEXT_COLOR 110,100,125
#define WIDTH_GRID_BORDER 8  // 6 + 1 (width cell ) * 2
#define HALF_WIDTH_GRID_BORDER 4 // 3 + 1 (width cell )
#define HALF_WIDTH_BLOCK_BORDER 4 //
#define WIDTH_W_BORDER 40
#define WIDTH_H_BORDER 40
#define _2WIDTH_W_BORDER 80
#define _2WIDTH_H_BORDER 80

#define MAGICNUMBER   0x100CAFE5 

// Number of files SAVEx.sdk = MAX_UNDO + 1
#define MAX_UNDO 15

class EnlargeWindow;
class RulesDialog;
class GraphCell;
class SudokuBoard;

class SudokuView : public QFrame
{
    Q_OBJECT
    //Q_PROPERTY(QColor penColor READ penColor WRITE setPenColor)
    //Q_PROPERTY(QImage iconImage READ iconImage WRITE setIconImage)
    //Q_PROPERTY(int zoomFactor READ zoomFactor WRITE setZoomFactor)



signals:
    	void MinimunSize (int width, int height);
        void CanUndo(bool );
	void CanRedo(bool );
	void ctrlZPressed();
	void ctrlXPressed();

private: 
	volatile bool isPlaying;

public:
    //QWidget *padre;

    SudokuBoard * sudoku_board;
    SudokuBoard * check_board;
    SudokuBoard * thread_board;

    void * pMainWindow;
    QGraphicsScene * mScene;
    QGraphicsView *graphicsView;
    SenseiThread * mCheckThread;
    SenseiThread * mSenseiThread;

    int * pMaskCellsArray;

/*  COLOR VARIABLES */

    QColor   colorCell;
    QColor   colorBlock;
    QColor   colorGrid;
    QColor   colorBigText;
    QColor   colorSmallText;
    QColor   curColor;

/*  COLOR HELP VARIABLES */

    bool isActiveHelpBox;
    QColor colorHelpBox1;
    QColor colorHelpBox2;
    QColor colorHelpBox3;

    QColor colorRule1color1;
    QColor colorRule1color2;
    QColor colorRule1color3;

    QColor colorRule2color1;
    QColor colorRule2color2;
    QColor colorRule2color3;

    QColor colorRule3color1;
    QColor colorRule3color2;
    QColor colorRule3color3;

    QList <GraphCell *> ListLastColorModifed;
    QList <GraphCell *> ListLastRulesModifed;

    QImage image;
    int zoom;

    bool ThisSudokuHasErrors;
    bool flagSudokuWithErrorsinAllMoment;
    bool SudokuSolutionwasfound;

    RulesDialog * mRulesDialog;


    EnlargeWindow  * mEnlargeWindow;
    bool flagEnlarge;

    /* QColor   colorBack; */
    QGraphicsRectItem ** ppGroupsRect;
    short		nmGroups;

    QGraphicsRectItem ** ppGridsRect;
    short		nmGrids;
    int GridsX[15];
    int GridsY[15];

    int Table_TextCell_Width[15];
    int Table_TextCell_Height[15];
    float Table_TextCell_Proportion[15];
    int Table_TextCell_Size;


    QFont mFont;
    QFont mBigFont;

    /*  UNDO AND REDO VARIABLES */
    char nMaxUndo;
    char nIndexUndo;


   SudokuView( QWidget *parent = 0);
   ~SudokuView();

    SudokuBoard * CreateSimpleSudoku(int order = 3, char diag = 0);
    SudokuBoard * CreateSamurai(
                             char wb = 3,     /* Block width */
                             char hb = 3,     /* Block height */
                             char diag = 0);   /* TRUE == diagonals */
    SudokuBoard * CreateCustom ( int nGrids,  /* Num. composed grids */
                             int * xg,    /* Coordinates of */
                             int * yg,
			     char wb = 3,     /* Block width */
                             char hb = 3,     /* Block height */
                             char diag = 0   /* TRUE == diagonals */
                             );

   void ctrlZPress();
   void ctrlXPress();

   void ResizeCellFont();
   void SetupTextCell();

   void FreeSudokuInScene();

   void ClearListLastColorModifed();
   void ClearListLastRulesModifed();

   void RepaintBackGround( QColor mcolor );
   void RepaintGrid();
   void RepaintBlock();
   void RepaintCell();
   void RepaintBigText ();
   void RepaintSmallText ();
   void RepaintCursor ();

   void ActiveHelpBoxColor();

public slots:
	bool CreateEnlargeWindow();
	bool HideEnlargeWindow();
	bool ShowEnlargeWindow();
	bool ChangeWindowsLanguage();
	bool SudokuHasErrors();
	bool CheckIntegrity(int a);

public:


   bool writeFile (const QString &fileName);
   bool readFile (const QString &fileName);

	bool SaveMovement();
	bool NoPlayerMovement;
	void Undo();
	void Redo();
	bool RestartUndo();
        bool isPlayerPlaying();

	
	void setIsActiveHelpBox( bool value);
	
        void setPlayerControl()
        {
                MY_ASSERT (isPlaying==false);
                isPlaying = true;
        }
        
        void setComputerControl()
        {
                MY_ASSERT (isPlaying==true);
                isPlaying = false;
        }

   /* void setZoomFactor(int newZoom);
    int zoomFactor() const { return zoom; }
   
protected:
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event); */
  
    void paintEvent(QPaintEvent *event);
    void resizeEvent (QResizeEvent *event);
    void cleanAllCells();
    

/*
private:
    void setImagePixel(const QPoint &pos, bool opaque);
    QRect pixelRect(int i, int j) const;*/


  void OrderCheckBoard();
private:
    void PutSudokuInScene();
    void clearBoard();
  
    void OrderOnlyCheckBoard();
};

class mToolButton : public QToolButton
{
	Q_OBJECT

	bool Pressed;
public:
	mToolButton( QWidget * parent = 0): QToolButton(parent),Pressed(false)
	{
		connect (this,SIGNAL (clicked()) , this, SLOT(setPressed()) );
	}

public slots:
	void setPressed(){ Pressed= true; }
	bool checkPressed() { bool i = Pressed; Pressed = false; return i;}

};

class EnlargeWindow : public QMainWindow, Ui_EnlargeWindow
{
	Q_OBJECT

public:

	GraphCell *mCellFocus;

	EnlargeWindow (SudokuView * mSudokuView);
	~EnlargeWindow();
	void resizeEvent (QResizeEvent *event);
	void keyPressEvent (QKeyEvent * event);


public slots:
	void centerWindow( int cell);
	void ChangeSelect();
	void PressButton();
	bool LanguageChange();
	void ChangeSudoku();



private:
	mToolButton ** mButtons;
	QToolButton * mSelect;
	bool fFinalValue ;
	QGraphicsView * mView;
	SudokuView * mSudoku;

	int mScale;
	int nButtons;
};

/*************************/
/* RulesDialog           */
/*************************/

class RulesDialog : public QDialog
{
	Q_OBJECT

public:
	RulesDialog (SudokuView * mSud, QWidget * parent = 0, Qt::WindowFlags f = 0 ); //Constructor
	~RulesDialog();	// Destructor

private:
	bool isSimpleRuleActive;  // Flags
	bool isMediumRuleActive;
	bool isExpertRuleActive;
	bool isHiddenRuleActive;


	bool Show1orHide0;	// Flag. Is on  (1) if But_Show_Rules is in ShowMode
				//       Is off (0) if But_Show_Rules is in HideMode

	QMutex mRulesMutex;	// Mutex necessary to avoid problems.

	QLabel* label;		// Text labels
	QLabel* label_2;
	QLabel* label_3;
	QLabel* label_4;


	QListWidget* listSimpleRules; // Visual Lists
	QListWidget* listMediumRules;
	QListWidget* listExpertRules;
	QListWidget* listHiddenRules;

	QPushButton* But_Apply_Rule;  // Buttons
	QPushButton* But_Show_Rules;
	QPushButton* But_Apply_All ;

	QHBoxLayout* LayoutButtons;  // Layouts
	QVBoxLayout* MainLayout;

	SudokuView * mSudoku;        // Pointer to parent 

	
	QList<s_rule> mSimpleRules;  // Internal list of rules.
	QList<s_rule> mMediumRules;
	QList<s_rule> mExpertRules;
	QList<s_rule> mHiddenRules;
public:

	int nRules;		     // number of rules detected
// FUNCTIONS
	void retranslateUi(QDialog *Rules);

private:
	bool SelectPaint(s_rule * rule);
	bool Paint1_Rule(s_rule * rule);// old Paint1ARule
//	bool Paint1BRule(s_rule * rule);
	bool Paint2_Rule(s_rule * rule);
	bool IsRulesRedundant(s_rule * rule );



public slots:
	int AddNewRule();

	bool ClearRules();
	bool ChangeSelectedSimplyRule();
	bool ChangeSelectedMediumRule();
	bool ChangeSelectedExpertRule();
	bool ChangeSelectedHiddenRule();
	
	void LanguageChange();
	void CheckButtonApplyRule();	// This slot recives the clicks of But_Apply_Rule
	
	bool ApplyRule( s_rule  rule);  // Apply the rule in the board. Is called by CheckButtonApplyRule and
					// ApplyAllRules.

	bool ApplyAllRules();		// This slot recives the clicks of But_Apply_All
	bool ShowHideRules();		// This slot recives the clicks of But_Show_Rules
private:
	bool ChangeSelectedRules( int mlist);
	void AddNameOfGroup( s_rule rule, QString *a, SudokuView * mSudoku, bool isIterns=false);
};



#endif
