/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: SudokuWidget.cpp    * Status: stable * Version: 1.00 * last changes in version: 0.10 	*/
/************************************************************************************************/
/*  This file contains functions of SudokuWidget class                                  	*/
/*												*/
/*   The class are defined in SudokuWidget.h							*/
/*												*/
/*   It's called by MyNewBoard class defined in newtable.h, newtable.cpp, Dnewtable.ui	        */
/*												*/
/*   It's based in SudokuView class defined in SudokuView.h, SudokuView.cpp (early versions)	*/
/************************************************************************************************/

#include "SudokuWidget.h"
#include <QPainter>
#include <QMessageBox>
#include "dmem.h"

/***************/
/* Constructor */
/*************************************************************************************************/
/* This function builds a SudokuWidget object. If there isn't board it creates a classic sudoku */
/************************************************************************************************/
SudokuWidget::SudokuWidget( QWidget *parent) : QWidget(parent) , sudoku_board(NULL)
{
      setPalette(QPalette(QColor(250, 250, 200)));
      setAutoFillBackground(true);

	/* Create a Simple Board */
	/* TODO : Remove it */

	//t_board_p
	CreateSimpleSudoku(3 , 0);
	
}
/***************/
/* Constructor */
/*****************************************************************/
/* This function builds a SudokuWidget object with a given board */
/*****************************************************************/
/*
SudokuWidget::SudokuWidget(t_board * board , QWidget *parent) : QWidget(parent), sudoku_board (board)
{
}
*/

/***************/
/* PaintEvent  */
/**********************************/
/* This function paints the board */
/**********************************/
void SudokuWidget::paintEvent(QPaintEvent *event)
{
	short i=0;
	int width_window=0;
	int height_window=0;
	int w_block, h_block, size_block;
	
	// Set Painter
	QPainter painter(this);
	QPen myPen;
	myPen.setColor(Qt::blue);
	myPen.setWidth(1);

	painter.setPen(myPen);
	painter.setBrush(Qt::NoBrush);


	if (sudoku_board!=NULL)
	{
	
		/** Painting the CELLs **/
		width_window=rect().width();
		height_window=rect().height();
	
		int w = GetBoardWidth (sudoku_board);
		int h = GetBoardHeight (sudoku_board);
		int wb = GetBlockWidth (sudoku_board);
		int hb = GetBlockHeight (sudoku_board);
		int nGr = GetNumGroups (sudoku_board);
		int nSy = GetNumSymbols (sudoku_board);
		int nCl = GetNumCells (sudoku_board);

		w_block = width_window  / ( 2 + w ); // we add 2 block of separator
		h_block = height_window / ( 2 + h );
	
		if ( w_block < h_block )
			size_block=  w_block;
		else 
			size_block=  h_block;
	
		
		for (i =0; i < nCl; i++)
		{

			painter.drawRect(QRect(
				(GetCellX(sudoku_board,i) +1)*size_block,
				(GetCellY(sudoku_board,i) +2)*size_block,
					size_block,
					size_block));
			
		}
	
	
		/** Painting the BLOCKs **/
		myPen.setColor(Qt::blue);
		myPen.setWidth(3);
	
	
		painter.setPen(myPen);
	
		for (i =0; i < nGr; i++)
		{
			if (GetGroupType(sudoku_board,i) == GR_BLOCK)
			{
			        int first_cell = GetCellOfGroup (sudoku_board, 0, i);
				painter.drawRect(QRect(
					(GetCellX(sudoku_board,first_cell) +1)*size_block,
					(GetCellY(sudoku_board,first_cell) +2)*size_block,
					wb    * size_block,
					hb    * size_block));
			}	
		}
	
		/** Painting the GRIDs **/
	
		myPen.setColor(Qt::red);
		myPen.setWidth(3);
		painter.setPen(myPen);
	
		int k=0;
		for (i =0; i < nGr; i++)
		{
			if (i == k*(3* nSy)) //sudoku_board->pGr[i].type == GR_DIAG)
			{
			
			        int first_cell = GetCellOfGroup (sudoku_board, 0, i);
				painter.drawRect(QRect(
					(GetCellX(sudoku_board,first_cell) +1)*size_block,
					(GetCellY(sudoku_board,first_cell) +2)*size_block,
					wb * hb * size_block,
					hb * wb * size_block));
				k++; // Because the next block to a \ diagonal is the other diagonal / 
			
			}	
		}
	}
}

/**********************/
/* CreateSimpleSudoku */
/************************************************/
/* This function creates a classic sudoku board */
/************************************************/
t_board_p SudokuWidget::CreateSimpleSudoku(int order , char diag)
{
	// Delete old board
	if (sudoku_board )
	{
	
		 DestroyBoard (sudoku_board); 
		 sudoku_board = NULL; 
	
	}

	// Create classic board
	sudoku_board = ConstructBasicBoard ( order, diag);

	// Check if it's ok.
 	if (!sudoku_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Board"), tr("The board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
	}
	return	 sudoku_board;
}

/****************/
/* CreateCustom */
/****************************************/
/* This function creates a custom board */
/****************************************/
t_board_p SudokuWidget::CreateCustom(
                             int nGrids,  /* Num. composed grids */
                             int * xg,    /* Coordinates of */
                             int * yg,
                             char wb ,     /* Block width */
                             char hb ,     /* Block height */
                             char diag   /* TRUE == diagonals */
                             )
{
	// Delete old board
	if ( sudoku_board) 
	{
		 DestroyBoard (sudoku_board); 
		 sudoku_board = NULL; 
        }
	// Create custom board
	sudoku_board = ConstructCustomBoard ( wb, hb, diag, nGrids, xg, yg);

	// Check if it's ok.
	if (!sudoku_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Board"), tr("The board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
	}
	return	 sudoku_board;
}

/****************/
/* CreateSamurai*/
/*****************************************/
/* This function creates a samurai board */
/*****************************************/
t_board_p SudokuWidget::CreateSamurai (  char wb ,     /* Block width */
                             char hb ,     /* Block height */
                             char diag )  /* TRUE == diagonals */
{

	// Delete old board
	if (sudoku_board ) 
	{
		 DestroyBoard (sudoku_board); 
		 sudoku_board = NULL; 
	}
	// Create samurai board
	sudoku_board = ConstructSamuraiBoard( wb, hb , diag);

	// Check if it's ok.
	if (!sudoku_board)
	{
 	 	QMessageBox A(tr("Memory Error creating Board"), tr("The board wasn't be created. \n Not enough space in memory\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

		A.exec();
	}

	return	 sudoku_board;
}
