/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: diagrules.cpp  * Status: stable * Version: 2.00 * last changes in version: 0.36 	*/
/************************************************************************************************/
/*  This file contains functions of RulesDialog class   					*/
/*												*/
/*   The class are defined in SudokuView.h							*/
/*												*/
/*   It's called by SudokuView class defined in SudokuView.h, SudokuView.cpp			*/
/*												*/
/************************************************************************************************/

#include "sudoku-pub.h"
#include "mask.h"
#include "SudokuView.h"
#include "SudokuBoard.h"
#include "dmem.h"
#include "myassert.h"

#include <QMessageBox>

#define MAX_RULES 100
//#define _DEBUG

/***************/
/* Constructor */
/***************/
RulesDialog::RulesDialog (SudokuView * mSud,
			  QWidget * parent ,
                          Qt::WindowFlags f): QDialog(parent,f),
				      isSimpleRuleActive(false),
				      isMediumRuleActive(false),
				      isExpertRuleActive(false),
				      isHiddenRuleActive(false),
					    Show1orHide0(false),
						  mSudoku(mSud)

{ 

	/* Setup GUI */
	/* Text of the GUI */
	label 	     = DM_NEW ( new QLabel(tr("&Simple Rules"  )) );
	label_2      = DM_NEW ( new QLabel(tr("A&dvanced Rules")) );
	label_3	     = DM_NEW ( new QLabel(tr("&Expert Rules"  )) );
	label_4	     = DM_NEW ( new QLabel(tr("&Redundant Rules"  )) );
	listSimpleRules      = DM_NEW ( new QListWidget( ) );
	listMediumRules      = DM_NEW ( new QListWidget( ) );
	listExpertRules      = DM_NEW ( new QListWidget( ) );
	listHiddenRules      = DM_NEW ( new QListWidget( ) );

	nRules = 0;
	// link label and lists
	label->setBuddy(listSimpleRules);
	label_2->setBuddy(listMediumRules);
	label_3->setBuddy(listExpertRules);
	label_4->setBuddy(listHiddenRules);

	//Buttons Apply Rule and Apply All Rules
	But_Apply_Rule = DM_NEW ( new QPushButton(tr("Apply Rule")) );
	But_Show_Rules = DM_NEW ( new QPushButton(tr("Show Redundant Rules")) );
	But_Apply_All  = DM_NEW ( new QPushButton(tr("Apply All Rules")) );
	

	/* Put buttons on layouts */

	// Layout of Buttons
	LayoutButtons = DM_NEW ( new QHBoxLayout );
	LayoutButtons->addWidget(But_Apply_Rule);
	LayoutButtons->addStretch();
	LayoutButtons->addWidget(But_Show_Rules);
	LayoutButtons->addStretch();
	LayoutButtons->addWidget(But_Apply_All);

	// Final Layout 
	MainLayout = DM_NEW ( new QVBoxLayout );
	MainLayout->addWidget(label);
	MainLayout->addWidget(listSimpleRules);
	MainLayout->addWidget(label_2);
	MainLayout->addWidget(listMediumRules);
	MainLayout->addWidget(label_3);
	MainLayout->addWidget(listExpertRules);
	MainLayout->addWidget(label_4);
	MainLayout->addWidget(listHiddenRules);
	label_4->setVisible(false);
	listHiddenRules->setVisible(false);
	MainLayout->addLayout(LayoutButtons);

	// See the Final Layout:
	setLayout(MainLayout);

	setWindowTitle(tr("Rules"));

	//Ajust size
	//TODO: Review and change if is necessary
	//setFixedHeight(sizeHint().height());
//	setMinimumSize(632,488);
//	setSize(632,488);

	/* Set Typical Attributes  */
	setAttribute(Qt::WA_QuitOnClose,false);
	setAttribute(Qt::WA_DeleteOnClose,false);
	setWindowFlags(Qt::WindowShadeButtonHint);

	/* ClearSet Typical Attributes  */
//	ClearRules(); This is not necessary

	/* Set ListViewTypical Attributes  */
	listSimpleRules->setSelectionMode( QAbstractItemView::SingleSelection);
	listMediumRules->setSelectionMode( QAbstractItemView::SingleSelection);
	listExpertRules->setSelectionMode( QAbstractItemView::SingleSelection);
	listHiddenRules->setSelectionMode( QAbstractItemView::SingleSelection);

	listSimpleRules->setSelectionBehavior( QAbstractItemView::SelectRows);
	listMediumRules->setSelectionBehavior( QAbstractItemView::SelectRows);
	listExpertRules->setSelectionBehavior( QAbstractItemView::SelectRows);
	listHiddenRules->setSelectionBehavior( QAbstractItemView::SelectRows);


	/* Connections */

	// Lists connections

	connect(listSimpleRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedSimplyRule() ));
	connect(listMediumRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedMediumRule() ));
	connect(listExpertRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedExpertRule() ));
	connect(listHiddenRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedHiddenRule() ));

	connect(listSimpleRules, SIGNAL ( itemClicked ( QListWidgetItem *)),this, SLOT (ChangeSelectedSimplyRule() ));
	connect(listMediumRules, SIGNAL ( itemClicked ( QListWidgetItem *)),this, SLOT (ChangeSelectedMediumRule() ));
	connect(listExpertRules, SIGNAL ( itemClicked ( QListWidgetItem *)),this, SLOT (ChangeSelectedExpertRule() ));
	connect(listHiddenRules, SIGNAL ( itemClicked ( QListWidgetItem *)),this, SLOT (ChangeSelectedHiddenRule() ));

//	connect(listSimpleRules, SIGNAL (currentRowChanged ( int)),this, SLOT (ChangeSelectedSimplyRule() ));
//	connect(listMediumRules, SIGNAL (currentRowChanged ( int)),this, SLOT (ChangeSelectedMediumRule() ));
//	connect(listExpertRules, SIGNAL (currentRowChanged ( int)),this, SLOT (ChangeSelectedExpertRule() ));
//	connect(listHiddenRules, SIGNAL (currentRowChanged ( int)),this, SLOT (ChangeSelectedHiddenRule() ));


	// Buttons connections
	connect(But_Apply_Rule, SIGNAL ( clicked() ),this, SLOT ( CheckButtonApplyRule() ) );
	connect(But_Apply_All , SIGNAL ( clicked() ),this, SLOT ( ApplyAllRules() ) );
	connect(But_Show_Rules, SIGNAL ( clicked() ),this, SLOT ( ShowHideRules() ) );

	/* Translation */

	retranslateUi(this);
}

/***************/
/* Destructor  */
/***************/
RulesDialog::~RulesDialog()
{
	delete DM_DEL (label       );
	delete DM_DEL (label_2     );
	delete DM_DEL (label_3     );
	delete DM_DEL (label_4     );
	delete DM_DEL (listSimpleRules    );
	delete DM_DEL (listMediumRules    );
	delete DM_DEL (listExpertRules    );
	delete DM_DEL (listHiddenRules    );

	delete DM_DEL (But_Apply_Rule     );
	delete DM_DEL (But_Apply_All      );
	delete DM_DEL (But_Show_Rules     );

	delete DM_DEL (LayoutButtons );
	delete DM_DEL (MainLayout    );
}
 
/**************/
/* ClearRules */
/*********************************************************/
/* Clear variables and lists. 				 */
/* This function returns the dialog to the initial state.*/
/* It is necessary disconnect lists to clear all lists.  */
/*********************************************************/
bool RulesDialog::ClearRules()
{
	/* Clear colored cells */
	//  mSudoku->ClearListLastColorModifed();

	/* Disconnect */

	disconnect(listSimpleRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedSimplyRule() ));
	disconnect(listMediumRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedMediumRule() ));
	disconnect(listExpertRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedExpertRule() ));
	disconnect(listHiddenRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedHiddenRule() ));

/*	disconnect(listSimpleRules, SIGNAL ( itemClicked ( QListWidgetItem *)),this, SLOT (ChangeSelectedSimplyRule() ));
	disconnect(listMediumRules, SIGNAL ( itemClicked ( QListWidgetItem *)),this, SLOT (ChangeSelectedMediumRule() ));
	disconnect(listExpertRules, SIGNAL ( itemClicked ( QListWidgetItem *)),this, SLOT (ChangeSelectedExpertRule() ));
	disconnect(listHiddenRules, SIGNAL ( itemClicked ( QListWidgetItem *)),this, SLOT (ChangeSelectedHiddenRule() ));
*/

#ifdef _DEBUG
	printf("RulesDialog::ClearRules 1\n");
#endif
	/* Clear Variables */
	isSimpleRuleActive=false;
	isMediumRuleActive=false;
	isExpertRuleActive=false;
	isHiddenRuleActive=false;

#ifdef _DEBUG
	printf("RulesDialog::ClearRules 1.5\n");
#endif
	/* Clear List of rules */
      mSimpleRules.clear();
      mMediumRules.clear();
      mExpertRules.clear();
      mHiddenRules.clear();


      nRules=0;
#ifdef _DEBUG
	printf("RulesDialog::ClearRules 2\n");
#endif
	/* Clear ListView of rules */
      listSimpleRules->clear();
      listMediumRules->clear();
      listExpertRules->clear();
      listHiddenRules->clear();

	/* Reconnect the list */
	connect(listSimpleRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedSimplyRule() ));
	connect(listMediumRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedMediumRule() ));
	connect(listExpertRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedExpertRule() ));
	connect(listHiddenRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedHiddenRule() ));

/*	connect(listSimpleRules, SIGNAL ( itemClicked ( QListWidgetItem *)),this, SLOT (ChangeSelectedSimplyRule() ));
	connect(listMediumRules, SIGNAL ( itemClicked ( QListWidgetItem *)),this, SLOT (ChangeSelectedMediumRule() ));
	connect(listExpertRules, SIGNAL ( itemClicked ( QListWidgetItem *)),this, SLOT (ChangeSelectedExpertRule() ));
	connect(listHiddenRules, SIGNAL ( itemClicked ( QListWidgetItem *)),this, SLOT (ChangeSelectedHiddenRule() ));
*/

#ifdef _DEBUG
	printf("RulesDialog::ClearRules END\n");
#endif
      return true;
}



/**************/
/* AddNewRule */
/***************************************************************************/
/* This function is called by mSenseiThread when all rules are found       */
/* It saves the rules, adds them to the lists and types their rules names' */
/***************************************************************************/
int RulesDialog::AddNewRule()
{
	int i=0;
	s_rule rule;
	int nSy = GetNumSymbols (mSudoku->sudoku_board->Board());

	bool TheRuleIsReduntant=false;


	/* Take Rules */
	if ( mSudoku->mSenseiThread->DumpRules()  )
	{
		/* Save Rules */ 
		for (i=0; i < mSudoku->mSenseiThread->rules.size(); i++ )
		{	
			rule = mSudoku->mSenseiThread->rules[i];

			
			nRules = nRules + 1;
//#define X_DEBUG
#ifdef X_DEBUG
	//		printf ("RULES N=%d\n", nRules);
#endif
			if (nRules > 100)
			{
#ifdef X_DEBUG
				printf ("*** MAXIMUN OF RULES N=%d ***\n", nRules);
#endif
				mSudoku->mSenseiThread->Abort();
//				return 0;
			}
			if (nRules > 1000)			return 0;


			/* Add each rule to the QListView */
			QString a;

			 TheRuleIsReduntant=IsRulesRedundant( &rule );

			/* Order rule by type and set the text of the rule */
			switch (rule.type)
			{
				/* Victor Antolin NOTE: 
				   Although I prefer use if->else if structures, I used switch-case
				   structure because it is more clear in that case. 
				   
				   In my opinion if->else if structures usually explain better
				   the code that switch->case, but  it takes up too space in that case */

				case symbols:
			
				// FIRST RULE ( 1A):
				// N symbols are possible in only N cells, so
				// no other symbol can be possible in these cells

				/* Victor Antolin NOTE: 
				  Text in Spanish becuase their syntax is more compex that English */
				a.append(tr("Regla 1A (N%n) ",""  , rule.level)       );

				/* Each group has a different name */
				AddNameOfGroup( rule, &a, mSudoku);


				/* Rest of the text */
				a.append(tr("simbolo(s) ","", SimbolsOfMask (rule.symbols_mask,nSy,tr(" and") ).size()));
				a.append(SimbolsOfMask (rule.symbols_mask,nSy,tr(" and")));
				a.append(tr(" posible(s) unicamente en ","", SimbolsOfMask (rule.symbols_mask,nSy,tr(" and") ).size()));
				a.append(tr("celda(s) ","",SimbolsOfMask (rule.cells_mask,nSy,tr(" and")).size() ));
				a.append(SimbolsOfMask (rule.cells_mask,nSy,tr(" and")));
				a.append(tr(" --> demas simbolos no"));
				break;

			       case cells:

				// FIRST RULE (1 B):
				// N cells have only a set of N different possible
				// symbols, so these are impossible in other cells
			
				a.append(tr("Regla 1B (N%n) ","", rule.level));

				/* Each group has a different name */
				AddNameOfGroup( rule, &a, mSudoku);

				a.append(tr("celda(s) ","", SimbolsOfMask (rule.cells_mask,nSy,tr(" and")).size() ));
				a.append(SimbolsOfMask (rule.cells_mask,nSy,tr(" and")));
				a.append(tr(" solo puede(n) contener ","", SimbolsOfMask (rule.cells_mask,nSy,tr(" and")).size() ));
				a.append(tr("simbolo(s) ","",SimbolsOfMask (rule.symbols_mask,nSy,tr(" and")).size() ));
				a.append(SimbolsOfMask (rule.symbols_mask,nSy,tr(" and")));
				a.append(tr(" --> demas celdas no"));
				{

#ifdef _DEBUG	
	//		QMessageBox A(tr("Regla A"), a , QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
		//		A.exec();
#endif
				}
				
				break;
			
				case inters:
			
				// SECOND RULE:
				// A symbol is only possible in N cells, and they
				// all belong to the intersection with another
				// group, so it will not appear anywhere else in
				// that other group
			
				a.append(tr("Regla 2 (N%n) ","", rule.level));

				AddNameOfGroup( rule, &a, mSudoku);

				a.append(tr("simbolo(s) ", "", SimbolsOfMask (rule.symbols_mask,nSy,tr(" and")).size() ) );
				a.append(SimbolsOfMask (rule.symbols_mask,nSy,tr(" and")));
				a.append(tr(" posible(s) solo en interseccion con "));

				/* Each group has a different name */
				AddNameOfGroup( rule, &a, mSudoku, true);
				
				a.append(tr("--> en otras celdas "));

				/* Each group has a different name */
				switch (GetGroupType(mSudoku->sudoku_board->Board(),
				                     rule.inters_group))
				{
					case GR_BLOCK:

					a.append(tr("del "));
					break;

					default:
					a.append(tr("de "));
				}
				AddNameOfGroup( rule, &a, mSudoku, true);

 				a.append(tr(" no"));
				break;
			
				default:

				/* Error Message. It would not be possible */
				a.append(tr("Imprevisto (raro raro raro...)"));
			}

			/* Order rules by Simplicity  */
			if (TheRuleIsReduntant)
			{
				listHiddenRules->addItem(a);
				mHiddenRules.append( rule );
			}
			else
			{
				if ( rule.level < 2)
				{	
					listSimpleRules->addItem(a);
					mSimpleRules.append( rule );
				}
				else if ( rule.level > 2)
				{	
					listExpertRules->addItem(a);
					mExpertRules.append( rule );
					
				}
				else 
				{	
					listMediumRules->addItem(a);
					mMediumRules.append( rule );
				}
			}

		}
		/* Clear the Thread List */
		mSudoku->mSenseiThread->rules.clear();

	}

#ifdef _DEBUG
//	  	printf("RulesDialog::AddNewRule END\n");
#endif
	return i;
}

/****************************/
/* ChangeSelectedSimplyRule */
/******************************************************************************/
/* This function is called by User when he selects a rule in listSimpleRules  */
/* It calls ChangeSelectedRules.					      */
/******************************************************************************/
bool RulesDialog::ChangeSelectedSimplyRule()
{
	return ChangeSelectedRules( 1);
}

/****************************/
/* ChangeSelectedMediumRule */
/******************************************************************************/
/* This function is called by User when he selects a rule in listMediumRules  */
/* It calls ChangeSelectedRules.					      */
/******************************************************************************/
bool RulesDialog::ChangeSelectedMediumRule()
{
	return ChangeSelectedRules( 2);
}
/****************************/
/* ChangeSelectedExpertRule */
/******************************************************************************/
/* This function is called by User when he selects a rule in listExpertRules  */
/* It calls ChangeSelectedRules.					      */
/******************************************************************************/
bool RulesDialog::ChangeSelectedExpertRule()
{
	return ChangeSelectedRules( 3);
}

/****************************/
/* ChangeSelectedExpertRule */
/******************************************************************************/
/* This function is called by User when he selects a rule in listHiddenRules  */
/* It calls ChangeSelectedRules.					      */
/******************************************************************************/
bool RulesDialog::ChangeSelectedHiddenRule()
{
	return ChangeSelectedRules( 4);
}
/***********************/
/* ChangeSelectedRules */
/******************************************************************************/
/* This function is called by ChangeSelectedXXXXRule                          */
/* It updates all lists and calls SelectPaint to paint the selected rule.     */
/******************************************************************************/
bool RulesDialog::ChangeSelectedRules( int mlist)
{
 #ifdef _DEBUG
 	  	printf("RulesDialog::ChangeSelectedRules 1\n");
 #endif
	s_rule rule;

	/* Security Checks */	
	if (!mSudoku->isPlayerPlaying() ) {	return false; }
	if (!mSudoku->isActiveHelpBox) 	  {	return false; }
	if (!mSudoku->sudoku_board)	  {	return false; }

	// Semaphore necesarry in theory.
	mRulesMutex.lock();
	disconnect(listSimpleRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedSimplyRule() ));
	disconnect(listMediumRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedMediumRule() ));
	disconnect(listExpertRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedExpertRule() ));
	disconnect(listHiddenRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedHiddenRule() ));


 #ifdef _DEBUG
 	  	printf("RulesDialog::ChangeSelectedRules 1.5\n");
 #endif
	if (mlist == 1) // SimpleRuleSelected
	{
#ifdef _DEBUG
 	  	printf("RulesDialog::ChangeSelectedRules 1-1\n");
 #endif
		// Unselect other lists
/*		if (listMediumRules->currentItem() )	listMediumRules->currentItem()->setSelected(false);
		if (listExpertRules->currentItem() )	listExpertRules->currentItem()->setSelected(false);
		if (listHiddenRules->currentItem() )	listHiddenRules->currentItem()->setSelected(false);
*/
//		listSimpleRules->clearSelection();
		listMediumRules->clearSelection();
		listExpertRules->clearSelection();
		listHiddenRules->clearSelection();

		isSimpleRuleActive= true;
		isMediumRuleActive=false;
		isExpertRuleActive=false;
		isHiddenRuleActive=false;

		// Get the rule
		rule = mSimpleRules[ listSimpleRules->row(listSimpleRules->selectedItems()[0] ) ];

	}
	else if(mlist == 2) // MediumRuleSelected
	{
#ifdef _DEBUG
 	  	printf("RulesDialog::ChangeSelectedRules 1-2\n");
 #endif
		// Unselect other lists
/*		if (listSimpleRules->currentItem() )	listSimpleRules->currentItem()->setSelected(false);
		if (listExpertRules->currentItem() )	listExpertRules->currentItem()->setSelected(false);
		if (listHiddenRules->currentItem() )	listHiddenRules->currentItem()->setSelected(false);
*/
		listSimpleRules->clearSelection();
//		listMediumRules->clearSelection();
		listExpertRules->clearSelection();
		listHiddenRules->clearSelection();

		isSimpleRuleActive=false;
		isMediumRuleActive= true;
		isExpertRuleActive=false;
		isHiddenRuleActive=false;

		// Get the rule
		rule = mMediumRules[ listMediumRules->row(listMediumRules->selectedItems()[0] ) ];
	}
	else if (mlist == 3) // ExpertRuleSelected
	{
#ifdef _DEBUG
 	  	printf("RulesDialog::ChangeSelectedRules 1-3\n");
 #endif
		// Unselect other lists
/*		if (listSimpleRules->currentItem() )	listSimpleRules->currentItem()->setSelected(false);
		if (listMediumRules->currentItem() )	listMediumRules->currentItem()->setSelected(false);
		if (listHiddenRules->currentItem() )	listHiddenRules->currentItem()->setSelected(false);
*/

		listSimpleRules->clearSelection();
		listMediumRules->clearSelection();
//		listExpertRules->clearSelection();
		listHiddenRules->clearSelection();

		isSimpleRuleActive=false;
		isMediumRuleActive=false;
		isExpertRuleActive= true;
		isHiddenRuleActive=false;

		// Get the rule
		rule = mExpertRules[ listExpertRules->row(listExpertRules->selectedItems()[0] ) ];
	}
	else if (mlist == 4) // HiddenRuleSelected
	{
#ifdef _DEBUG
 	  	printf("RulesDialog::ChangeSelectedRules 1-4\n");
 #endif
		// Unselect other lists
/*		if (listSimpleRules->currentItem() )	listSimpleRules->currentItem()->setSelected(false);
		if (listMediumRules->currentItem() )	listMediumRules->currentItem()->setSelected(false);
		if (listExpertRules->currentItem() )	listExpertRules->currentItem()->setSelected(false);
*/

		listSimpleRules->clearSelection();
		listMediumRules->clearSelection();
		listExpertRules->clearSelection();
//		listHiddenRules->clearSelection();

		isSimpleRuleActive=false;
		isMediumRuleActive=false;
		isExpertRuleActive=false;
		isHiddenRuleActive=true;

		// Get the rule
		rule = mHiddenRules[ listHiddenRules->row(listHiddenRules->selectedItems()[0] ) ];
	}
	else 
	{
		connect(listSimpleRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedSimplyRule() ));
		connect(listMediumRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedMediumRule() ));
		connect(listExpertRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedExpertRule() ));
		connect(listHiddenRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedHiddenRule() ));
		mRulesMutex.unlock();
		return false;
	}


	// Paint the rule
	SelectPaint ( &rule);
 
 #ifdef _DEBUG
 	  	printf("RulesDialog::ChangeSelectedRules END\n");
 #endif
	connect(listSimpleRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedSimplyRule() ));
	connect(listMediumRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedMediumRule() ));
	connect(listExpertRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedExpertRule() ));
	connect(listHiddenRules, SIGNAL ( itemSelectionChanged()),this, SLOT (ChangeSelectedHiddenRule() ));
	mRulesMutex.unlock();

 	return true;


}

/***************/
/* SelectPaint */
/************************************************************/
/* This fuction calls Paint1_Rule or Paint2_Rule            */
/************************************************************/
bool RulesDialog::SelectPaint(s_rule * rule)
{
	if (rule->type == symbols)
	{
			return Paint1_Rule( rule);
	}
	else if ( rule->type == cells)
	{
			return Paint1_Rule( rule);
	}
	else if ( rule->type == inters )
	{
			return Paint2_Rule( rule);
	}
	else return false;
}


/***************/
/* Paint1_Rule */
/********************************************************/
/* This fuction paints the rule of level 1 in the board */
/********************************************************/
//Old Paint1ARule
bool RulesDialog::Paint1_Rule(s_rule * rule)
{
#ifdef _DEBUG
	  	printf("RulesDialog::Paint1_Rule 1\n");
#endif
	int  j=0, nSy;
	t_board_p pBoard;
	GraphCell * mCell= NULL;

	/* Clear others painted rules in the board */
	mSudoku->ClearListLastColorModifed();	

	/* Paint all cell of the block on a different color */
	pBoard = mSudoku->sudoku_board->Board ();
    nSy = GetNumSymbols (pBoard);
	for (j=0; j < nSy; j++)
	{
		mCell = mSudoku->sudoku_board->
		          GCell (GetCellOfGroup (pBoard, j, rule->group));

		// If is a cell marked by the rule.
		if (rule->cells_mask & ( int (1) << j) )
		{
		
			if (rule->type == symbols ) //  If is rule 1A
			{

//                               0000000000000000000000000XXXXX -> 000000000000000000000000XXXXX
// Inverting rule->symbols_mask  0...sizeof(int) - nSy...010001 -> 0...sizeof(int)- nSy...001110
				int mask = int(1) << nSy;
				mask--;
				mask = mask xor rule->symbols_mask;

				// Paint with a different color 
				mCell->setBrush ( mSudoku->colorRule3color2 );

				// Mark with X the others symbols of the cell
				mCell->UpdatePossUserText( mCell->pTextPoss[0]->font().pixelSize(),
							mCell->nTextPoss,  mask );
			}	
			else    mCell->setBrush ( mSudoku->colorRule2color2 );
		}
		else
		{
			// Paint with the group color 
			mCell->setBrush ( mSudoku->colorRule2color1 );

			if (rule->type == cells ) // Rule 1B
			{
				// Paint with the other color 
				if (mCell->mTextPoss & rule->symbols_mask ) mCell->setBrush ( mSudoku->colorRule2color3 );

				// Mark with X the symbols of others cells group's
				mCell->UpdatePossUserText( mCell->pTextPoss[0]->font().pixelSize(),
							   mCell->nTextPoss, rule->symbols_mask );
			}

		}

		//mCell->update();	
		mSudoku->ListLastColorModifed.append (mCell);
		
	}

	 mCell= NULL;
#ifdef _DEBUG
	  	printf("RulesDialog::Paint1_Rule END\n");
#endif
	return true;
}
// Old Paint1BRule
// bool RulesDialog::Paint1BRule(s_rule * rule)
// {
// 	int  j;
// 	/* Changes when a cell focus changes */
// 
// 	mSudoku->ClearListLastColorModifed();	
// 
// 	/* Paint all cell of the block on a different color */
// 	for (j=0; j < mSudoku->sudoku_board->pGr[ rule->group ].nCl ; j++)
// 	{
// 
// 		if (rule->cells_mask & ( int (1) << j) )
// 		{
// 			mSudoku->sudoku_board->pGr[ rule->group ].ppCl[j]->pGraphCell->setBrush ( mSudoku->colorRule2color2 );
// 		}
// 		else
// 		{
// 			mSudoku->sudoku_board->pGr[ rule->group ].ppCl[j]->pGraphCell->setBrush ( mSudoku->colorRule2color1 );
// 		}
// 		mSudoku->ListLastColorModifed.append (mSudoku->sudoku_board->pGr[ rule->group ].ppCl[j]->pGraphCell);
// 	}
// 	return true;
// }

/***************/
/* Paint2_Rule */
/********************************************************/
/* This fuction paints the rule of level 2 in the board */
/********************************************************/
bool RulesDialog::Paint2_Rule(s_rule * rule)
{

#ifdef _DEBUG
	  	printf("RulesDialog::Paint2_Rule 1\n");
#endif
	bool modify_symbol=true;
	int i=0, j=0, nSy;
	t_board_p pBoard;
	GraphCell * mCell = NULL;

	/* Clear others painted rules in the board */
	mSudoku->ClearListLastColorModifed();	

	/* Paint all cell of the block on a different color */
	pBoard = mSudoku->sudoku_board->Board ();
	nSy = mSudoku->sudoku_board->NumSymbols ();
	for (j=0; j < nSy; j++)
	{

		mCell = mSudoku->sudoku_board->
		          GCell (GetCellOfGroup (pBoard, j, rule->group));

		mCell->setBrush ( mSudoku->colorRule2color1 );
		
		mSudoku->ListLastColorModifed.append (mCell);
		
	}

	/* Paint all cell of the intersected block on a different color */
	for (j=0; j < nSy ; j++)
	{
	        int cell = GetCellOfGroup (pBoard, j, rule->inters_group);
		mCell = mSudoku->sudoku_board->GCell (cell);
		mCell->setBrush ( mSudoku->colorRule3color1 );

		/* Put intersection cells in other group */
		modify_symbol= true;
		for (i=0 ; i < GetNumGroupsOfCell(pBoard,cell); i++)
		{

                        if ( GetGroupOfCell (pBoard, i, cell) == rule->group )
			{
				mCell->setBrush ( mSudoku->colorRule3color2 );
				modify_symbol= false;

			}
			//else mCell->setBrush ( mSudoku->colorRule2color2 );
		}
		if (modify_symbol)
		{
			if (!mCell->FvActivate) mCell->setBrush ( mSudoku->colorRule2color3 );
			mCell->UpdatePossUserText( mCell->pTextPoss[0]->font().pixelSize(),
							mCell->nTextPoss,  rule->symbols_mask );
		}

		mSudoku->ListLastColorModifed.append (mCell);
		
	}

		mCell = NULL;

#ifdef _DEBUG
	  	printf("RulesDialog::Paint1_Rule END\n");
#endif
	return true;
}


/******************/
/* LanguageChange */
/********************************************************/
/* This fuction is called by SudokuView class	        */
/* It changes the language                              */
/********************************************************/
void RulesDialog::LanguageChange()
{
#ifdef _DEBUG
	printf ("RulesDialog::LanguageChange 1\n");
#endif
	retranslateUi(this);
	
	label  ->setText( label  ->text() );
	label_2->setText( label_2->text() );
	label_3->setText( label_3->text() );
	label_4->setText( label_4->text() );

	But_Apply_Rule->setText( But_Apply_Rule->text() );
	But_Show_Rules->setText( But_Show_Rules->text() );
	But_Apply_All ->setText( But_Apply_All ->text() );
	
	if (nRules >1 && mSudoku->sudoku_board && mSudoku->isPlayerPlaying())
		mSudoku->OrderCheckBoard();
#ifdef _DEBUG
	printf ("RulesDialog::LanguageChange end\n");
#endif

	return;

}

/********************/
/* IsRulesRedundant */
/********************************************************/
/* This fuction finds redundant rules		        */
/********************************************************/
bool RulesDialog::IsRulesRedundant( s_rule * rule )
{
	int i=0,j=0,k=0, nSy;
	t_board_p pBoard;
	bool isReduntant= true;
	bool modify_symbol = true;
//	s_rule * rule = NULL;

        nSy = mSudoku->sudoku_board->NumSymbols ();
        pBoard = mSudoku->sudoku_board->Board ();

	isReduntant= false;

	int * pMask = mSudoku->pMaskCellsArray;

#ifdef _XDEBUG


	printf ("SAVE ALL MASK 1 | nCl= %d | sizeof(pMask)=%d \n",mSudoku->sudoku_board->nCl, sizeof(pMask) );
#endif


#ifdef _DEBUG
//	printf ("ENTRING IN THE LOOP 1\n");
#endif
#ifdef _DEBUG
//		printf ("***** START CHECK RULE 1\n");
#endif
		isReduntant=true;

//		if (rule->type == cells ) // if  Rule 1B is Redundant=false
//			isReduntant=false;			

		if (rule->type == symbols || rule->type == cells)
		{

#ifdef _DEBUG
//			printf ("***********************************\n");
//			printf ("*ENTRING IN RULE 1 LOOP 2. i= %d *\n", i);
#endif
			for (j=0; j < nSy ; j++)
			{
				k = GetCellOfGroup (pBoard, j, rule->group);

				if (rule->cells_mask & ( int (1) << j) )
				{
					
					if (rule->type == symbols ) // Rule 1A
					{
		//                               0000000000000000000000000XXXXX -> 000000000000000000000000XXXXX
		// Inverting rule->symbols_mask  0...sizeof(int) - nSy...010001 -> 0...sizeof(int)- nSy...001110
						int mask = int(1) << nSy;
						mask--;
						mask = mask xor rule->symbols_mask;
						//
#ifdef _DEBUG
//						printf ("*pMask[k]= %x  | mask= %x *\n",pMask[k],mask);
#endif
						if (mask & pMask[k])
						{
#ifdef _XDEBUG
							printf ("*pMask[k]= %x  |  CHANGED *\n",pMask[k]);
#endif
							isReduntant=false;
						}
	
						pMask[k] &= ~mask; // Quitar posibilidades eliminadas por la regla.
#ifdef _DEBUG
//						printf ("*pMask[k]= %x  *\n",pMask[k]);
#endif
					}
				}
				else
				{
					if (rule->type == cells ) // Rule 1B
					{
#ifdef _XDEBUG
						printf ("*pMask[k]= %x  | symbols_mask= %x *\n",pMask[k],rule->symbols_mask);
#endif
						if (( rule->symbols_mask & pMask[k]))
						{
#ifdef _XDEBUG
							printf ("*pMask[k]= %x  |  CHANGED *\n",pMask[k]);
#endif
							isReduntant=false;
						}

						pMask[k] &= ~rule->symbols_mask; // Quitar posibilidades eliminadas por la regla.
#ifdef _XDEBUG
						printf ("*pMask[k]= %x  *\n",pMask[k]);
#endif
					}
		
				}
			}
		}
		else if ( rule->type == inters )
		{
#ifdef _DEBUG
//			printf ("ENTRING IN RULE 2 LOOP 2. i= %d\n", i);
#endif
			// Rule 2
			// Same loop than Paint2_Rule.
			for (j=0; j < nSy ; j++)
			{
	                        int cell = GetCellOfGroup (pBoard, j, rule->inters_group);
				/* Put intersection cells in other group */
				k = GetCellOfGroup (pBoard, j, rule->inters_group);
				modify_symbol= true;

				
				for (i=0 ; i < GetNumGroupsOfCell(pBoard,cell); i++)
				{
					if ( GetGroupOfCell (pBoard, i, cell) == rule->group )
					{
						modify_symbol= false;
					}
				}
				if (modify_symbol)
				{
						if (  rule->symbols_mask & pMask[k])
						isReduntant=false;
#ifdef _DEBUG
//						printf ("*pMask[k]= %x  | symbols_mask= %x *\n",pMask[k],rule->symbols_mask);
#endif

						pMask[k] &= ~rule->symbols_mask; // Quitar posibilidades eliminadas por la regla.
#ifdef _DEBUG
//						printf ("*pMask[k]= %x  | CHANGED *\n",pMask[k]);
#endif
				}
			}
		}
//	}
	
	return isReduntant;
}

/************************/
/* CheckButtonApplyRule */
/***********************************************************/
/* This fuction take the selected rule and calls ApplyRule */
/***********************************************************/
void RulesDialog::CheckButtonApplyRule()
{
#ifdef _DEBUG
	printf ("RulesDialog::CheckButtonApplyRule() 1\n");
#endif

	// desactivate Save Movement
	MY_ASSERT (mSudoku->isPlayerPlaying()==true);
	mSudoku->setComputerControl();

	/* Find selected rule and take it */
	if (isSimpleRuleActive)
	{
		if (    listSimpleRules->count() > 0 && (listSimpleRules->currentItem()->isSelected() )  )
		{
			ApplyRule (  mSimpleRules[ listSimpleRules->currentRow() ]  );
		}
	}
	else if (isMediumRuleActive)
	{
		if (    listMediumRules->count() > 0 && (listMediumRules->currentItem()->isSelected() )  )
		{
			ApplyRule (  mMediumRules[ listMediumRules->currentRow() ]    );
		}
	}
	else if (isExpertRuleActive)
	{
		if (    listExpertRules->count() > 0 && (listExpertRules->currentItem()->isSelected() )  )
		{
			ApplyRule (  mExpertRules[ listExpertRules->currentRow() ]  );
		}
	}
	else if (isHiddenRuleActive)
	{
		if (    listHiddenRules->count() > 0 && (listHiddenRules->currentItem()->isSelected() )  )
		{
			ApplyRule (  mHiddenRules[ listHiddenRules->currentRow() ]  );
		}
	}
	else // nothing selected but the button was pressed
	{
	        MY_ASSERT (mSudoku->isPlayerPlaying()==false);
		mSudoku->setPlayerControl();
		return; // If there is not rules selected
	}

	/* Clear Rules and Save Movement */

	ClearRules();
	mSudoku->ClearListLastColorModifed();
	MY_ASSERT (mSudoku->isPlayerPlaying()==false);
	mSudoku->setPlayerControl();
	mSudoku->SaveMovement();
#ifdef _DEBUG
	printf ("RulesDialog::CheckButtonApplyRule() END\n");
#endif
}
/**************/
/* ApplyRule  */
/*******************************************/
/* This fuction apply a rule.              */
/* The loop is similar to paint rules loop */
/*******************************************/
bool RulesDialog::ApplyRule( s_rule rule)
{
#ifdef _DEBUG
	printf ("RulesDialog::ApplyRule()\n");
#endif
	int i=0,j=0,l=0, k=0, m_mask=0, nSy;
	t_board_p pBoard;
	bool modify_symbol= false;
	//s_rule  rule;
	GraphCell * mCell= NULL;

#ifdef _DEBUG
	printf ("RULE\n");
	printf ("type = %d\n", rule.type);
	printf ("level= %d\n", rule.level);
	printf ("group= %d\n", rule.group);
	printf ("inters_group= %d\n", rule.inters_group);
	printf ("cells_mask= %d\n", rule.cells_mask);
	printf ("symbols_mask= %d\n", rule.symbols_mask);
#endif

        pBoard = mSudoku->sudoku_board->Board ();
        nSy = GetNumSymbols (pBoard);

 	if (rule.type == symbols || rule.type == cells)
	{
		// Rule 1
		// Same loop than Paint1_Rule.
		for (j=0; j < nSy ; j++)
		{
			mCell = mSudoku->sudoku_board->GCell (GetCellOfGroup(pBoard,j,rule.group));
			if (rule.cells_mask & ( int (1) << j) )
			{
				if (rule.type == symbols ) // Rule 1A
				{

//                             0000000000000000000000000XXXXX -> 000000000000000000000000XXXXX
// Inverting rulesymbols_mask  0...sizeof(int) - nSy...010001 -> 0...sizeof(int)- nSy...001110
					m_mask = int(1) << nSy;
					m_mask--;
					m_mask = m_mask xor rule.symbols_mask;

					for(l=0; l < nSy; l++)
					{
						if ( m_mask &  ( ::mask(l) ) )
						{
							if (mCell->mTextPoss &  ( ::mask(l) ) )
								mCell->setPossUserValue (l);
						}
					}
				}
			}
			else
			{
				
				if (rule.type == cells ) // Rule 1B
				{
					for(l=0; l < nSy; l++)
					{
						m_mask = rule.symbols_mask;

						if ( m_mask  &  ::mask(l)  )
						{
							//if ( m_mask & mCell->mTextPoss)
							if (mCell->mTextPoss &  ( ::mask(l) ) )
								mCell->setPossUserValue (l);
						}
					}

				}
	
			}
		}
	}
	else if ( rule.type == inters )
	{
		
		// Rule 2
		// Same loop than Paint2_Rule.
		for (j=0; j < nSy; j++)
		{
		        int cell = GetCellOfGroup (pBoard,j,rule.inters_group);
			mCell = mSudoku->sudoku_board->GCell (cell);

			/* Put intersection cells in other group */
			modify_symbol= true;

			for (i=0 ; i < GetNumGroupsOfCell(pBoard,cell); i++)
			{
				if ( GetGroupOfCell (pBoard, i, cell) == rule.group )
				{
					modify_symbol= false;
				}
			}
			if (modify_symbol)
			{
				m_mask = rule.symbols_mask;
				for(k=0; k < nSy; k++)
				{
//					printf ("Rule 2 | m_mask %d | symbols_mask %d | mTextPoss %d \n", m_mask, rule.symbols_mask, mCell->mTextPoss);
					if ( m_mask &  ::mask(k) )
					{
						if (mCell->mTextPoss &  ( ::mask(k) ) )
							mCell->setPossUserValue(k);
					}
				}
			}
		}
	}
	else /* It can not be possible */
	{
		QMessageBox A(tr("Error"), tr("The rule is corrupt \n The rule can not be possible \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
		A.exec();
	}


	mCell= NULL;
#ifdef _DEBUG
	printf ("RulesDialog::ApplyRule END\n");
#endif
	return true;
}

/*****************/
/* ApplyAllRules */
/********************************************************/
/* This fuction select all rules and calls to ApplyRule */
/********************************************************/
bool RulesDialog::ApplyAllRules()
{
#ifdef _DEBUG
	printf ("RulesDialog::ApplyAllRules()\n");
#endif
	int i=0;
 	QList<s_rule> mRules;

	MY_ASSERT (mSudoku->isPlayerPlaying()==true);
       	mSudoku->setComputerControl();

	// Take all rules in one list
 	mRules = mSimpleRules + mMediumRules + mExpertRules;//Hidden rules not necesary

	// Apply all rules loop
 	for ( i = 0; i <  mRules.count(); i++)
 	{
 		ApplyRule ( mRules[i] );
	}
	
	/* Clear and Save Movement */
	mSudoku->ClearListLastColorModifed();	
	ClearRules();
	/*for ( i = 0; i <  mSudoku->sudoku_board->nCl; i++)
 	{
 		mSudoku->sudoku_board->pCl[i].pGraphCell->setBrush(Qt::NoBrush);
	}*/

	MY_ASSERT (mSudoku->isPlayerPlaying()==false);
	mSudoku->setPlayerControl();
	mSudoku->SaveMovement();

#ifdef _DEBUG
	printf ("RulesDialog::ApplyAllRules() END\n");
#endif
	return true;
}

/*****************/
/* ShowHideRules */
/********************************************************/
/* This fuction shows the hiden rules or Hides them     */
/********************************************************/
bool RulesDialog::ShowHideRules()
{
//	int i=0;
	MY_ASSERT (mSudoku->isPlayerPlaying()==true);
	mSudoku->setComputerControl();
	
	if (Show1orHide0) // Show was selected before the click. We must hide all redundant rules
	{
		// Hide Rules
		label_4->setVisible(false);
		listHiddenRules->setVisible(false);
/*
		// Hide rules loops
		for ( i = 0; i <  listSimpleRules->count(); i++)
		{
			if ( listSimpleRules->item(i)->text().startsWith("*")  )
			{
				listSimpleRules->item(i)->setHidden(true);
			}
		}
		
		for ( i = 0; i <  listMediumRules->count(); i++)
		{
			if ( listMediumRules->item(i)->text().startsWith("*")  )
			{
				listMediumRules->item(i)->setHidden(true);
			}
		}
		for ( i = 0; i <  listExpertRules->count(); i++)
		{
			if ( listExpertRules->item(i)->text().startsWith("*")  )
			{
				listExpertRules->item(i)->setHidden(true);
				
			}
		}
*/

		// Update Button and Flag
		But_Show_Rules->setText(tr("Show Redundant Rules"));
		But_Show_Rules->setDown(false);
		Show1orHide0 = false;
		
	}
	else		// Hide was selected before the click-> We must show all redundant rules
	{

		// Show rules loops
		label_4->setVisible(true);
		listHiddenRules->setVisible(true);
/*
		for ( i = 0; i <  listSimpleRules->count(); i++)
		{
			if ( listSimpleRules->item(i)->text().startsWith("*")  )
			{
				listSimpleRules->item(i)->setHidden(false);
				listSimpleRules->addItem( listSimpleRules->takeItem(i));
				mSimpleRules.append( mSimpleRules.takeAt(i));
			}
		}
		for ( i = 0; i <  listMediumRules->count(); i++)
		{
			if ( listMediumRules->item(i)->text().startsWith("*")  )
			{
				listMediumRules->item(i)->setHidden(false);
				listMediumRules->addItem( listMediumRules->takeItem(i));
				mMediumRules.append( mMediumRules.takeAt(i));
			}
		}
		for ( i = 0; i <  listExpertRules->count(); i++)
		{
			if ( listExpertRules->item(i)->text().startsWith("*")  )
			{
				listExpertRules->item(i)->setHidden(false);
				listExpertRules->addItem( listExpertRules->takeItem(i));
				mExpertRules.append( mExpertRules.takeAt(i));
			}
		}
*/
		//	Update Button and Flag
		But_Show_Rules->setText(tr("Hide Redundant Rules"));
		But_Show_Rules->setDown(true);
		Show1orHide0 = true;
	}

	MY_ASSERT (mSudoku->isPlayerPlaying()==false);
	mSudoku->setPlayerControl();
	return true;
}



void RulesDialog::AddNameOfGroup( s_rule rule, QString *a, SudokuView * mSudoku, bool isIterns)
{
	int nSy = mSudoku->sudoku_board->NumSymbols ();
	t_board_p pBoard = mSudoku->sudoku_board->Board ();
	int nmGrids = mSudoku->nmGrids;
	int index = rule.group;
	int numberGroup = rule.group;
	short numberGrids = 1;
	bool flagAddGridText = false;

	if (isIterns)
	{
		index = numberGroup= rule.inters_group;

	}
	if (nmGrids > 1)
	{
	    flagAddGridText = true;
	    if ( numberGroup >= (3*nSy)  ) 
	    {
		numberGrids = (numberGroup / (3*nSy) ) + 1;
		numberGroup = numberGroup % (3*nSy);
	    }
	}

	numberGroup++;

	if (flagAddGridText && ! isIterns) a->append(tr("En la rejilla %n, ","", numberGrids));

	if (! isIterns) 
		a->append(tr("en "));

	switch (GetGroupType(pBoard,index))
	{
		case GR_BLOCK:

		a->append(tr("grupo %n: "  ,""  , numberGroup ));
		break;

		case GR_ROW:

		a->append(tr("fila %n: "  ,""  , numberGroup - nSy));
		break;

		case GR_COL:
		a->append(tr("columna %n: "  ,""  , numberGroup - (2 * nSy)) );
		break;

		default:
		// Error message. It would not be possible
		a->append(tr("any group")); 
	}
	
}

/*****************/
/* retranslateUi */
/********************************************************/
/* This fuction translates all texts in the Dialogs     */
/********************************************************/
void RulesDialog::retranslateUi(QDialog *Rules)
    {
    Rules->setWindowTitle(QApplication::translate("Rules", "Rules", 0));
    label->setText(QApplication::translate("Rules", "&Simple Rules", 0));
    label_2->setText(QApplication::translate("Rules", "A&dvanced Rules", 0));
    label_3->setText(QApplication::translate("Rules", "&Expert Rules", 0));
    label_4->setText(QApplication::translate("Rules", "&Redundant Rules", 0));
    But_Apply_Rule->setText(QApplication::translate("Rules", "Apply Rule", 0));
    But_Apply_All->setText(QApplication::translate("Rules", "Apply All Rules", 0));
    But_Show_Rules->setText(QApplication::translate("Rules","Show Redundant Rules", 0));
    Q_UNUSED(Rules);
}
