/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: dmem.h  * Status: stable * Version: 2.00 * last changes in version: 0.36 		*/
/************************************************************************************************/
/*  This file contains definition of functions to check memory leaks 				*/
/*												*/
/*   The fuction dump_blocks is defined in dmem.cpp						*/
/*												*/
/*   It's called by all files.									*/
/*												*/
/************************************************************************************************/

#ifndef _DMEM_H_
#define _DMEM_H_

// DM_NEW and DM_DEL definitions
//#define _DEBUG
#ifdef _DEBUG
#define DM_NEW(X) anotar_new(X,__FILE__,__FUNCTION__,__LINE__)
#define DM_DEL(X) anotar_delete(X,__FILE__,__FUNCTION__,__LINE__)
#else
#define DM_NEW(X) (X)
#define DM_DEL(X) (X)
#endif

/************/
/* DM_NEW   */
/********************************************************/
/* This fuction mark all selected memory.	        */
/********************************************************/
/* Used only in debug					*/
/********************************************************/
/* Use:							*/
/* Write before new or malloc				*/
/********************************************************/
/* Examples:						*/
/*  int *a;						*/
/*   a = DM_NEW ( new (int ) );				*/
/********************************************************/


/************/
/* DM_DEL   */
/********************************************************/
/* This fuction unmark all liberated memory.	        */
/********************************************************/
/* Used only in debug					*/
/********************************************************/
/* Use:							*/
/* Write after delete or free				*/
/********************************************************/
/* Examples:						*/
/*  int *a;						*/
/*   a = DM_NEW ( new (int ) );				*/
/*   delete( DM_DEL (a ));				*/
/********************************************************/





#include <map>
#include <iostream>

using namespace std;

struct mem_reg
{
  void * p;
  const char * file;
  const char * function;
  int line;

  operator void * () { return p; }
  bool operator< (const mem_reg & x) { return p<x.p; }
};

extern std::map<void *, mem_reg> bloques;


/***********************/
/* DM_NEW = anotar_new */
/*************************************************************/
/* DM_NEW calls this fuction. It marks all selected memory.  */
/*************************************************************/

template<class T>
T anotar_new (T p, const char * file,
                   const char * function,
                   int line)
{
  mem_reg r = { p, file, function, line };
  bloques.insert (pair<void *, mem_reg>(p,r));
  return p;
}

/**************************/
/* DM_DEL = anotar_delete */
/****************************************************************/
/* DM_DEL calls this fuction. It unmarks all liberated memory.  */
/****************************************************************/
template<class T>
T anotar_delete (T p, const char * file,
                      const char * function,
                      int line)
{
  typedef std::map<void *, mem_reg>::iterator iterator;
  iterator it = bloques.find (p);

 if (it==iterator() || it==bloques.end())
  {
   cout.flush ();
   cerr << "Liberando bloque que no ha sido reservado "
         << "(delete en " << file << ":" << function << ":"
         << line << ")" << endl;
   cerr << endl;
  cerr.flush ();
  }
  else
    bloques.erase (it);

  return p;
}

/***************/
/* dump_blocks */
/********************************************************/
/* This fuciton is defined in dmem.cpp                  */
/********************************************************/
/* This fuction check if there are memory leaks.        */
/* It must be the last called fuction                   */
/********************************************************/
/* Used only in debug					*/
/********************************************************/
void dump_blocks ();

#endif
