/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: editor.cpp    * Status: testing * Version: 2.00 * last changes in version: 0.26 	*/
/************************************************************************************************/
/*  This file contains functions of MyEditor    class                                 	  	*/
/*                                   EditorCell  class                                    	*/
/*                                   EditorScene class					  	*/
/*											  	*/
/************************************************************************************************/

#include <QMessageBox>
#include <QtWidgets>
#include <QPointF>
#include "editor.h"
#include "SudokuView.h"

#include "dmem.h"

/*******************/
/* mousePressEvent */
/*********************************************************************************************/
/* This function catchs a mousePressEvent in the Scene. It emits coordinates of pressed cell */
/*********************************************************************************************/
void EditorScene::mousePressEvent ( QGraphicsSceneMouseEvent * event )
{
	int x,y;
	float w_block, h_block;

	w_block = width()  /  N_CELL ;
	h_block = height() / N_CELL ;

	x = int ( event->scenePos().x() / w_block );
	y = int ( event->scenePos().y() / h_block ); 
	
	emit Pressed ( x, y);
}


/***************/
/* Constructor */
/******************************************/
/* This function builds a MyEditor object */
/******************************************/
MyEditor::MyEditor(QWidget *parent, int w, int h): QDialog(parent)
{
	int x,y;
	for (x=0; x < N_CELL; x++)
	{
		for (y =0 ; y < N_CELL; y++)
		{
			ppmEdCells[x][y]= NULL;
		}
	}
	mScene =NULL;
	// build dialog
	setupUi(this, w, h);

}

/***************/
/* Constructor */
/*****************************************************************/
/* This function builds a MyEditor object  and load a board file.*/
/* Used to edit board files                                      */
/*****************************************************************/
MyEditor::MyEditor(QWidget * parent, QString filename): QDialog(parent)
{
	int k,i,j,mw, mh, x,y;
	

	for (x=0; x < N_CELL; x++)
	{
		for (y =0 ; y < N_CELL; y++)
		{
			ppmEdCells[x][y]= NULL;
		}
	}
	mScene =NULL;

	// build dialog
	setupUi(this);
	
	// load board file 
	if ( !LoadFile ( filename, &nmGrids, GridsX, GridsY, &mw, &mh) ) return;
	

	// Show board data on dialog
	FileEdit->setText(filename);
	
	spinBoxWidth ->setValue(mw);
	spinBoxHeight->setValue(mh);
	if (nmGrids >0)
	{
		spinBoxHeight->setEnabled(false);
		spinBoxWidth ->setEnabled(false);
	}
	// Paint used cells.
	for (k=0; k < nmGrids; k++)
	{
		for (i=0; i < mw; i++)
		{
			for (j=0; j < mh; j++)
			{
				ppmEdCells[GridsX[k] + i][GridsY[k] + j]->setBrush( Qt::blue);
			}
		}
	}

}

/***************/
/* Destructor  */
/********************************************/
/* This function destroy a MyEditor object  */
/********************************************/

MyEditor::~MyEditor()
{
#ifdef _DEBUG
	printf ("MyEditor::~MyEditor 1\n");
#endif
	int x,y;


	for (x=0; x < N_CELL; x++)
	{
		for (y =0 ; y < N_CELL; y++)
		{

		#ifdef _DEBUG
			printf ("MyEditor::~MyEditor x=%d, y=%d\n",x,y);
		#endif
			delete DM_DEL (ppmEdCells[x][y] );
		}
	}

#ifdef _DEBUG
	printf ("MyEditor::~MyEditor 2\n");
#endif

	if (mScene)   delete DM_DEL (mScene) ;


#ifdef _DEBUG
	printf ("MyEditor::~MyEditor end\n");
#endif	
}
/****************/
/*  setupUi     */
/*********************************************************************************************/
/* This function creates and show the dialog, makes all connections and calls SetupGraphView */
/*********************************************************************************************/
void MyEditor::setupUi(QDialog * mEditor, int w, int h)
    {
	Ui_Dialog::setupUi(mEditor);

	nmGrids = 0;
	boxselect= false;

//	setLayout(vboxLayout);

	// Set hight and width
	spinBoxHeight->setValue( h );
	spinBoxWidth->setValue( w );
	
	/* CONNECTIONS */
	connect(NewButton   ,SIGNAL(clicked() ),this, SLOT ( NewBox() ) );
	connect(EmptyButton ,SIGNAL(clicked() ),this, SLOT ( Empty () ) );
	connect(DeleteButton,SIGNAL(clicked() ),this, SLOT ( Delete() ) );
	connect(SaveButton  ,SIGNAL(clicked() ),this, SLOT ( Save  () ) );

	SetupGraphView();
}


/*******************/
/*  SetupGraphView */
/********************************************************************/
/* This function sets scene and cells and paints them at first time */
/********************************************************************/
void MyEditor::SetupGraphView()
{
#ifdef _DEBUG
	printf ("MyEditor::SetupGraphView 1\n");
#endif
	int x,y;

	float w_block, h_block;


	// Create scene and setup
	//mScene = new EditorScene ;
	
	mScene = DM_NEW(new EditorScene );
#ifdef _DEBUG
	printf ("MyEditor::SetupGraphView 2\n");
#endif

	mScene->setSceneRect( QRectF(0,0, SCENE_W_SIZE , SCENE_H_SIZE) );
	graphicsView->setScene( mScene );

	// connect Pressed signal with CellPressed slot
	// must do it here because mScene didn't exist before.
	connect ( mScene, SIGNAL(Pressed(int,int)),this, SLOT (CellPressed(int,int)));


	// set size of block
	w_block = mScene->width()  /  N_CELL ;
	h_block = mScene->height() / N_CELL ;

#ifdef _DEBUG
	printf ("MyEditor::SetupGraphView 3\n");
#endif	
	// Create and paint cells
	for (x=0; x < N_CELL; x++)
	{
		for (y =0 ; y < N_CELL; y++)
		{
			 ppmEdCells[x][y]= DM_NEW( new EditorCell) ;

			mScene->addItem ( ppmEdCells[x][y] );

			ppmEdCells[x][y]->setPen( QPen (Qt::black) );
			 //ppmEdCells[x][y]->setBrush( Qt::nobrush);

			ppmEdCells[x][y]->setRect(
				QRectF(
				(x  * w_block ),
				(y  * h_block ),
				w_block,
				h_block) );

			ppmEdCells[x][y]->x = x;
			ppmEdCells[x][y]->y = y;

			//EditorCell * pCell = ppmEdCells[x][y];
			
			ppmEdCells[x][y]->show();
			
		}
	}
#ifdef _DEBUG
	printf ("MyEditor::SetupGraphView end\n");
#endif
}

/**************/
/*  NewBox    */
/**************************************/
/* This function activates a new box  */
/**************************************/
void MyEditor::NewBox()
{
	// Special situations
	if( boxselect==true )	// if you didn't push the new box. remove new box.
	{
		boxselect= false;
		QApplication::restoreOverrideCursor();	 // restore cursor
		return;
	}

	if(nmGrids== GRID_LIMIT) // if you have alredy taken all posible grids
	{
		QMessageBox::warning(this, tr("Sudoku Editor"), tr("Cannot add more Grids"));
	 	return;
	}

	if ( spinBoxWidth ->value() * spinBoxHeight->value() >30 )
	{	QMessageBox::warning(this, tr("Sudoku Editor"), tr("Cannot add the Grid.\n It is too big"));
		return;
	}

	if(nmGrids==0) // if its the first time you take a new box.
	{
		spinBoxHeight->setEnabled(false);
		spinBoxWidth ->setEnabled(false);
	}

	// Normal situation
	boxselect= true;
	QApplication::setOverrideCursor (Qt::CrossCursor); // change cursor to cross cursor.
}



/***********/
/*  Empty  */
/***********************************/
/* This function clears the board  */
/***********************************/
void MyEditor::Empty()
{
	// Clear all cells
	for (int x=0; x < N_CELL; x++)
	{
		for ( int y =0 ; y < N_CELL; y++)
		{
			ppmEdCells[x][y]->setBrush(Qt::NoBrush);
		}
	}
	
	// Clear data and return to initial state.
	nmGrids=0;
	boxselect= false;
	QApplication::restoreOverrideCursor();
	spinBoxHeight->setEnabled(true);
	spinBoxWidth->setEnabled(true);
}


/************/
/*  Delete  */
/***********************************/
/* This function deletes last box  */
/***********************************/
void MyEditor::Delete()
{
	int i,j,k;
	if (nmGrids > 0)
	{
		// Clean selected cells
		for (i=0; i < spinBoxWidth->value(); i++)
		{
			for (j=0; j < spinBoxHeight->value(); j++)
			{
				ppmEdCells[GridsX[nmGrids-1] + i]
                                          [GridsY[nmGrids-1] + j] ->setBrush(Qt::NoBrush);
			}
		}	
		nmGrids--; //delete GridsX[nmGrids] and GridsY[nmGrids] is not necessary. They are garbage.

		// Repaint other cells.
		for(k=0; k < nmGrids; k++)
		{
			for (i=0; i < spinBoxWidth->value(); i++)
			{
				for (j=0; j < spinBoxHeight->value(); j++)
				{
					ppmEdCells[GridsX[k] + i][GridsY[k] + j]->setBrush(Qt::blue);
				}
			}
		}
	}
	// If it's empty enable width and height changes.
	if (nmGrids ==0)
	{
		spinBoxWidth ->setEnabled(true);
		spinBoxHeight->setEnabled(true);
	}
	// Restore cursor and flag
	boxselect= false;
	QApplication::restoreOverrideCursor();
}


/***********/
/*  Reject */
/******************************************************/
/* This function is called when you cancel the dialog */
/* Restore cursor if it has changed                   */
/******************************************************/
void MyEditor::reject ()
{
	QApplication::restoreOverrideCursor();
	QDialog::reject();
}

/****************/
/*  CellPressed */
/****************************************************/
/* This function is called when you push a new box  */
/* It changes data and paint the new box            */
/****************************************************/
void MyEditor::CellPressed(  int x , int y)
{
	int i=0, j=0;

	// Check if all is good.
        // If only Pressed signal calls it, there isn't problems.

	if (boxselect==false)	return;
	if (nmGrids== GRID_LIMIT) 
	{
		QApplication::restoreOverrideCursor();
		return;
	}

	// Add data
	GridsX[nmGrids]=x;
	GridsY[nmGrids]=y;

	// Paint new cells
	for (i=0; i < spinBoxWidth->value(); i++)
	{
		for (j=0; j < spinBoxHeight->value(); j++)
		{
			ppmEdCells[x + i][y + j]->setBrush( Qt::blue);
		}
	}

	nmGrids++;

	// Restore cursor.
	QApplication::restoreOverrideCursor();
	boxselect=false;
}

/************/
/*  Save    */
/****************************************************************/
/* this fuction saves all data in a board file and close dialog */
/****************************************************************/
bool MyEditor::Save()
{

	int i=0, x=0, y=0, maxCellsInBoardX=0, maxCellsInBoardY=0;

	// Check if there is a file name
	if ( FileEdit->text().size() <1 )
	{	QMessageBox::warning(this, tr("Sudoku Editor"), tr("A File Name is needed."));
		return false;
	}
	// Check if there is one or more grids
	if ( nmGrids < 1 )
	{	QMessageBox::warning(this, tr("Sudoku Editor"), tr("This Board is Empty. \nYou cann't play with it. Please add grids one or more grids"));
		return false;
	}

	// Check if there board exceds 128x128 cells

	x= GridsX[0];
	y= GridsY[0];
	for (i=1; i < nmGrids; i++)
	{
		if ( GridsX[i] > x ) x = GridsX[i];
		if ( GridsY[i] > y ) y = GridsY[i];
	}

	// x = maximun of GridsX elements;
	// y = maximun of GridsY elements;
	maxCellsInBoardX = int(x + spinBoxWidth ->value() ) *  spinBoxHeight->value();
	maxCellsInBoardY = int(y + spinBoxHeight->value() ) *  spinBoxWidth ->value();

	if ( maxCellsInBoardX > 127 || maxCellsInBoardY > 127 )
	{
		QString ErrorMessage;
		ErrorMessage.append(tr("This Board is too big. \n The maximum of cells is 128 x 128\n") );
		ErrorMessage.append(tr("Your board has %n","",maxCellsInBoardX) );
		ErrorMessage.append(tr(" x  %n \n","",maxCellsInBoardY) );
		QMessageBox::warning(this, tr("Sudoku Editor"),ErrorMessage);
		return false;
	}

	// Open file with selected file name
	QFile file (QDir::currentPath() + "/board/" + FileEdit->text());
	if (!file.open(QIODevice::WriteOnly))
	{
		QMessageBox::warning(this, tr("Sudoku Editor"), tr("Cannot write file %1:\n%2.").arg(file.fileName()).arg(file.errorString()));
		return false;
	}

	// Change cursor to wait cursor
	QApplication::setOverrideCursor (Qt::WaitCursor);
	
	QDataStream out(&file);
	out.setVersion (QDataStream::Qt_4_2);


	// Save Data;

	out << quint32( EDIT_MAGIC_NUMBER );

	out << nmGrids;
	for ( i=0; i < nmGrids; i++)
	{
		out << quint32 (i);
		out << GridsX[i];
		out << GridsY[i];
	}
	out << spinBoxWidth->value();
	out << spinBoxHeight->value();
	

	// Restore cursor
	QApplication::restoreOverrideCursor();

	// Close dialog
	emit reject();
	return true;
}
/****************/
/*  LoadFile    */
/******************************************************/
/* this fuction load a board file and return all data */
/******************************************************/
bool MyEditor::LoadFile( QString filename, int * Grids, int *X, int *Y, int * mw, int *mh )
{
	int i=0;
	int aux=0;

	// Check if there is a file name
	if ( filename.size() <1 )
	{	QMessageBox::warning(this, tr("Sudoku Editor"), tr("A File Name is needed."));
		return false;
	}
	
	// Open file with selected file name 
	QFile file (QDir::currentPath() + "/board/" + filename);
	if (!file.open(QIODevice::ReadWrite))
	{
		QMessageBox::warning(this, tr("Sudoku Editor"), tr("Cannot write file %1:\n%2.").arg(file.fileName()).arg(file.errorString()));
		return false;
	}
	
	// Change cursor to wait cursor
	QApplication::setOverrideCursor (Qt::WaitCursor);
	
	QDataStream in(&file);
	in.setVersion (QDataStream::Qt_4_2);

	// Check Magic Number
	in >> i;
	if ( i != EDIT_MAGIC_NUMBER )
	{
		QMessageBox::warning(this, tr("Sudoku Editor"), tr("Data corrupt loading file %1:\n%2.").arg(file.fileName()).arg(file.errorString()));
		QApplication::restoreOverrideCursor();
		return false;
	}
	
	// Load data 
	in >> *Grids;
	for ( i=0; i < *Grids; i++)
	{
		in >> aux;
		in >> X[i];
		in >> Y[i];
		
	}

	in >> *mw;
	in >> *mh;

	// Restore cursor
	QApplication::restoreOverrideCursor();
	return true;
}

