/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: editor.h    * Status: testing * Version: 2.00 * last changes in version: 0.26 	*/
/************************************************************************************************/
/*  This file contains definition of MyEditor    class                                  	*/
/*                                   EditorCell  class                                          */
/*                                   EditorScene class						*/
/*   *MyEditor*											*/
/*   This dialog is defined in Deditor.ui                                       		*/
/*   The class functions are defined in editor.cpp                              		*/
/*												*/
/*   It's called by MyNewBoard class defined in newtable.h, newtable.cpp, Dnewtable.ui	        */
/*												*/
/*   *EditorCell*										*/
/*   These Objects are created  by MyEditor class						*/
/*												*/
/*   *EditorScene*										*/
/*   This Object is created  by MyEditor class							*/
/************************************************************************************************/

#ifndef _EDITOR_H_
#define _EDITOR_H_


#include <QDialog>
#include <QObject>
#include <QGraphicsRectItem>
#include <QGraphicsScene>

#include "ui_Deditor.h"

/* Define Magic Number in All Edit Files*/
#define EDIT_MAGIC_NUMBER 0xE0D0107  


/* Define Size of Scene */

/* NOTE: If you like to do bigger sudokus change them */
/* But Probably you don't need to change it           */
#define N_CELL 25
#define SCENE_W_SIZE 250
#define SCENE_H_SIZE 250

/* Don't Change it. This parameter must be same in other files */
/* Grid Limit can be upgrade in future versions                */
#define GRID_LIMIT 15


/********************************************************************************/
/* Class EditorCell                                                             */
/********************************************************************************/
/* This class is a simple derivated QGraphicsRectItem class.                    */
/* Finally no more funcionality added                                           */
/*                                                                              */
/********************************************************************************/

class EditorCell : public QGraphicsRectItem
{
	
public:
	EditorCell ( QGraphicsItem * parent=0 ) : QGraphicsRectItem::QGraphicsRectItem ( parent ) {}

	int x,y;

	//void mousePressEvent ( QGraphicsSceneMouseEvent * event );
};

/********************************************************************************/
/* Class EditorScene                                                            */
/********************************************************************************/
/* This class is a simple derivated QGraphicsScene class                        */
/* Added mousePressEvent funcionality                                           */
/*										*/
/*  It capture mousePressed Event and send it to MyEditor                       */
/********************************************************************************/

class EditorScene : public QGraphicsScene
{

	Q_OBJECT


signals:

	void Pressed ( int x, int y); // Send coordinates of cell pressed
protected:
		
	void mousePressEvent ( QGraphicsSceneMouseEvent * event ); // Catch mousePressEvent
	


};


/********************************************************************************/
/* Class MyEditor                                                               */
/********************************************************************************/
/* This class creates and show a dialog when the user can select and build      */
/* new boards.He can modify a lot of board parameters, and save his board in a  */
/* board file                                                                   */
/********************************************************************************/

class MyEditor : public QDialog, Ui_Dialog
{
	   Q_OBJECT

public:
	
	EditorScene * mScene;			 /* Scene which show editor cells and future board */
	EditorCell * ppmEdCells[N_CELL][N_CELL]; /* Array 2d of editor cells                       */


	MyEditor(QWidget *, int w=3, int h=3);      /* Constructor. Build a new empty board */
	MyEditor(QWidget *, QString filename);      /* Constructor. Edit a board */

	~MyEditor(); /* Destructor. Destroy the Editor object */

	void setupUi(QDialog * mEditor, int w=3, int h=3); /* Setup the dialog */

	bool LoadFile( QString filename, int * Grids, int *X, int *Y, int * mw, int *mh ); /* Function which load a board file and give all boards parameters */


private:
	void SetupGraphView();
	
	
	bool boxselect;        /* Flag. True when new box is selected */

	int GridsX[GRID_LIMIT]; /* Array with Position X of Grids        */
	int GridsY[GRID_LIMIT]; /* Array with Position Y of Grids        */
	int nmGrids;            /* Number of Grids which the board has		 */
			        /* It is number of elements of GridsX and GridsY */



public slots:

	void NewBox();
	void Empty();
	void Delete();
	bool Save();
	void reject ();
	void CellPressed( int x , int y);

};
#endif
