/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: enlargewindow.cpp * Status: stable * Version: 2.00 * last changes in version:0.36	*/
/************************************************************************************************/
/*  This file contains fuctions EnlargeWindow class						*/
/*												*/
/*   The class are defined in SudokuView.h							*/
/*												*/
/*   It's called by SudokuView calls defined in SudokuView.h.					*/
/*												*/
/*   This class is a zoom of the sudoku scene. It can catch mouse and key events		*/
/************************************************************************************************/

#include "SudokuView.h"
#include "SudokuBoard.h"
#include "mask.h"
#include "mainwindow.h"
#include "dmem.h"

#define WIDTH  350
#define HEIGTH 370

/***************/
/* Constructor */
/***************/
EnlargeWindow::EnlargeWindow (SudokuView * mSudokuView) :  
        mButtons (NULL),
   fFinalValue (false) ,
   mSudoku(mSudokuView),
           mScale   (5),
	   nButtons (0) 
{
#ifdef _DEBUG
	printf ("EnlargeWindow::EnlargeWindow 1\n");
#endif

	// Setup
	setupUi(this);

	//Set Typical attributes
	setAttribute(Qt::WA_QuitOnClose,false);
	setAttribute(Qt::WA_DeleteOnClose,false);
	setWindowFlags(Qt::WindowShadeButtonHint);

	// Define View and button 
	mView = DM_NEW (new QGraphicsView (centralwidget) );
	mSelect =  DM_NEW ( new QToolButton(this) );
	mSelect->setText(tr("Possibility"));
	/*fFinalValue = false;
	nButtons=0;
	mButtons=NULL;*/
	

	//Add button to the ToolBar
	mToolBar->addWidget(mSelect);
	//mView->setDragMode (QGraphicsView::RubberBandDrag);

	// Connect button
	connect (mSelect, SIGNAL (clicked()), this, SLOT (ChangeSelect()));
	

	// Setup View
	mView->setScene(mSudoku->mScene);
	mView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	mView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

//	mScale = 5;
	mView->scale ( mScale , mScale );

	resize( WIDTH, HEIGTH );
//	setFixedSize (WIDTH, HEIGTH );	


	
/*
	mButtons = (mToolButton **) DM_NEW (malloc ( sizeof (mToolButton*) * mSudoku->sudoku_board->nSy) );
	if (mButtons== NULL) return;

	for (int i=0; i < mSudoku->sudoku_board->nSy ; i++ )
	{
		mButtons[i] = DM_NEW (new mToolButton(this) );
		mButtons[i]->setText(QString (simbols(i)));
		mToolBar->addWidget(mButtons[i]);

		connect (mButtons[i], SIGNAL (clicked()), this, SLOT (PressButton()));
		mButtons[i]->show();
	}

	mToolBar->setAllowedAreas(Qt::TopToolBarArea
        		  | Qt::BottomToolBarArea);
	addToolBar (mToolBar);

	mView->centerOn( mCellFocus->scenePos().x() , mCellFocus->scenePos().y() - (40 * WIDTH_GRID_BORDER));
	mView->setSceneRect(mCellFocus->rect());

	printf ("CreateEnlargeWindow 3\n");

	update();*/
	//mToolBar->update();
	//mToolBar->show();
	
	//toolButton->hide();
	//mView->resize( WIDTH , HEIGTH + WIDTH_H_BORDER);
	
	//mView->ensureVisible(mSudoku->sudoku_board->pCl[0].pGraphCell->rect());

	// Set this window close when other windows close.
#ifdef _DEBUG
	printf ("EnlargeWindow::EnlargeWindow end\n");
#endif
}

/***************/
/* Destructor  */
/***************/
EnlargeWindow::~EnlargeWindow()
{
#ifdef _DEBUG
	printf ("EnlargeWindow::~EnlargeWindow 1\n");
#endif
	for (int i=0; i < nButtons ; i++ )
	{
		delete DM_DEL ( mButtons[i] );
	}
	delete [] (DM_DEL (mButtons) );
	mButtons=NULL;

	mSudoku->mEnlargeWindow = NULL;

	delete DM_DEL (mSelect);
	mSelect=NULL;

	delete DM_DEL (mView);
	mView=NULL;
#ifdef _DEBUG
	printf ("EnlargeWindow::~EnlargeWindow end\n");
#endif
}
/***************/
/* resizeEvent */
/***************/
void EnlargeWindow::resizeEvent (QResizeEvent *event)
{
#ifdef _DEBUG
	printf ("EnlargeWindow::resizeEvent 1\n");
#endif
	float width_window=0;
	float height_window=0;
	
	width_window  = width();
	height_window = height();
	
	mView->resize( (int) width_window  , (int) height_window +  mToolBar->height() );
	
	mView->centerOn( mCellFocus->scenePos().x() , mCellFocus->scenePos().y() - (40 * WIDTH_GRID_BORDER));
	mView->setSceneRect(mCellFocus->rect());

#ifdef _DEBUG
	printf ("EnlargeWindow::resizeEvent end\n");
#endif
}

/*****************/
/* centralWindow */
/*****************/
void EnlargeWindow::centerWindow(int cell)
{
#ifdef _DEBUG
	printf ("EnlargeWindow::centralWindow 1\n");
#endif
	mCellFocus = mSudoku->sudoku_board->GCell (cell);
	
	mView->centerOn( mCellFocus->scenePos().x() , mCellFocus->scenePos().y() );
	mView->setSceneRect(mCellFocus->rect().x(), mCellFocus->rect().y() , mCellFocus->rect().width() + (1 * WIDTH_GRID_BORDER), mCellFocus->rect().height() + ( WIDTH_GRID_BORDER));

}

/*****************/
/* keyPressEvent */
/*****************/
void EnlargeWindow::keyPressEvent (QKeyEvent * event)
{
#ifdef _DEBUG
	printf ("EnlargeWindow::keyPressEvent 1\n");
#endif
	if(mSudoku->pMainWindow) 
		((MyMainWindow *) mSudoku->pMainWindow )->keyPressEvent (event);
	
}

/****************/
/* ChangeSelect */
/****************/
void EnlargeWindow::ChangeSelect()
{
#ifdef _DEBUG
	printf ("EnlargeWindow::ChangeSelect 1\n");
#endif
	if (fFinalValue)
	{
		mSelect->setText(tr("Possibility"));
		fFinalValue = false;
		mSelect->setDown(false);
		
	}	
	else	
	{
		mSelect->setText(tr("Final Value"));
		fFinalValue = true;
		mSelect->setDown(true);
	}
	mCellFocus->setFocus();
}

/***************/
/* PressButton */
/***************/
void EnlargeWindow::PressButton()
{
#ifdef _DEBUG
	printf ("EnlargeWindow::PressButton 1\n");
#endif
	for (int i=0; i < mSudoku->sudoku_board->NumSymbols() ; i++)
	{
		if (mButtons[i]->checkPressed())
		{
			if (fFinalValue)
				mCellFocus->setFinalUserValue( i);
			else    mCellFocus->setPossUserValue( i);
		}
	}
	mCellFocus->setFocus();
}

/******************/
/* LanguageChange */
/******************/
bool EnlargeWindow::LanguageChange()
{
#ifdef _DEBUG
	printf ("EnlargeWindow::LanguageChange 1\n");
#endif

	retranslateUi(this);
	if (fFinalValue)
	{
		mSelect->setText(tr("Final Value"));
	}
	else
	{
		mSelect->setText(tr("Possibility"));
	}
	return true;
#ifdef _DEBUG
	printf ("EnlargeWindow::LanguageChange end\n");
#endif
}

/****************/
/* ChangeSudoku */
/****************/
void EnlargeWindow::ChangeSudoku()
{
#ifdef _DEBUG
	printf ("EnlargeWindow::ChangeSudoku 1.0\n");
#endif

    int nSy = 0;
    t_board_p pBoard = NULL;
    
	if(mSudoku->sudoku_board)
	{
	    nSy = mSudoku->sudoku_board->NumSymbols ();
	    pBoard = mSudoku->sudoku_board->Board ();
	    
#ifdef _DEBUG
		printf ("EnlargeWindow::En IF ChangeSudoku 1.1\n");
#endif
        mCellFocus = mSudoku->sudoku_board->GCell (
                         mSudoku->sudoku_board->NumSymbols()+1 );
		mCellFocus->setFocus();
#ifdef _DEBUG
		printf ("EnlargeWindow::Out IF ChangeSudoku 1.1\n");
#endif
	}
//	else return;
#ifdef _DEBUG
	printf ("EnlargeWindow::ChangeSudoku 2\n");
#endif
	if (mButtons!= NULL)
	{
		for (int i=0; i < nButtons ; i++ )
		{
			delete DM_DEL (mButtons[i]);
		}
		delete [] (DM_DEL(mButtons) );
		mButtons=NULL;
	}
#ifdef _DEBUG
	printf ("EnlargeWindow::ChangeSudoku 2.5\n");
#endif

	nButtons = nSy;
#ifdef _DEBUG
	printf ("EnlargeWindow::ChangeSudoku 3\n");
#endif
	//mButtons = (mToolButton **) DM_NEW (malloc ( sizeof (mToolButton*) * mSudoku->sudoku_board->nSy) );
	mButtons = DM_NEW ( new mToolButton*[nSy]);
	
	for (int i=0; i < nSy ; i++ )
	{
		mButtons[i] = DM_NEW (new mToolButton(this));
		mButtons[i]->setText(QString (simbols(i)));
		mToolBar->addWidget(mButtons[i]);

		connect (mButtons[i], SIGNAL (clicked()), this, SLOT (PressButton()));
		mButtons[i]->show();
	}

	mToolBar->setAllowedAreas(Qt::TopToolBarArea
        		  | Qt::BottomToolBarArea);
	addToolBar (mToolBar);

	mView->centerOn( mCellFocus->scenePos().x() , mCellFocus->scenePos().y() - (40 * WIDTH_GRID_BORDER));
	mView->setSceneRect(mCellFocus->rect());


	update();

	#ifdef _DEBUG
		printf ("EnlargeWindow::ChangeSudoku end\n");
	#endif
}

