/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: graphics.cpp * Status: stable * Version: 2.00 * last changes in version: 0.36 	*/
/************************************************************************************************/
/*  This file contains fuctions GraphCell class							*/
/*												*/
/*   The class are defined in sudoku-pr.h							*/
/*												*/
/*   It's called by t_cell struct defined in sudoku-pr.h.					*/
/*												*/
/*   This class links the GUI classes and objects with the board structure created in 		*/
/*   Sudoku Sensei v1.00.									*/
/************************************************************************************************/


//#define _DEBUG
#include "graphics.h" 
#include <QSettings>
#include <QtWidgets>
#include "sudoku-pub.h"
#include "mask.h"
#include "SudokuView.h"
#include "SudokuBoard.h"
#include "math.h"
#include "mainwindow.h"
#include "dmem.h"
#include "mask.h"


/** CELLS FUNCTIONS **/
/***************/
/* Constructor */
/***************/
GraphCell::GraphCell ( QGraphicsItem * parent )
 :  QGraphicsRectItem (parent),
	            pTextPoss (NULL ),     // Fast varibables initialization 
		    nTextPoss ( 0   ),
		    mTextPoss ( 0   ),
		    canChange (true ),
		    FvActivate(false)
{
        board = NULL;
	nTextPoss=0;
	setFlags ( QGraphicsItem::ItemIsFocusable );// Or ( QGraphicsItem::ItemIsSelectable );
}

/***************/
/* Destructor  */
/***************/
GraphCell::~GraphCell()
{
	if (pTextPoss)
	{
		for (int i=0; i < nTextPoss; i++)
			delete DM_DEL (pTextPoss[i]);
		delete DM_DEL (pTextPoss);
	}
}





void GraphCell::mousePressEvent (QGraphicsSceneMouseEvent * event)
{
#ifdef _DEBUG
	printf("GraphCell::mousePressEvent 1\n");
#endif
	setFocus();
	SudokuView * p = board->View ();
	if (p->isPlayerPlaying()) p->ActiveHelpBoxColor();

	/* TODO: If Help mode is activate do more actions*/

#ifdef _DEBUG
	printf("* GraphCell Values \n");
	printf("* FvActivate= %d \n", FvActivate);
	printf("* pCell-> = %d \n", pCell->FV);
	
#endif

}

void GraphCell::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event )
{
#ifdef _DEBUG
	printf("GraphCell::mouseDoubleClickEvent 1\n");
#endif
/*	if (canChange)
	{
		if (FvActivate)
		{
			FvActivate =  0;
	
			for ( int h=0; h < pCell->ppGr[0][0].pBoard->hb; h++ )
					pTextPoss[h].show();
			TextFinalValue.hide();

			//TODO: unset Symbols in cell
			
		}
	
	}
*/
	// Activate and raise the EnlargeWindow;
	// TODO: Test it in MSwindows. QT Help is not clear about it.
	SudokuView * p = board->View ();
	if (p->mEnlargeWindow)
	{
		p->ShowEnlargeWindow();
	}		
	UpdateCell();
}

/* FocusInEvent */
/* Set the colour of Selected Cell */
void GraphCell::focusInEvent ( QFocusEvent * event )
{
#ifdef _DEBUG
	printf("GraphCell::focusInEvent 1\n");
#endif
	QPen myPen;
	SudokuView * p = board->View ();

	myPen.setColor( p->curColor);
	myPen.setWidth(3);

	setZValue(5);
	//printf ("Focus IN\n");
	setPen(myPen);

	if (p->mEnlargeWindow)
	{
		( (EnlargeWindow *)p->mEnlargeWindow ) ->
		centerWindow (board->CellNumber(this));
	}

	event->accept();
}

/* FocusOutEvent */
/* Set the colour of Unselected Cell */
void GraphCell::focusOutEvent ( QFocusEvent * event )
{
#ifdef _DEBUG
	printf("GraphCell::focusOutEvent 1\n");
#endif
	QPen myPen;
	
	SudokuView * p = board->View ();
	myPen.setColor( p->colorCell);
	myPen.setWidth(1);

	//printf ("Focus OUT\n");
	setZValue(0);
	setPen(myPen);

	event->accept();
}

void GraphCell::keyPressEvent (QKeyEvent * event)
{
#ifdef _DEBUG
	printf("GraphCell::keyPressEvent 1\n");
#endif
	int ret= -1;
	int n, x, y;
	t_board_p pBoard;
	//bool isValue=false;

	/* Clear Marked errors if help box is active */
	SudokuView * p = board->View ();
	pBoard = board->Board ();
	n = board->CellNumber (this);
	x = GetCellX (pBoard, n);
	y = GetCellY (pBoard, n);
	
	if (p->isActiveHelpBox)
	{
		p->ActiveHelpBoxColor();
	}

	/* Setup Shift */
	if ( event->key() == Qt::Key_Shift )
	{
		event->ignore ();
		return;
	}
	if ( event->key() == Qt::Key_Shift )
	{
		event->ignore ();
		return;
	}

	/*********************/
	/* Change Cell  Keys */
	/*********************/
	if ( event->key() == Qt::Key_Up)
	{
		
		/* Set Focus Upper Cell */          
		ret = GetCellNumber (pBoard, x, y-1);
	}
	
	else if ( event->key() == Qt::Key_Down)
	{
		
		/* Set Focus Upper Cell */          
		ret = GetCellNumber (pBoard, x, y+1);
	}

	else if ( event->key() == Qt::Key_Left)
	{
		
		/* Set Focus Upper Cell */
		ret = GetCellNumber (pBoard, x-1, y);
	}
	else if ( event->key() == Qt::Key_Right)
	{
		
		/* Set Focus Upper Cell */     
		ret = GetCellNumber (pBoard, x+1, y);
	}

	if ( ret > -1)	/* Test if Cell exists */
	{
		board->GCell (ret) -> setFocus ();
                SudokuView * p = board->View ();
		if (p->isPlayerPlaying()) p->ActiveHelpBoxColor();

		event->ignore ();
		return;
	}

	/*******************************/
	/*  End Test Change Cell  Keys */
	/*******************************/

	/*******************************/
	/*  Undo and Remove   Keys     */
	/*******************************/
	if (event->modifiers() & Qt::ControlModifier)
	{
		if (event->key() == Qt::Key_Z)
		{
			// MainWindow will catch that key.
			//SudokuView * p = (SudokuView *) pCell->ppGr[0][0].pBoard->parent;
			//p->ctrlZPress();

			event->ignore ();	

			return;
		}
		else if (event->key() == Qt::Key_X)
		{
			// MainWindow will catch that key.
			//SudokuView * p = (SudokuView *) pCell->ppGr[0][0].pBoard->parent;
			//p->ctrlXPress();
		
			event->ignore ();
			return;
		}

	}



	/***********************/
	/* Test Value Keys     */
	/***********************/


	/*********************************************************************************/
	/* In this form for avoid problems with CTRL and SHIFT with NUMBER and LETTER    */
	/*										 */
	/* Not Switch-Case used for avoid problems in overloaded == function		 */
	/*********************************************************************************/

	int i, mPressSimbol = -1;
	const static int symbols_keys[] = SYMBOLS_KEYS;
	
	for (i=0; i<(int)(sizeof(symbols_keys)/
	                  sizeof(*symbols_keys)); i++)
	  if (event->key() == symbols_keys[i])
	  {
	    mPressSimbol = i;
	    break;
	  }

	/**********************************/
	/*  Test if Simbols are valid     */
	/**********************************/

	//printf ("Symbols : %d",pCell->ppGr[0][0].pBoard->nSy );
	if (event->modifiers() & Qt::ControlModifier)
	{
		if ( -1 < mPressSimbol && mPressSimbol <  board->NumSymbols()  )
		{
			/* Put FV Value mPressSimbol*/
			
			setFinalUserValue( mPressSimbol );
			event->accept();
		}
	}
	else	
	{
		if ( -1 < mPressSimbol && mPressSimbol  <  board->NumSymbols() )
			/* Put possible Value mPressSimbol*/ 
		{	
			setPossUserValue( mPressSimbol);
			event->accept();
		}
	}

	event->ignore ();
	//QGraphicsRectItem::keyPressEvent( event);
	//else printf("Pulsó  %d\n",mPressSimbol );
#ifdef _DEBUG
	printf("GraphCell::keyPressEvent END \n");
#endif
}


void GraphCell::paint ( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{	
#ifdef _DEBUG
//	printf("GraphCell::paint 1\n");
#endif
/*	QPointF TextPos = pos();
	QPen myPen;
	myPen.setColor(Qt::red);
	TextPos.setX( TextPos.x() + 2 );*/
	/*if (!FvActivate)
	{
		pTextPoss.show();
	}*/

	QGraphicsRectItem::paint ( painter, option, widget);
#ifdef _DEBUG
//	printf("GraphCell::paint end\n");
#endif
}

void GraphCell:: setFinalUserValue (int value)
{
        int this_cell, cell, group, nSy, nGr;
        t_board_p pBoard;
#ifdef _DEBUG
	printf("GraphCell::setFinalUserValue 1\n");
#endif
        nSy = board->NumSymbols ();
        pBoard = board->Board ();
        this_cell = board->CellNumber (this);
        nGr = GetNumGroupsOfCell (pBoard, this_cell);
        
	if ( canChange )
	{
		if (!FvActivate)
		{
	                SudokuView * p = board->View ();

		
			bool WrongValue = false;
			int i,j=0;


			for (i=0; i < nGr && !WrongValue; i++)
			{
			        group = GetGroupOfCell (pBoard, i, this_cell);
			                                
				for (j=0; j < nSy && !WrongValue ; j++)
				{
				        cell = GetCellOfGroup (pBoard, j, group);
				        
					if ( board->GCell(cell)->FvActivate &&
					     GetSymbolOfCell(pBoard,cell) == value )
					{
						WrongValue = true;
						break;
					}
				}
			}		

#ifdef _DEBUG
	printf("GraphCell::setFinalUserValue Checking symbol\n");
#endif
			//if ( !SetSymbolInCell(pBoard, value, board->CellNumber (this) ) )
			if (WrongValue)
			{

				#ifdef _DEBUG			
						printf ("ERROR: SET SYMBOL NO VALID\n");
						printf ("cambio mask %x . FvActve =%d\n", mTextPoss, FvActivate);
						printf ("mPV         %x . canChange=%d\n", GetPossValuesOfCell(pBoard,this_cell),canChange );
						printf ("****************************\n");
				#endif
				if (p->isActiveHelpBox)
				{
					/* Clear Marked errors if help box is active */
				//	p->ActiveHelpBoxColor();

					#ifdef _DEBUG			
						printf ("* SHOWING THE ERROR        *\n");
						printf ("****************************\n");
					#endif
		
					// Error the symbol is not possible
					
					// Find the postcondition to give me that error
					int i,j=0;
	
					for (i=0; i < nGr; i++)
					{
		                	        group = GetGroupOfCell (pBoard, i, this_cell);
		                	        
						for (j=0; j < nSy ; j++)
						{
			                	        cell = GetCellOfGroup (pBoard, j, group);
							if ( board->GCell(cell)->FvActivate &&
				                	     GetSymbolOfCell(pBoard,cell) == value)
							{
								board->GCell(cell)->setBrush(p->colorRule1color1);
								board->GCell(cell)->show();
								// Add in proyect 39b
								p->ListLastColorModifed.append (board->GCell(cell));
#ifdef _DEBUG
	printf("GraphCell::setFinalUserValue FOUND\n");
//	sleep(1);
#endif
							}
						}
					}

					setBrush(p->colorRule1color2);
					// Add in proyect 39b
					p->ListLastColorModifed.append (this);
#ifdef _DEBUG
	printf("GraphCell::setFinalUserValue 3\n");
//	sleep(1);
#endif
					return;
				}
				else
				{
					#ifdef _DEBUG			
						printf ("* FORCING THE SETTING      *\n");
						printf ("****************************\n");
					#endif
					p->NoPlayerMovement = false;
					p->SudokuHasErrors();
					/*SetSymbolInCell(pBoard, value, board->CellNumber (this), true );*/
				}
			}
		ForceSymbolInCell (pBoard, value, this_cell);
		FvActivate =  1;

#ifdef _DEBUG
	printf("GraphCell::setFinalUserValue 3.5\n");
	printf ("nTextPoss= %d\n",nTextPoss);
#endif
		for ( int h=0; h < nTextPoss; h++ )
		{
#ifdef _DEBUG
		  printf ("pTextPoss= %p\n",pTextPoss[h]);
#endif
		  if (pTextPoss[h]) pTextPoss[h]->hide();
		}

#ifdef _DEBUG
	printf("GraphCell::setFinalUserValue 4\n");
#endif

		TextFinalValue.setText(QString (simbols(value)));
			
		  // Added proyect 27
		  mTextPoss = (int(1)<<value)  ;
#ifdef _DEBUG
		  printf ("cambio mask %x . FvActve =%d\n", mTextPoss, FvActivate);
		  printf ("mPV         %x . canChange=%d\n", GetPossValuesOfCell(pBoard,this_cell),canChange );
#endif


		  TextFinalValue.show();
		  if ( int ( (rect().width() * 100) / ( TextFinalValue.font().pixelSize()) ) > 1)
		  	TextFinalValue.font().setStretch( int ( (rect().width() * 100) / ( TextFinalValue.font().pixelSize()) ));


		  /* Save Movement for Undo/Redo */
			
		 if (p->isPlayerPlaying())
 			{
				 p->NoPlayerMovement=false;
				 p->SaveMovement();
			}

		}
	}
}
void GraphCell:: setPossUserValue (int value)
{
#ifdef _DEBUG
	printf("GraphCell::setPossUserValue 1\n");
#endif

	if ( canChange )
	{
		if (!FvActivate)
		{
			// TODO: revisar 
			 if (mTextPoss== (int(1)<<value) ) return;
			//if ( mTextPoss != (int(1)<<value) )
			//if (mTepCell->ppGr[i]->ppCl[j]->pGraphCell->show();xtPoss ^ (int(1)<<value)  )
			{
				
				mTextPoss ^=  (int(1)<<value);
#ifdef _DEBUG
				printf ("cambio mask %x . FvActve =%d\n", mTextPoss, FvActivate);
#endif
			}

	                SudokuView * p = board->View ();

			// Don't save 2 movement if UpdatePossUserText calls setFinalUserValue
			bool PlayerPlayingState= p->isPlayerPlaying();


			p->setComputerControl();

			UpdatePossUserText( pTextPoss[0]->font().pixelSize(),nTextPoss);

	        	if (PlayerPlayingState)
			{

				 p->setPlayerControl();
			}
	

			/* Save Movement for Undo/Redo */

			/* Clear Marked errors if help box is active */
			/*if (p->isActiveHelpBox)
			{
				p->ActiveHelpBoxColor();
			}*/
			//RemovePossValuesFromCell (pCell->ppGr[0][0].pBoard, mTextPoss, pCell - pCell->ppGr[0][0].pBoard->pCl);
			
			if (p->isPlayerPlaying())
			{
				 p->NoPlayerMovement=false;
				 p->SaveMovement();
			}

		}
	}

#ifdef _DEBUG
 /*   		printf ("******************************\n");
    		printf ("* mPV     =%x  \t\t  *\n", pCell->mPV);
    		printf ("* mRPV    =%x  \t\t  *\n", pCell->mRPV);
		printf ("* TextPoss=%x  \t\t  *\n", mTextPoss);
    		printf ("******************************\n");
*/
#endif

}

/****************************************************/
/*  ClearCell                                       */
/*   WARNING: Be careful                            */
/*   sudoku-mem.c function: ClearBoard( t_board * b)*/
/*   calls this function.                           */
/*   DON'T USE ANY visual function in it            */ 
/****************************************************/

void GraphCell::ClearCell()
{
#ifdef _DEBUG
//	printf("GraphCell::ClearCell 1\n");
#endif
	canChange  =  true ;
	FvActivate =  false;
	mTextPoss = mask_all (board->NumSymbols());
}
void GraphCell::UpdateCell()
{
#ifdef _DEBUG
	printf("GraphCell::UpdateCell 1\n");
#endif
	if (canChange)
	{

	//	QString * mstring;
	
		if (!FvActivate)
		{
		//	setPossUserValue (1);
		//	setPossUserValue (1);
			for ( int h=0; h < nTextPoss; h++ )
					pTextPoss[h]->show();
			TextFinalValue.hide();
			//TODO: unset Symbols in cell
		}
		else
		{
			for ( int h=0; h < nTextPoss; h++ )
					pTextPoss[h]->hide();
			TextFinalValue.show();
		}
		
	}
}

void GraphCell::UpdatePossUserText( int littlesize, int n_lines, int maskToHide, bool clearRule )
{
#ifdef _DEBUG
//	printf("GraphCell::UpdatePossUserText 1\n");
#endif
        int nSy = board->NumSymbols ();
        
	if (canChange)
	{
	
		QString *mstring;
		QColor mColor;
		int n_pos=0;
		//float w_box = pCell->ppGr[0][0].pBoard->wb;
		//float h_box = pCell->ppGr[0][0].pBoard->hb;
	
		int h=0,w=0,i=0,k=0;
		
		int n_char  = int ( ceil ( float (nSy )/ n_lines ) );
		mstring = DM_NEW ( new QString[n_lines]);
		
		int n_spaces = int ( (rect().width() + 1 ) / littlesize );
		n_spaces = ((n_spaces )/ ( n_char ) )- 1;
		//n_spaces = 0;
		
		/**  ** ** **/
		int testTextPoss = 1;
		//printf ("Nº Spaces %d\n",n_spaces);
		//printf ("Nº Char   %d\n",n_char);
				
		for (  h=0; h < n_lines ; h++ )
		{
			mstring[h].append(' ');
			for ( w=0 ; w < n_char ; w++)
			{
				if( k >  nSy ) break;
				
				if(  (mTextPoss & (testTextPoss )) )
				{
					if(  (maskToHide & (testTextPoss )) )
					{
						mstring[h].append('X');  //('X');
					}
					else
					{
						if( maskToHide )
							mstring[h].append('.');
						else
							mstring[h].append( simbols(k));
						n_pos++;
					}
				//	printf ("Escribo test       mask %x. FvActve =%d\n", testTextPoss, FvActivate);
				//	printf ("Escribo letra con ,mask %x. FvActve =%d\n", mTextPoss, FvActivate);
				}
				else
				{
					mstring[h].append(' ');
				//	printf ("Escribo espacio con ,mask %x. FvActve =%d\n", mTextPoss, FvActivate);
				}
				k++;
				testTextPoss = testTextPoss << 1;
				for (i=0; i < n_spaces ; i++)
				{
					mstring[h].append(' ');
				}
			}
			// TODO
			pTextPoss[h]->setText(mstring[h]);
			if ( int ( (rect().width() * 100) / (littlesize  * n_char) ) > 1)
				pTextPoss[h]->font().setStretch( int ( (rect().width() * 100) / (littlesize  * n_char) ));
			pTextPoss[h]->font().setKerning(true);
			
		}
		for (  h=0; h < n_lines ; h++ )
		{
			int alpha;
			if (! maskToHide )
			{
				alpha = ( nSy + 1 - n_pos ) * ( 255 / (nSy + 1));
			}
			else alpha = 255;	

			SudokuView * p = board->View ();
			mColor = p->colorSmallText;
			mColor.setAlpha (alpha);
			pTextPoss[h]->setBrush( mColor );
		}
		/**  **/
		if (clearRule == false)
		{
	
			/*Test if only have 1 possibility */
			/* Setup of mTextPoss */
			if (mTextPoss == 1)
			{
				
				setFinalUserValue (0);
	
				delete [] DM_DEL (mstring);
				return;
			}
				testTextPoss = 1;
			for ( i=0; i < nSy; i++ )
			{
				testTextPoss = testTextPoss << 1;
				if (mTextPoss == testTextPoss)
				{
					setFinalUserValue (i+1);
	
					delete [] DM_DEL (mstring);
					return;
				}
			}
		}
		/* ELSE Set pTextPoss*/
		if (!FvActivate)
		{
			for (  h=0; h < nTextPoss ; h++ )
				pTextPoss[h]->show();
		}
		delete [] DM_DEL (mstring);
		
	}

}

