/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: main.cpp 	 * Status: stable * Version: 2.00 * last changes in version: 0.50	*/
/************************************************************************************************/
/*  This file contains the Start function		   					*/
/*												*/
/* This function create a MyMainWindow (mainwindow.cpp) and set up the application		*/ 
/************************************************************************************************/
#include <QApplication>
#include <QMainWindow>
#include <QMessageBox>
#include <QString>
#include <QFile>
#include "mainwindow.h"
#include "ui_Dnewtable.h"
#include "dmem.h"

//#define PRINT_CRC

quint32 crc32 (QDataStream & in,
               bool isText = false,
               int num_bytes = -1);

bool VerifyCRC ();

#define _RELEASE
int main(int argc, char *argv[])
{
    QT_REQUIRE_VERSION(argc, argv, "4.2.3")

    int value=0;
    /* DEFINITIONS */	
    QApplication app(argc, argv);


 //   Ui::MainWindow ui;
    MyMainWindow *myWindow = DM_NEW (new MyMainWindow(&app) );
//    myWindow->setupUi(myWindow);

    //-- Check CRC of docs here
    bool isCRC_correct = VerifyCRC ();

#ifdef Q_OS_UNIX
    if (!isCRC_correct)
    {
	QString path(argv[0]);
	path.truncate (path.lastIndexOf('/'));
	QDir::setCurrent(path);

	isCRC_correct = VerifyCRC ();

    }
#endif
    if (isCRC_correct)
    {
	/* CONECTIONS */

	/* RUN */	
	myWindow->show();
	
	value = app.exec();
    }
    else
    {
	QMessageBox A(QString (" CORRUPTED INSTALLATION "), QString (" Some data required for this program have been corrupted.\n\n If you are using Linux try to activate the program with the console.\n\nIf it fails, please download and install again a fresh copy, which you can find at:\n\n http://sourceforge.net/projects/sudoku-sensei\n\nThe cause might be a damaged or deleted file. Though, if the problem persists after reinstalling, or if you feel that the cause is an illegal distribution of the program, please contact the author:\n\n Victor Antolin Urtueta (mentur.SudokuSensei AT gmail.com)\n\n or his tutor:\n\n Martin Knoblauch Revuelta (comocomocomo AT users.sourceforge.net) \n\n Please, provide detailed information about the source of your copy."), QMessageBox::Information, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
	A.exec();
    }
	




    delete DM_DEL (myWindow);

#ifndef _RELEASE
    dump_blocks ();
#endif
    return value;


}

quint32 crc32 (QDataStream & in,
               bool isText, // = false
               int num_bytes) // = -1
{
  quint32 crc;
  quint8 byte, bit;
  bool lastWasCR;
  int count;

  crc = 0xFFFFFFFF;
  lastWasCR = false;
  count = 0;

  while (count<num_bytes || num_bytes<0)
  {
    in >> byte;

    if (in.status()!=QDataStream::Ok)
      break;

    count ++;

    if (isText)
    {
      if (byte==10 && lastWasCR)  // When processing a text file,
      {                             // convert all new-line
        lastWasCR = false;          // encodings (LF for Unix,
        continue;                   // CR-LF for Windows, and just
      }                             // CR for Macintosh) into a
      else if (byte==13)            // single LF 
      {
        lastWasCR = true;
        byte = 10;
      }
#ifdef PRINT_CRC                                 // Warn accents
      else if (byte>127)                         // and those
        printf ("Non 7-bit character detected: " // things that
                "(%d) \'%c\'\n", byte, byte);    // look different
#endif                                           // in different
    }                                            // encodings

    crc ^= byte;

    for (bit=0; bit<8; bit++)
    {
      if (crc&1)                        // 0xEDB88320 is the
        crc = (crc >> 1) ^ 0xEDB88320;  // reverse of 0x04C11DB7
      else
        crc = (crc >> 1);        // Shift right instead of left
    }                            // and that's all: the operations
  }                              // have been reversed, so that
                                 // there's no need to reverse
  return ~ crc;                  // input and output
}

bool VerifyCRC ()
{
  bool isOk;
  int i;

  struct
  {
    const char * pathName;
    bool isText;
    quint32 crc;
  }
  Files[] =
  {
    { "license.txt", true, 0x1765105C },
    { "doc/LogoUAH.bmp", false, 0x27A0097D },

    { "doc/About_en.html", true, 0x9A195BDB },
    { "doc/Manual_en.html", true, 0x2B0026AD },
    { "doc/manual_en/7LicenseGPL.html", true, 0x5DAFF8C9 },
    { "language/sudoku_en.qm", false, 0x25C8F98D },

    { "doc/About_es.html", true, 0x1A8F3807 },
    { "doc/Manual_es.html", true, 0x63C6B67F },
    { "doc/manual_es/7LicenseGPL.html", true, 0x05447CC9 },
    { "language/sudoku_es.qm", false, 0x1497FCAD },

    { NULL, false, 0 }
  };

  for (i=0, isOk=true; Files[i].pathName; i++)
  {
    QFile file (Files[i].pathName);

    if (!file.open(QIODevice::ReadOnly))
      return false;

    QDataStream in(&file);

    quint32 crc = crc32 (in, Files[i].isText);

#ifdef PRINT_CRC
    printf ("    { \"%s\", %s, 0x%08X },\n",
            Files[i].pathName,
            Files[i].isText?"true":"false",
            crc);
#endif

    if (crc != Files[i].crc)
      isOk = false;
  }
  return isOk;
}
