/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: mainwindow.cpp  * Status: stable * Version: 2.00 * last changes in version: 0.51	*/
/************************************************************************************************/
/*  This file contains the functions of MyMainWindow class					*/
/*												*/
/*  The MyMainWindow class is defined in mainwindow.h and Dmainwindow.ui			*/
/************************************************************************************************/
#include <stdio.h>
#include <QProcess>
#include <QSettings>
#include <QFileDialog>
#include <QPrinter>
#include <QPrintDialog>

#include "mainwindow.h"
#include "newgame.h"
#include "newtable.h"
#include "SudokuView.h"
#include "SudokuBoard.h"
#include "thread-creator.h"
#include "progress.h"
#include "Dshowsolution.h"
#include "Dshowabout.h"
#include "Dshowmanual.h"
#include "Dshowtutorial.h"
#include "dmem.h"
#include "myassert.h"

/***************/
/* CONSTRUCTOR */
/***************/
MyMainWindow::MyMainWindow( QApplication * miAp) : 
	app(miAp),
	languageActionGroup (NULL),
	FlagHelpMode (false),
	FlagExtraHelpMode (false)
{
#ifdef _DEBUG
	printf ("MainWindow::MainWindow 1\n");
#endif		

/*	FlagHelpMode=false;
	FlagExtraHelpMode=false;
	languageActionGroup = NULL;
*/
	chars_espanol= QString ("es");
	chars_english= QString ("en");
	//activelanguage=chars_espanol;

	qmPath = miAp->applicationDirPath() + "/language";
	miAp->installTranslator(&appTranslator);


	//colorBackground  = QColor(255,255,255);

#ifdef _DEBUG
	printf ("MainWindow::MainWindow pre setupUi\n");
#endif	
	setupUi(this);

#ifdef _DEBUG
	printf ("MainWindow::MainWindow POST setupUi\n");
#endif
	//setUpdatesEnabled(true);	
	//mSudFrame->setUpdatesEnabled(true);
//	mSudFrame->setParent (this);
	mSudFrame->pMainWindow = this;

#ifdef _DEBUG
	printf ("MainWindow::MainWindow 2\n");
#endif
	mSudFrame->mScene->setBackgroundBrush( colorBackground );
	mSudFrame->autoFillBackground();

	mSudFrame->repaint();



/* Connect Minimun Size */
	//connect (mSudFrame, SIGNAL (MinimunSize(int , int )), this, SLOT (Set_MinimumSize(int , int )));
	

	//adjustSize();

#ifdef _DEBUG
	printf ("MainWindow::MainWindow END\n");
#endif
}

/**************/
/* Destructor */
/*****************************************************************/
/*  This fuction destroys the MyMainWindow object.               */
/*****************************************************************/
MyMainWindow::~MyMainWindow()
{
#ifdef _DEBUG
	printf ("MainWindow::~MainWindow 1\n");
#endif
	delete DM_DEL (languageActionGroup );
#ifdef _DEBUG
	printf ("MainWindow::~MainWindow END\n");
#endif
}

/*********/
/* Frame */
/*****************************************************************/
/*  This fuction returns mSudFrame, the SudokuView object.       */
/*****************************************************************/
 SudokuView* MyMainWindow::Frame() const
{
#ifdef _DEBUG
	printf ("MainWindow::Frame 1\n");
#endif
 return mSudFrame;
}


/*****************/
/* writeSettings */
/**************************************************************/
/*  This fuction saves settings of window and color settings  */
/**************************************************************/
void MyMainWindow::writeSettings()
{
#ifdef _DEBUG
	printf ("MainWindow::writeSettings 1\n");
#endif
    /* TODO: Works!, but this function is not finished */
    QSettings settings("GPL Program", "SudokuSensei2");
 
    settings.setValue("geometry", geometry());
    settings.setValue("HelpMode", FlagHelpMode);
    settings.setValue("ExtraHelpMode", FlagExtraHelpMode);
    settings.setValue("ActiveLanguage",activelanguage);

    /* Color Settings */

    /* Save Background Color */
    settings.setValue("ColorBackgroundRed"  ,colorBackground.red()  );
    settings.setValue("ColorBackgroundGreen",colorBackground.green());
    settings.setValue("ColorBackgroundBlue" ,colorBackground.blue() );
    settings.setValue("ColorBackgroundAlpha",colorBackground.alpha() );

    /* Save Grid Color */
    settings.setValue("ColorGridRed"  ,colorGrid.red()  );
    settings.setValue("ColorGridGreen",colorGrid.green());
    settings.setValue("ColorGridBlue" ,colorGrid.blue() );
    settings.setValue("ColorGridAlpha",colorGrid.alpha() );

    /* Save Box Color */
    settings.setValue("ColorBoxRed"  ,colorBox.red()  );
    settings.setValue("ColorBoxGreen",colorBox.green());
    settings.setValue("ColorBoxBlue" ,colorBox.blue() );
    settings.setValue("ColorBoxAlpha",colorBox.alpha() );

    /* Save Cell Color */
    settings.setValue("ColorCellRed"  ,colorCell.red()  );
    settings.setValue("ColorCellGreen",colorCell.green());
    settings.setValue("ColorCellBlue" ,colorCell.blue() );
    settings.setValue("ColorCellAlpha",colorCell.alpha() );


    /* Save Big Text Color */
    settings.setValue("ColorBigTextRed"  ,colorBigText.red()  );
    settings.setValue("ColorBigTextGreen",colorBigText.green());
    settings.setValue("ColorBigTextBlue" ,colorBigText.blue() );
    settings.setValue("ColorBigTextAlpha",colorBigText.alpha() );

    /* Save Small Text Color */
    settings.setValue("ColorSmallTextRed"  ,colorSmallText.red()  );
    settings.setValue("ColorSmallTextGreen",colorSmallText.green());
    settings.setValue("ColorSmallTextBlue" ,colorSmallText.blue() );
    settings.setValue("ColorSmallTextAlpha",colorSmallText.alpha() );

    /* Save Cursor Color */
    settings.setValue("ColorCursorRed"  ,colorCursor.red()  );
    settings.setValue("ColorCursorGreen",colorCursor.green());
    settings.setValue("ColorCursorBlue" ,colorCursor.blue() );
    settings.setValue("ColorCursorAlpha",colorCursor.alpha());

    /* Save Help Box 1 Color */
    settings.setValue("ColorHelpBox1Red"  ,colorHelpBox1.red()  );
    settings.setValue("ColorHelpBox1Green",colorHelpBox1.green());
    settings.setValue("ColorHelpBox1Blue" ,colorHelpBox1.blue() );
    settings.setValue("ColorHelpBox1Alpha",colorHelpBox1.alpha());

    /* Save Help Box 2 Color */
    settings.setValue("ColorHelpBox2Red"  ,colorHelpBox2.red()  );
    settings.setValue("ColorHelpBox2Green",colorHelpBox2.green());
    settings.setValue("ColorHelpBox2Blue" ,colorHelpBox2.blue() );
    settings.setValue("ColorHelpBox2Alpha",colorHelpBox2.alpha());

    /* Save Help Box 3 Color */
    settings.setValue("ColorHelpBox3Red"  ,colorHelpBox3.red()  );
    settings.setValue("ColorHelpBox3Green",colorHelpBox3.green());
    settings.setValue("ColorHelpBox3Blue" ,colorHelpBox3.blue() );
    settings.setValue("ColorHelpBox3Alpha",colorHelpBox3.alpha());

    /* Save Rule 1 Color 1 */
    settings.setValue("ColorRule1color1Red"  ,colorRule1color1.red()  );
    settings.setValue("ColorRule1color1Green",colorRule1color1.green());
    settings.setValue("ColorRule1color1Blue" ,colorRule1color1.blue() );
    settings.setValue("ColorRule1color1Alpha",colorRule1color1.alpha());

    /* Save Rule 1 Color 2 */
    settings.setValue("ColorRule1color2Red"  ,colorRule1color2.red()  );
    settings.setValue("ColorRule1color2Green",colorRule1color2.green());
    settings.setValue("ColorRule1color2Blue" ,colorRule1color2.blue() );
    settings.setValue("ColorRule1color2Alpha",colorRule1color2.alpha());

    /* Save Rule 1 Color 3 */
    settings.setValue("ColorRule1color3Red"  ,colorRule1color3.red()  );
    settings.setValue("ColorRule1color3Green",colorRule1color3.green());
    settings.setValue("ColorRule1color3Blue" ,colorRule1color3.blue() );
    settings.setValue("ColorRule1color3Alpha",colorRule1color3.alpha());

    /* Save Rule 2 Color 1 */
    settings.setValue("ColorRule2color1Red"  ,colorRule2color1.red()  );
    settings.setValue("ColorRule2color1Green",colorRule2color1.green());
    settings.setValue("ColorRule2color1Blue" ,colorRule2color1.blue() );
    settings.setValue("ColorRule2color1Alpha",colorRule2color1.alpha());

    /* Save Rule 2 Color 2 */
    settings.setValue("ColorRule2color2Red"  ,colorRule2color2.red()  );
    settings.setValue("ColorRule2color2Green",colorRule2color2.green());
    settings.setValue("ColorRule2color2Blue" ,colorRule2color2.blue() );
    settings.setValue("ColorRule2color2Alpha",colorRule2color2.alpha());

    /* Save Rule 2 Color 3 */
    settings.setValue("ColorRule2color3Red"  ,colorRule2color3.red()  );
    settings.setValue("ColorRule2color3Green",colorRule2color3.green());
    settings.setValue("ColorRule2color3Blue" ,colorRule2color3.blue() );
    settings.setValue("ColorRule2color3Alpha",colorRule2color3.alpha());


    /* Save Rule 3 Color 1 */
    settings.setValue("ColorRule3color1Red"  ,colorRule3color1.red()  );
    settings.setValue("ColorRule3color1Green",colorRule3color1.green());
    settings.setValue("ColorRule3color1Blue" ,colorRule3color1.blue() );
    settings.setValue("ColorRule3color1Alpha",colorRule3color1.alpha());

    /* Save Rule 3 Color 2 */
    settings.setValue("ColorRule3color2Red"  ,colorRule3color2.red()  );
    settings.setValue("ColorRule3color2Green",colorRule3color2.green());
    settings.setValue("ColorRule3color2Blue" ,colorRule3color2.blue() );
    settings.setValue("ColorRule3color2Alpha",colorRule3color2.alpha());

    /* Save Rule 3 Color 3 */
    settings.setValue("ColorRule3color3Red"  ,colorRule3color3.red()  );
    settings.setValue("ColorRule3color3Green",colorRule3color3.green());
    settings.setValue("ColorRule3color3Blue" ,colorRule3color3.blue() );
    settings.setValue("ColorRule3color3Alpha",colorRule3color3.alpha());

   /*settings.setValue("recentFiles", recentFiles);
    settings.setValue("showGrid", showGridAction->isChecked());
    settings.setValue("autoRecalc", autoRecalcAction->isChecked());*/

#ifdef _DEBUG
	printf ("MainWindow::writeSettings END\n");
#endif
}

/****************/
/* readSettings */
/**************************************************************/
/*  This fuction reads settings of window and color settings  */
/**************************************************************/
void MyMainWindow::readSettings()
{
#ifdef _DEBUG
	printf ("MainWindow::readSettings 1\n");
#endif
    /* TODO: Works!, but this function is not finished */
    int r,g,b,a;
    QSettings settings("GPL Program", "SudokuSensei2");

    /* Load Window Size */
    QRect rect = settings.value("geometry",
                                 QRect(200, 200, 400, 400)).toRect();
    move(rect.topLeft());
    resize(rect.size());

    /* Load Help Mode */

    FlagHelpMode      = settings.value("HelpMode", false).toBool();
    FlagExtraHelpMode = settings.value("ExtraHelpMode", false).toBool();
    activelanguage    = settings.value("ActiveLanguage",chars_english).toString();


    /* Load Background Color */
    r   = settings.value("ColorBackgroundRed"  ,255).toInt();
    g   = settings.value("ColorBackgroundGreen",255).toInt();
    b   = settings.value("ColorBackgroundBlue" ,255).toInt();
    a   = settings.value("ColorBackgroundAlpha",125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorBackground.setRgb( r, g, b, a);
    }

    /* Load Grid Color */
    r   = settings.value("ColorGridRed"  ,200).toInt();
    g   = settings.value("ColorGridGreen",0).toInt();
    b   = settings.value("ColorGridBlue" ,0).toInt();
    a   = settings.value("ColorGridAlpha" ,125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorGrid.setRgb( r, g, b, a);
    }

    /* Load Box Color */
    r   = settings.value("ColorBoxRed"  ,10).toInt();
    g   = settings.value("ColorBoxGreen",10).toInt();
    b   = settings.value("ColorBoxBlue" ,10).toInt();
    a   = settings.value("ColorBoxAlpha" ,125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorBox.setRgb( r, g, b, a);
    }

     /* Load Cell Color */
    r   = settings.value("ColorCellRed"  ,125).toInt();
    g   = settings.value("ColorCellGreen",125).toInt();
    b   = settings.value("ColorCellBlue" ,255).toInt();
    a   = settings.value("ColorCellAlpha" ,125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorCell.setRgb( r, g, b, a);
    }


    /* Load Big Text Color */
    r   = settings.value("ColorBigTextRed"  ,1).toInt();
    g   = settings.value("ColorBigTextGreen",1).toInt();
    b   = settings.value("ColorBigTextBlue" ,1).toInt();
    a   = settings.value("ColorBigTextAlpha" ,225).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorBigText.setRgb( r, g, b, a);
    }

    /* Load Small Text Color */
    r   = settings.value("ColorSmallTextRed"  ,1).toInt();
    g   = settings.value("ColorSmallTextGreen",1).toInt();
    b   = settings.value("ColorSmallTextBlue" ,255).toInt();
    a   = settings.value("ColorSmallTextAlpha" ,225).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorSmallText.setRgb( r, g, b, a);
    }

    /* Load Cursor Color */
    r   = settings.value("ColorCursorRed"  ,0).toInt();
    g   = settings.value("ColorCursorGreen",255).toInt();
    b   = settings.value("ColorCursorBlue" ,50).toInt();
    a   = settings.value("ColorCursorAlpha" ,125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorCursor.setRgb( r, g, b, a);
    }

    /* Load Help Box 1 Colors */
    r   = settings.value("ColorHelpBox1Red"  ,255).toInt();
    g   = settings.value("ColorHelpBox1Green",255).toInt();
    b   = settings.value("ColorHelpBox1Blue" ,0).toInt();
    a   = settings.value("ColorHelpBox1Alpha",125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorHelpBox1.setRgb( r, g, b, a);
    }

    /* Load Help Box 2 Colors */
    r   = settings.value("ColorHelpBox2Red"  ,0).toInt();
    g   = settings.value("ColorHelpBox2Green",255).toInt();
    b   = settings.value("ColorHelpBox2Blue" ,255).toInt();
    a   = settings.value("ColorHelpBox2Alpha",125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorHelpBox2.setRgb( r, g, b, a);
    }

    /* Load Help Box 3 Colors */
    r   = settings.value("ColorHelpBox3Red"  ,0).toInt();
    g   = settings.value("ColorHelpBox3Green",255).toInt();
    b   = settings.value("ColorHelpBox3Blue" ,0).toInt();
    a   = settings.value("ColorHelpBox3Alpha",125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorHelpBox3.setRgb( r, g, b, a);
    }

    /* Load Rule 1 Color 1 */
    r   = settings.value("ColorRule1color1Red"  ,255).toInt();
    g   = settings.value("ColorRule1color1Green",0).toInt();
    b   = settings.value("ColorRule1color1Blue" ,255).toInt();
    a   = settings.value("ColorRule1color1Alpha",125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorRule1color1.setRgb( r, g, b, a);
    }

    /* Load Rule 1 Color 2 */
    r   = settings.value("ColorRule1color2Red"  ,255).toInt();
    g   = settings.value("ColorRule1color2Green",55).toInt();
    b   = settings.value("ColorRule1color2Blue" ,50).toInt();
    a   = settings.value("ColorRule1color2Alpha",125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorRule1color2.setRgb( r, g, b, a);
    }

    /* Load Rule 1 Color 3 */
    r   = settings.value("ColorRule1color3Red"  ,10).toInt();
    g   = settings.value("ColorRule1color3Green",255).toInt();
    b   = settings.value("ColorRule1color3Blue" ,50).toInt();
    a   = settings.value("ColorRule1color3Alpha",125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorRule1color3.setRgb( r, g, b, a);
    }

   /* Load Rule 2 Color 1 */
    r   = settings.value("ColorRule2color1Red"  ,192).toInt();
    g   = settings.value("ColorRule2color1Green",214).toInt();
    b   = settings.value("ColorRule2color1Blue" ,211).toInt();
    a   = settings.value("ColorRule2color1Alpha",125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorRule2color1.setRgb( r, g, b, a);
    }

    /* Load Rule 2 Color 2 */
    r   = settings.value("ColorRule2color2Red"  ,110).toInt();
    g   = settings.value("ColorRule2color2Green",255).toInt();
    b   = settings.value("ColorRule2color2Blue" ,55).toInt();
    a   = settings.value("ColorRule2color2Alpha",125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorRule2color2.setRgb( r, g, b, a);
    }

    /* Load Rule 2 Color 3 */
    r   = settings.value("ColorRule2color3Red"  ,255).toInt();
    g   = settings.value("ColorRule2color3Green",0).toInt();
    b   = settings.value("ColorRule2color3Blue" ,0).toInt();
    a   = settings.value("ColorRule2color3Alpha",125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorRule2color3.setRgb( r, g, b, a);
    }
   /* Load Rule 3 Color 1 */
    r   = settings.value("ColorRule3color1Red"  ,0).toInt();
    g   = settings.value("ColorRule3color1Green",255).toInt();
    b   = settings.value("ColorRule3color1Blue" ,125).toInt();
    a   = settings.value("ColorRule3color1Alpha",125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorRule3color1.setRgb( r, g, b, a);
    }

    /* Load Rule 3 Color 2 */
    r   = settings.value("ColorRule3color2Red"  ,120).toInt();
    g   = settings.value("ColorRule3color2Green",205).toInt();
    b   = settings.value("ColorRule3color2Blue" ,0).toInt();
    a   = settings.value("ColorRule3color2Alpha",125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorRule3color2.setRgb( r, g, b, a);
    }

    /* Load Rule 3 Color 3 */
    r   = settings.value("ColorRule3color3Red"  ,31).toInt();
    g   = settings.value("ColorRule3color3Green",155).toInt();
    b   = settings.value("ColorRule3color3Blue" ,150).toInt();
    a   = settings.value("ColorRule3color3Alpha",125).toInt();

    if (-1 < r  && r <256 && -1 < g && g < 256 && -1 < b && b < 256  && -1 < a && a < 256)
    {
	colorRule3color3.setRgb( r, g, b, a);
    }

    // /* TODO :  Please review */

    /* Update the language buttons for  consistency */
    if (activelanguage.compare(chars_espanol) == 0)
    {
	//action_Espa_ol->setChecked ( true);
	emit action_Espa_ol->trigger();
    }
    else if (activelanguage.compare(chars_english) == 0)
    {
	//action_English->setChecked ( true);
	emit action_English->trigger();
    }
    // /*****************************/
    
    /*Load the language file */
    appTranslator.load("sudoku_" + activelanguage, qmPath);

/*
    recentFiles = settings.value("recentFiles").toStringList();
    updateRecentFileActions();
    bool showGrid = settings.value("showGrid", true).toBool();
    showGridAction->setChecked(showGrid);

    bool autoRecalc = settings.value("autoRecalc", true).toBool();
    autoRecalcAction->setChecked(autoRecalc);*/

#ifdef _DEBUG
	printf ("MainWindow::readSettings END\n");
#endif
}


/***********/
/* setupUi */
/***************************************************/
/*  This fuction setup the window and connections  */
/***************************************************/
void MyMainWindow::setupUi(QMainWindow *MainWindow)
 {
#ifdef _DEBUG
	printf ("MainWindow::MyMainWindowSetupUI 1\n");
#endif
 	if (languageActionGroup == NULL)languageActionGroup = DM_NEW (new QActionGroup(this) );
 
	Ui_MainWindow::setupUi(MainWindow);

	/*  CONNECTIONS */
	
	/* Build the LanguageGroup */
	languageActionGroup->addAction(action_Espa_ol);
	languageActionGroup->addAction(action_English);
        // Add the action_<NEW LANGUAGE>
	// 2 of 3 changes to add a new language
	
	/* LANGUAGE CONNECTIONS */
	connect(languageActionGroup, SIGNAL(triggered(QAction *)), this,SLOT(switchLanguage(QAction *)));
	
	connect(this,SIGNAL(LanguageChanged()),mSudFrame,SLOT(ChangeWindowsLanguage()));


	/* FILE MENU CONNECTIONS */
	connect (action_New_Game, SIGNAL (triggered()),this,SLOT(CreateNewGame()));
	connect (actionNew_Board, SIGNAL (triggered()),this,SLOT(CreateNewBoard()));
	connect (actionPrint, SIGNAL (triggered()),this,SLOT(Printing()));


	connect (action_Exit,SIGNAL (triggered()),this, SLOT(writeSettings()));
	connect (action_Exit,SIGNAL (triggered()),app, SLOT(closeAllWindows()));

	readSettings();
	
	/* Color Options */
	connect (actionColor_Background, SIGNAL (triggered()), this, SLOT (ChangeColorBackground()));
	connect (actionColor_Grid, SIGNAL (triggered()), this, SLOT (ChangeColorGrid()));
	connect (actionColor_Box, SIGNAL (triggered()), this, SLOT (ChangeColorBox()));
        connect (actionColor_Cell, SIGNAL (triggered()), this, SLOT (ChangeColorCell()));
	connect (actionColor_Text, SIGNAL (triggered()), this, SLOT (ChangeColorText()));
	connect (actionColor_Small_Text, SIGNAL (triggered()), this, SLOT (ChangeSmallColorText()));
        connect (actionColor_Cursor, SIGNAL (triggered()), this, SLOT (ChangeColorCursor()));

	connect (actionColor_HelpBox_1, SIGNAL (triggered()), this, SLOT (ChangeColorHelpBox1()));
	connect (actionColor_HelpBox_2, SIGNAL (triggered()), this, SLOT (ChangeColorHelpBox2()));
	connect (actionColor_HelpBox_3, SIGNAL (triggered()), this, SLOT (ChangeColorHelpBox3()));

	connect (actionColor_Rule_1_color_1, SIGNAL (triggered()), this, SLOT (ChangeColorRule1color1()));
	connect (actionColor_Rule_1_color_2, SIGNAL (triggered()), this, SLOT (ChangeColorRule1color2()));
	connect (actionColor_Rule_1_color_3, SIGNAL (triggered()), this, SLOT (ChangeColorRule1color3()));

	connect (actionColor_Rule_2_color_1, SIGNAL (triggered()), this, SLOT (ChangeColorRule2color1()));
	connect (actionColor_Rule_2_color_2, SIGNAL (triggered()), this, SLOT (ChangeColorRule2color2()));
	connect (actionColor_Rule_2_color_3, SIGNAL (triggered()), this, SLOT (ChangeColorRule2color3()));

	connect (actionColor_Rule_3_color_1, SIGNAL (triggered()), this, SLOT (ChangeColorRule3color1()));
	connect (actionColor_Rule_3_color_2, SIGNAL (triggered()), this, SLOT (ChangeColorRule3color2()));
	connect (actionColor_Rule_3_color_3, SIGNAL (triggered()), this, SLOT (ChangeColorRule3color3()));
	/*********************************/
	/* Update and Connect GAME MODES */


	/*************************************/
	/* Save and Load Game Connections    */

	connect(action_Save_Game, SIGNAL (triggered()), this, SLOT (SaveGameAs()));
	connect(action_Load_Game, SIGNAL (triggered()), this, SLOT (LoadGame  ()));


	/********************************/
	/* Undo and Redo Connections    */

	connect(action_Undo, SIGNAL (triggered())   , this, SLOT (Undo()));
	connect(action_Redo, SIGNAL (triggered())   , this, SLOT (Redo()));
	connect(mSudFrame  , SIGNAL (ctrlZPressed()), this, SLOT (Undo()));
	connect(mSudFrame  , SIGNAL (ctrlXPressed()), this, SLOT (Redo()));

	connect(mSudFrame, SIGNAL (CanUndo(bool)), this, SLOT (EnableUndo(bool)));
	connect(mSudFrame, SIGNAL (CanRedo(bool)), this, SLOT (EnableRedo(bool)));

	/********************************/
	/* Enlarge Window Connections    */

	connect(actionEnlarge_Window,SIGNAL (triggered()),this, SLOT (clickEnlarge()));

 	connect(menubar, SIGNAL (hovered ( QAction * )),this, SLOT (updateEnlarge(QAction *) ));
	/****************/
	/* NO HELP mode */
	actionNo_Help->setChecked(MODE_NO_HELP);
	connect(actionNo_Help,SIGNAL(triggered()), this,SLOT(UpdateNoHelp()));

	/********************/
	/* LITTLE HELP mode */
	actionLittle_Help->setChecked( MODE_LITTLE_HELP);
	connect(actionLittle_Help, SIGNAL(triggered()), this,SLOT(UpdateLittleHelp()));

	/*****************/
	/*   SENSEI MODE */
	action_Sensei->setChecked( MODE_SENSEI);
	connect(action_Sensei, SIGNAL(triggered()), this,SLOT(UpdateHelpSensei()));

	/********************************/
	/* Show Solution Connections    */
	connect (actionShow_Solution, SIGNAL (triggered()), this, SLOT (ShowSolution()));
	EnableShowSolution( MODE_SENSEI);

	/*********************************/
	/* Connect About Connections     */

	connect (action_About, SIGNAL (triggered()), this, SLOT (ShowAbout()));

	/**********************************/
	/* Connect Manual Connections     */

	connect (action_Manual, SIGNAL (triggered()), this, SLOT (ShowManual()));

	/**********************************/
	/* Connect Manual Connections     */

	connect (actionInit_Tutorial, SIGNAL (triggered()), this, SLOT (ShowTutorial()));

	/* Finally Translate the mainwindow */
	/* TODO: Check if can be deleted */
  	retranslateUi(this);
	emit LanguageChanged();

#ifdef _DEBUG
	printf ("MainWindow::MyMainWindowSetupUI END\n");
#endif
    }

void MyMainWindow::closeEvent(QCloseEvent *event)
{
	writeSettings();
	QMainWindow::closeEvent(event);
}

/*****************/
/* CreateNewGame */
/*****************************************************************/
/*  This fuction calls a NewGame dialog and generates a new game */
/*****************************************************************/
void MyMainWindow::CreateNewGame()
{
#ifdef _DEBUG
	printf ("MainWindow::CreateNewGame 1\n");
#endif
	/* Added in project 0.51 */
	bool RulesHided= false;
	bool EnlargeHided= false;
	/**/

	MY_ASSERT (mSudFrame->isPlayerPlaying()==true);
	mSudFrame->setComputerControl();
	NewGame * NewGameWindow = DM_NEW (new NewGame(this) );

	/* Added in project 0.51 */
	if (mSudFrame->mRulesDialog)
	{
		if (mSudFrame->mRulesDialog->isVisible())
		{
			RulesHided= true;
			mSudFrame->mRulesDialog->hide();
		}
	}
	if (mSudFrame->mEnlargeWindow)
	{
		if (mSudFrame->mEnlargeWindow->isVisible())
		{
			EnlargeHided = true;
			mSudFrame->mEnlargeWindow->hide();
		}
	}
	/**/

	if ( NewGameWindow->exec() )
	{
		if (NewGameWindow->output() == -3)
		{
#ifdef _DEBUG
			printf ("BoardEmpty\n");
#endif
			mSudFrame->cleanAllCells();
		}
		else if (NewGameWindow->output() == -2)
		{
#ifdef _DEBUG
			printf ("BoardWrited\n");
#endif
			mSudFrame->cleanAllCells();
			LoadRawGame( NewGameWindow->rawTable() );

// TODO:  GenerateBoard(TextBoard);

		}
		else if (NewGameWindow->output() > 0)
		{
#ifdef _DEBUG
			printf ("BoardGenerate\n");// TODO:
#endif
			GenerateGame ( NewGameWindow->output() );
		}
		else if (NewGameWindow->output() == -1)
		{
			GenerateGame (-1);// Difficult High 
			
		}
		else
		{
			//QPrintf (tr("Error in NewGame::aceptClicked\n"));
		}
		mSudFrame->RestartUndo(); // Restart Undo/Redo System
		mSudFrame->ActiveHelpBoxColor();
	}

	/* Added in project 0.51 */
		if (RulesHided) mSudFrame->mRulesDialog->show();
		if (EnlargeHided) mSudFrame->mEnlargeWindow->show();
	/***/

	MY_ASSERT (mSudFrame->isPlayerPlaying()==false);
	mSudFrame->setPlayerControl();


	delete DM_DEL (NewGameWindow);

#ifdef _DEBUG
	printf ("MyMainWindow::CreateNewGame END\n");
#endif
}

/******************/
/* CreateNewBoard */
/***********************************************************************/
/*  This fuction calls a MyNewBoard dialog which generates a new board */
/***********************************************************************/
void MyMainWindow::CreateNewBoard()
{
#ifdef _DEBUG
	printf ("MainWindow::CreateNewBoard 1\n");
#endif
	/* Added in project 0.51 */
	bool RulesHided= false;
	bool EnlargeHided= false;
	/**/

	MyNewBoard * NewBoardWindow = DM_NEW (new MyNewBoard(this) );

	/* Added in project 0.51 */
	if (mSudFrame->mRulesDialog)
	{
		if (mSudFrame->mRulesDialog->isVisible())
		{
			RulesHided= true;
			mSudFrame->mRulesDialog->hide();
		}
	}
	if (mSudFrame->mEnlargeWindow)
	{
		if (mSudFrame->mEnlargeWindow->isVisible())
		{
			EnlargeHided = true;
			mSudFrame->mEnlargeWindow->hide();
		}
	}
	/**/
	
	/* INITIAL SET OF COLORS */
	MY_ASSERT (mSudFrame->isPlayerPlaying()==true);
	mSudFrame->setComputerControl();
	SetAllSudokuColors();
	if ( NewBoardWindow->exec() )
	{
		EnableGame();
		//mSudFrame->RepaintBackGround( colorBackground );
#ifdef _DEBUG
		printf("MyMainWindow::CreateNewBoard 1\n");
#endif
		// Show Rules Dialog
		if (MODE_SENSEI) 
		{
			mSudFrame->mRulesDialog->show();
		}


		mSudFrame->RestartUndo(); // Restart Undo/Redo System
#ifdef _DEBUG
		printf("MyMainWindow::CreateNewBoard 2\n");
#endif
	}
	/* Added in project 0.51 */
	if (RulesHided) mSudFrame->mRulesDialog->show();
	if (EnlargeHided) mSudFrame->mEnlargeWindow->show();
	/***/

	MY_ASSERT (mSudFrame->isPlayerPlaying()==false);
	mSudFrame->setPlayerControl();

	delete DM_DEL (NewBoardWindow);
#ifdef _DEBUG
	printf("MyMainWindow::CreateNewBoard 3 end\n");
#endif
	
}

/******************/
/* UpdateGameMode */
/*********************************************/
/*  This fuction updates Help Mode.          */
/*********************************************/
void MyMainWindow::UpdateGameMode(bool flag1, bool flag2)
{
#ifdef _DEBUG
	printf("MyMainWindow::UpdateGameMode 1\n");
#endif
	// Update internal variables
	FlagHelpMode=flag1;
	FlagExtraHelpMode=flag2;

	// Update external bottons
	actionNo_Help->setChecked    (MODE_NO_HELP    );
	actionLittle_Help->setChecked(MODE_LITTLE_HELP);
	action_Sensei->setChecked    (MODE_SENSEI     );

	//Activate Help Box Mode
	mSudFrame->setIsActiveHelpBox(FlagHelpMode);

	//Activate Show Solution if exists.
	EnableShowSolution(MODE_SENSEI);

	//Activate Show Solution if exists.
	if (MODE_SENSEI)
	{
		mSudFrame->mRulesDialog->hide();
	 	mSudFrame->mRulesDialog->show();
		mSudFrame->mRulesDialog->activateWindow();
		
	}
	else mSudFrame->mRulesDialog->hide();
}

/*******************/
/* Switch Language */
/****************************************************************/
/*  This fuction change the language and retranslate the window */
/****************************************************************/
void MyMainWindow::switchLanguage(QAction *action)
{
#ifdef _DEBUG
	printf ("MyMainWindow::switchLanguage 1\n");
#endif
    QString language = action->text();
    if ( language.compare(action_Espa_ol->text()) == 0 )
    {
	activelanguage=chars_espanol;
    }
    else if ( language.compare(action_English->text())== 0)
    {
	activelanguage=chars_english;
    }
    // add more "else if" when you want to add more languages
    appTranslator.load("sudoku_" + activelanguage, qmPath);
    retranslateUi(this);
    emit LanguageChanged();

    //TODO: if EnlageWindow activated;
}

/***************/
/* paintEvent  */
/***************/
/*
void MyMainWindow::paintEvent(QPaintEvent *event)
{
	
	QMainWindow::paintEvent(event);
}
*/
/***************/
/* resizeEvent */
/***************/
void MyMainWindow::resizeEvent(QResizeEvent *event)
{
#ifdef _DEBUG
	printf ("MyMainWindow::resizeEvent 1\n");
#endif
	mSudFrame->resize(width(),height());
	QMainWindow::resizeEvent(event);
}

/**************/
/* EnableGame */
/*********************************************/
/*  This fuction enable new game menu option */
/*********************************************/
void MyMainWindow::EnableGame()
{
#ifdef _DEBUG
	printf ("MyMainWindow::EnableGame 1\n");
#endif
	action_New_Game->setEnabled(true);
	action_Save_Game->setEnabled(true);
	actionPrint->setEnabled(true);
}

/**************/
/* Printing   */
/***************************************************************************/
/*  This fuction create a Printer dialog and send the scene to the printer */
/***************************************************************************/
void MyMainWindow::Printing()
{
#ifdef _DEBUG
	printf ("MyMainWindow::Printing 1\n");
#endif
	QPrinter printer(QPrinter::HighResolution);
	QPrintDialog printDialog(&printer, this);
   	if (printDialog.exec()) 
	{
        	QPainter painter(&printer);
        	mSudFrame->mScene->render(&painter);	
    	}
}

/****************/
/* GenerateGame */
/********************************************************************/
/*  This fuction generate a new game. It creates a CreatorThread    */
/*  and send it to a MyProgress dialog which activates it.          */
/*                                                                  */
/********************************************************************/
void MyMainWindow::GenerateGame( short level )
{
#ifdef _DEBUG
	printf ("MyMainWindow::GenerateGame 1\n");
#endif
	QString levelString;
	QString a;

        CreatorThread MyThread;
  	
	QApplication::setOverrideCursor (Qt::WaitCursor);
	/*First Clean all the Board */
	mSudFrame->cleanAllCells();


        /* Take the arguments */
	MyThread.arguments <<"-name" << "START" << "-num" << "1";

  
/*  	MyThread.par.nGrids = 0;
  	MyThread.par.offsetsX = MyThread.par.offsetsY = NULL;
  	MyThread.par.samurai = MyThread.par.diagonals = false;
  	MyThread.par.num = 1;
  	MyThread.par.restrictinters = 0;
  	MyThread.par.name = NULL; // Add By Victor Antolin
 
	
	MyThread.par.name = (char*) DM_NEW (malloc ( sizeof("START")) );
	if (MyThread.par.name==NULL)
	{*/
		/* ERROR OF MEMORY
	}
	strcpy(MyThread.par.name, "START");*/

        t_board_p pBoard = mSudFrame->sudoku_board->Board ();
        int wb = GetBlockWidth (pBoard);
        int hb = GetBlockHeight (pBoard);

	/* Block Size */
	MyThread.arguments <<"-blocksize";
	MyThread.arguments << a.setNum(wb);
	MyThread.arguments << a.setNum(hb);
	/*MyThread.par.blockWidth  = GetBlockWidth(pBoard);
	MyThread.par.blockHeight = GetBlockHeight(pBoard);
*/
        /* Difficult level */
	if (level== -1)
	{  MyThread.highdificult = true;
	   MyThread.arguments<<"-level"<< levelString.setNum(1);
	}
	else MyThread.arguments<<"-level"<< levelString.setNum(level);
//	MyThread.par.level = level;
	

        /* Grids if multiple grids sudoku */
	if (mSudFrame->nmGrids > 1)
	{
		
		MyThread.arguments <<"-grids" << a.setNum(mSudFrame->nmGrids);
/*		MyThread.par.nGrids = mSudFrame->nmGrids;
		MyThread.par.offsetsX = (int*) DM_NEW (malloc (MyThread.par.nGrids * sizeof(int)) );
		MyThread.par.offsetsY = (int*) DM_NEW (malloc (MyThread.par.nGrids * sizeof(int)) );

		if (!MyThread.par.offsetsX || !MyThread.par.offsetsY)
		{
			// MEMORY ERROR
		}

		// Loop to find grid coordinates 
*/		int block_x=0;
		int block_y=0;
		for (int i=0; i < mSudFrame->nmGrids; i++)
		{
			/* Conversion units cell -> units block */
			block_x = ( mSudFrame->GridsX[i] -1 + wb) / wb;
			block_y = ( mSudFrame->GridsY[i] -1 + hb) / hb;
			
			MyThread.arguments << a.setNum(block_x );
			MyThread.arguments << a.setNum(block_y );
		}
		
	}
		
	/* Start the Creator Program */

	MyProgress ProgressDialog( this , &MyThread);

	ProgressDialog.exec();
	
	MyThread.setFlagCancel (true);   // Prevent nasty
	MyThread.Wait_for_completion (); // errors (ESC key...)

	if (ProgressDialog.user_say &&
	    MyThread.flagCreated)        // Verify success of creation
	{
		// Don't call setComputerControl() here
		// The caller of GenerateGame() already did it

		/* Read values */
		FILE * pf = fopen("START.txt", "rb");
		mReadBoard(mSudFrame->sudoku_board, pf, NULL, NULL);
		fclose(pf);

#ifdef _DEBUG
		printf("MyMainWindow::GenerateGame All ok\n");
#endif

		// Don't call setPlayerControl() here
		// The caller of GenerateGame() will do it later
	}
	else
	{
#ifdef _DEBUG
		printf("MyMainWindow::GenerateGame Canceled\n");
#endif

		mSudFrame->cleanAllCells();
	}

	/* else do nothing, because it was canceled */
	QApplication::restoreOverrideCursor ();
}


/**************/
/* mReadBoard */
/***************************************************************************/
/*  Read a game generated by sudoku-sensei console.                        */
/*  This fuction reads the output of sudoku-creator called in GenerateGame */
/*                                                                         */
/* TODO: Change this fuction when sudoku-creator fuction  would be ready   */
/***************************************************************************/
int MyMainWindow::mReadBoard (SudokuBoard * b,
               FILE * pf,
               const char * symbols,
               const char * empty)
{
        t_board_p pBoard = b->Board ();
        int cell, val, resul;

#ifdef _DEBUG
	printf ("MyMainWindow::mReadBoard 1\n");
#endif

        resul = ReadBoard (pBoard, pf, symbols, empty);
        
        for (cell=0; cell<b->NumCells(); cell++)
        {
            val = GetSymbolOfCell (pBoard, cell);
            
            if (val != -1)
            {
#ifdef _DEBUG
	  	printf("valores de c= %d | j= %c \n", cell , b->pCl, symbols[val]);
#endif

		b->GCell(cell)->setFinalUserValue( (val ) ); /* TODO: Change to setFinalCompValue */
		b->GCell(cell)->canChange  =  false ;

		//b->GCell(cell)->FvActivate  = true ;
            }
        }

        return resul;
}

/************************/
/* COLOR FUNCTIONS      */
/************************/

/*************************/
/* ChangeColorBackground */
/********************************************************************/
/*  This function open a select color dialog and calls to           */
/*  SudokuView->RepaintBackGround function                          */
/********************************************************************/
void MyMainWindow::ChangeColorBackground()
{
    QColor color = QColorDialog::getColor(colorBackground,this);
    if ( color.isValid() )
    {
	colorBackground= color;
	centralwidget->setAutoFillBackground (true);
	mSudFrame->RepaintBackGround( colorBackground );
	
    }

}

/*************************/
/* ChangeColorGrid       */
/********************************************************************/
/*  This function open a select color dialog and calls to           */
/*  SudokuView->RepaintGrid function                                */
/********************************************************************/
void MyMainWindow::ChangeColorGrid()
{
    QColor color = QColorDialog::getColor(colorGrid,this);
    if ( color.isValid() )
    {
	colorGrid = color;
	mSudFrame->colorGrid = colorGrid ;
	mSudFrame->RepaintGrid ();
    }


}

/************************/
/* ChangeColorBox       */
/********************************************************************/
/*  This function open a select color dialog and calls to           */
/*  SudokuView->RepaintBox function                                 */
/********************************************************************/
void MyMainWindow::ChangeColorBox()
{
    QColor color = QColorDialog::getColor(colorBox,this);
    if ( color.isValid() )
    {
	colorBox = color;
	mSudFrame->colorBlock = colorBox ;
	mSudFrame->RepaintBlock ();
    }
}

/*************************/
/* ChangeColorCell       */
/********************************************************************/
/*  This function open a select color dialog and calls to           */
/*  SudokuView->RepaintCell function                                */
/********************************************************************/
void MyMainWindow::ChangeColorCell()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorCell 1\n");
#endif
    QColor color = QColorDialog::getColor(colorCell,this);
    if ( color.isValid() )
    {
	colorCell = color;
	mSudFrame->colorCell = colorCell ;
	mSudFrame->RepaintCell ();
    }
}


/*************************/
/* ChangeColorText       */
/********************************************************************/
/*  This function open a select color dialog and calls to           */
/*  SudokuView->RepaintBigText function                             */
/********************************************************************/
void MyMainWindow::ChangeColorText()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorText 1\n");
#endif
    QColor color = QColorDialog::getColor(colorBigText,this);
    if ( color.isValid() )
    {
	colorBigText = color;
	mSudFrame->colorBigText = colorBigText ;
	mSudFrame->RepaintBigText () ;
    }
}

/*************************/
/* ChangeSmallColorText  */
/********************************************************************/
/*  This function open a select color dialog and calls to           */
/*  SudokuView->RepaintSmallText function                           */
/********************************************************************/
void MyMainWindow::ChangeSmallColorText()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeSmallColorText 1\n");
#endif
    QColor color = QColorDialog::getColor(colorSmallText,this);
    if ( color.isValid() )
    {
	colorSmallText = color;
	mSudFrame->colorSmallText = colorSmallText ;
	mSudFrame->RepaintSmallText () ;
    }
}

/*************************/
/* ChangeColorCursor     */
/********************************************************************/
/*  This function open a select color dialog and calls to           */
/*  SudokuView->RepaintCursor function                              */
/********************************************************************/
void MyMainWindow::ChangeColorCursor()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorCursor 1\n");
#endif
    QColor color = QColorDialog::getColor(colorCursor,this);
    if ( color.isValid() )
    {
	colorCursor = color;
	mSudFrame->curColor = colorCursor;
	mSudFrame->RepaintCursor() ;
    }
}

/*********************************/
/* ChangeColorHelpBox1, 2 and 3  */
/********************************************************************/
/*  This function open a select color dialog and calls to           */
// TODO:
/********************************************************************/
void MyMainWindow::ChangeColorHelpBox1()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorHelpBox1 1\n");
#endif
    QColor color = QColorDialog::getColor(colorHelpBox1,this);
    if ( color.isValid() )
    {
	colorHelpBox1 = color;
	mSudFrame->colorHelpBox1 = colorHelpBox1;
//	mSudFrame->RepaintCursor() ;
    }
}

void MyMainWindow::ChangeColorHelpBox2()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorHelpBox2 1\n");
#endif
    QColor color = QColorDialog::getColor(colorHelpBox2,this);
    if ( color.isValid() )
    {
	colorHelpBox2 = color;
	mSudFrame->colorHelpBox2 = colorHelpBox2;
//	mSudFrame->RepaintCursor() ;
    }
}

void MyMainWindow::ChangeColorHelpBox3()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorHelpBox3 1\n");
#endif
    QColor color = QColorDialog::getColor(colorHelpBox3,this);
    if ( color.isValid() )
    {
	colorHelpBox3 = color;
	mSudFrame->colorHelpBox3 = colorHelpBox3;
    }
}

/****************************************************/
/* ChangeColorRuleXcolorY  when X=1,2,3 and Y=1,2,3 */
/********************************************************************/
/*  This function open a select color dialog and calls to           */
//TODO:
/********************************************************************/

void MyMainWindow::ChangeColorRule1color1()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorRule1color1 1\n");
#endif
    QColor color = QColorDialog::getColor(colorRule1color1,this);
    if ( color.isValid() )
    {
	colorRule1color1 = color;
	mSudFrame->colorRule1color1 = colorRule1color1;
    }
}

void MyMainWindow::ChangeColorRule1color2()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorRule1color2 1\n");
#endif
    QColor color = QColorDialog::getColor(colorRule1color2,this);
    if ( color.isValid() )
    {
	colorRule1color2 = color;
	mSudFrame->colorRule1color2 = colorRule1color2;

    }
}

void MyMainWindow::ChangeColorRule1color3()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorRule1color3 1\n");
#endif
    QColor color = QColorDialog::getColor(colorRule1color3,this);
    if ( color.isValid() )
    {
	colorRule1color3 = color;
	mSudFrame->colorRule1color3 = colorRule1color3;

    }
}

void MyMainWindow::ChangeColorRule2color1()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorRule2color1 1\n");
#endif
    QColor color = QColorDialog::getColor(colorRule2color1,this);
    if ( color.isValid() )
    {
	colorRule2color1 = color;
	mSudFrame->colorRule2color1 = colorRule2color1;

    }
}

void MyMainWindow::ChangeColorRule2color2()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorRule2color2 1\n");
#endif
    QColor color = QColorDialog::getColor(colorRule2color2,this);
    if ( color.isValid() )
    {
	colorRule2color2 = color;
	mSudFrame->colorRule2color2 = colorRule2color2;

    }
}

void MyMainWindow::ChangeColorRule2color3()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorRule2color3 1\n");
#endif
    QColor color = QColorDialog::getColor(colorRule2color3,this);
    if ( color.isValid() )
    {
	colorRule2color3 = color;
	mSudFrame->colorRule2color3 = colorRule2color3;
    }
}

void MyMainWindow::ChangeColorRule3color1()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorRule3color1 1\n");
#endif
    QColor color = QColorDialog::getColor(colorRule3color1,this);
    if ( color.isValid() )
    {
	colorRule3color1 = color;
	mSudFrame->colorRule3color1 = colorRule3color1;
    }
}

void MyMainWindow::ChangeColorRule3color2()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorRule3color2 1\n");
#endif
    QColor color = QColorDialog::getColor(colorRule3color2,this);
    if ( color.isValid() )
    {
	colorRule3color2 = color;
	mSudFrame->colorRule3color2 = colorRule3color2;
    }
}

void MyMainWindow::ChangeColorRule3color3()
{
#ifdef _DEBUG
	printf ("MyMainWindow::ChangeColorRule3color3 1\n");
#endif
    QColor color = QColorDialog::getColor(colorRule3color3,this);
    if ( color.isValid() )
    {
	colorRule3color3 = color;
	mSudFrame->colorRule3color3 = colorRule3color3;
    }
}
/******************************************************/
/******************************************************/

/*
void MyMainWindow::Set_MinimumSize(int width, int height)
{
	setMinimumSize (width, height);
#ifdef _DEBUG
	printf("setMinWidnwSize()\n");
#endif
	setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred);
}*/



/***************************/
/* SAVE AND LOAD FUNCTIONS */
/***************************/

/******************/
/* SaveGameAs     */
/********************************************************************/
/*  This function open a QFile dialog and write the sudoku board    */
/*  in the file.                                                    */
/********************************************************************/
void MyMainWindow::SaveGameAs()
{
#ifdef _DEBUG
	printf ("MyMainWindow::SaveGameAs 1\n");
#endif
	if (mSudFrame->sudoku_board == NULL)
	{
		return;
	}

	/* If the sudoku have errors we don't save.*/
	if (mSudFrame->ThisSudokuHasErrors)
	{
		if (mSudFrame->SudokuSolutionwasfound)
		{
			MyLoadLastSave Dialog;
			if ( Dialog.exec() )
			{
				QString path = QDir::currentPath();
				path.append("/system/NOERRSAV.sdk");

	                        MY_ASSERT (mSudFrame->isPlayerPlaying()==true);
				mSudFrame->setComputerControl();
				SetAllSudokuColors();
				mSudFrame->readFile(path);
				
				resize(width(),height());
				mSudFrame->RestartUndo();
	                        MY_ASSERT (mSudFrame->isPlayerPlaying()==false);
				mSudFrame->setPlayerControl();
				return;
			}
			else return;
		}
		else
		{
		 	QMessageBox A(tr("Error"), tr("The board can not be saved. \n It has errors\n Sorry :,,-( \n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

			A.exec();
			return;
		}
		
	}

	QString fileName = QFileDialog::getSaveFileName(this, tr("Save SudokuSensei"),"saves",tr("Sudoku Files (*.sdk)"));

	if (fileName.isEmpty()) return;

	mSudFrame->writeFile(fileName);
}

/****************/
/* LoadGame     */
/********************************************************************/
/*  This function open a QFile dialog and read the sudoku board     */
/*  in the file.                                                    */
/********************************************************************/
void MyMainWindow::LoadGame()
{
	bool ret = false;
#ifdef _DEBUG
	printf ("MyMainWindow::LoadGame 1\n");
#endif
	MY_ASSERT (mSudFrame->isPlayerPlaying()==true);
	mSudFrame->setComputerControl();
	QString fileName = QFileDialog::getOpenFileName(this, tr("Load SudokuSensei"),"saves",tr("Sudoku Files (*.sdk)"));

	if (fileName.isEmpty())
	{
	        MY_ASSERT (mSudFrame->isPlayerPlaying()==false);
        	mSudFrame->setPlayerControl();
	        return;
        }


	SetAllSudokuColors();

	ret = mSudFrame->readFile(fileName);

//-- If readFile() returns false, go back home!
//   (initial state of the application with a classic sudoku)
	if (ret == false )
	{
	        MY_ASSERT (mSudFrame->isPlayerPlaying()==false);
		mSudFrame->CreateSimpleSudoku();
		EnableGame();
        	mSudFrame->setPlayerControl();
		return;
	}
	
	resize(width(),height());
	mSudFrame->RestartUndo();
	MY_ASSERT (mSudFrame->isPlayerPlaying()==false);
	mSudFrame->setPlayerControl();

	EnableGame();
}

/**********************/
/* SetAllSudokuColors */
/********************************************************************/
/*  This function updates the SudokuView color to save and selected */
/*  user colors.                                                    */
/********************************************************************/
void MyMainWindow::SetAllSudokuColors()
{
#ifdef _DEBUG
	printf ("MyMainWindow::SetAllSudokuColors 1\n");
#endif
	/* INITIAL SET OF COLORS */
	if (!mSudFrame) return;

	mSudFrame->RepaintBackGround( colorBackground );
	mSudFrame->colorGrid = colorGrid;
	mSudFrame->colorBlock= colorBox;
	mSudFrame->colorCell= colorCell;
	mSudFrame->colorBigText= colorBigText ;
        mSudFrame->colorSmallText= colorSmallText ;
        mSudFrame->curColor= colorCursor;

	mSudFrame->colorHelpBox1 = colorHelpBox1;
	mSudFrame->colorHelpBox2 = colorHelpBox2;
	mSudFrame->colorHelpBox3 = colorHelpBox3;

	mSudFrame->colorRule1color1 = colorRule1color1;
	mSudFrame->colorRule1color2 = colorRule1color2;
	mSudFrame->colorRule1color3 = colorRule1color3;

	mSudFrame->colorRule2color1 = colorRule2color1;
	mSudFrame->colorRule2color2 = colorRule2color2;
	mSudFrame->colorRule2color3 = colorRule2color3;

	mSudFrame->colorRule3color1 = colorRule3color1;
	mSudFrame->colorRule3color2 = colorRule3color2;
	mSudFrame->colorRule3color3 = colorRule3color3;

	/* Activate Helps Mode Fuctions */
	mSudFrame->setIsActiveHelpBox(FlagHelpMode);

}

/****************/
/* SaveMovement */
/********************************************************************/
/*  This function calls SudokuView->SaveMovement                    */
/* TODO: Test. Necesarry ??                                         */
/********************************************************************/
void MyMainWindow::SaveMovement()
{
#ifdef _DEBUG
	printf ("MainWindow::SaveMovement 1\n");
#endif
	
	if (!mSudFrame) return;
	mSudFrame->SaveMovement();
}

/********/
/* Undo */
/********************************************************************/
/*  This function calls SudokuView->Undo                            */
/* TODO: Test. Necesarry ??                                         */
/********************************************************************/
void MyMainWindow::Undo()
{
#ifdef _DEBUG
	printf ("MyMainWindow::Undo 1\n");
#endif
	if (!action_Undo->isEnabled() ) return;
	if (!mSudFrame) return;
	mSudFrame->Undo();
}

/********/
/* Redo */
/********************************************************************/
/*  This function calls SudokuView->Undo                            */
/* TODO: Test. Necesarry ??                                         */
/********************************************************************/
void MyMainWindow::Redo()
{
#ifdef _DEBUG
	printf ("MyMainWindow::Redo 1\n");
#endif
	if (!action_Redo->isEnabled() ) return;
	if (!mSudFrame) return;
	mSudFrame->Redo();
}

/**************/
/* EnableUndo */
/********************************************************************/
/*  This function enable and disable Undo menu option               */
/********************************************************************/
void MyMainWindow::EnableUndo(bool set)
{
#ifdef _DEBUG
	printf ("MyMainWindow::EnableUndo 1\n");
#endif
	action_Undo->setEnabled(set);
}

/**************/
/* EnableRedo */
/********************************************************************/
/*  This function enable and disable Redo menu option               */
/********************************************************************/
void MyMainWindow::EnableRedo(bool set)
{
#ifdef _DEBUG
	printf ("MyMainWindow::EnableRedo 1\n");
#endif
	action_Redo->setEnabled(set);
}

/**********************/
/* EnableShowSolution */
/********************************************************************/
/*  This function enable and disable ShowSolution menu option       */
/********************************************************************/
void MyMainWindow::EnableShowSolution(bool set)
{
#ifdef _DEBUG
	printf ("MyMainWindow::EnableShowSolution 1\n");
#endif
	actionShow_Solution->setEnabled(set);
}

/****************/
/* clickEnlarge */
/*********************************************************************/
/*  This function show and hide EnlargeWindow when the user click in */
/*  actionEnlarge_Window      					     */
/*********************************************************************/
void MyMainWindow::clickEnlarge()
{
#ifdef _DEBUG
	printf ("MyMainWindow::clickEnlarge 1\n");
#endif
	
      if (!mSudFrame) return;
      if (!mSudFrame->mEnlargeWindow) return;
	if (actionEnlarge_Window->isChecked())
	{
		if ( mSudFrame->ShowEnlargeWindow() );
		//else ( mSudFrame->CreateEnlargeWindow());
		mSudFrame->flagEnlarge = true;
	}
	else
	{
		mSudFrame->HideEnlargeWindow() ;
		mSudFrame->flagEnlarge = false;
	}
}

/*****************/
/* updateEnlarge */
/*********************************************************************/
/*  This function update the actionEnlage_Window before it is showed.*/
/*  The action is checked if EnlargeWindow window is showed.         */
/*  The action is unchecked if EnlargeWindow window is hidden.       */
/*********************************************************************/
void MyMainWindow::updateEnlarge(QAction *action)
{
	#ifdef _DEBUG
		printf ("MyMainWindow::updateEnlarge 1\n");
	#endif
	if (!mSudFrame) return;
	if (!mSudFrame->mEnlargeWindow) return;
	if (mSudFrame->mEnlargeWindow->isHidden())
	{
		actionEnlarge_Window->setChecked(false);
		mSudFrame->flagEnlarge = false;
	}
	else
	{
		actionEnlarge_Window->setChecked(true);
		mSudFrame->flagEnlarge = true;
		
	}
}
/*****************/
/* keyPressEvent */
/*********************************************************************/
/*  This function capture key events. Key events captured: 	     */
/*  Ctrl + Z -> Undo.						     */
/*  Ctrl + X -> Redo.						     */
/*********************************************************************/
void MyMainWindow::keyPressEvent (QKeyEvent * event)
{
	#ifdef _DEBUG
		printf ("MyMainWindow::keyPressEvent 1\n");
	#endif
	if (event->modifiers() & Qt::ControlModifier)
	{
		if (event->key() == Qt::Key_Z)
		{
			Undo();
			event->ignore ();
			activateWindow ();
			return;
		}
		else if (event->key() == Qt::Key_X)
		{
			Redo();
			event->ignore ();
			activateWindow();
			return;
		}

	}
	QMainWindow::keyPressEvent (event);

}

/***************/
/* LoadRawGame */
/*********************************************************************/
/*  This function load a Sudoku gived in a QString		     */
/*********************************************************************/
void MyMainWindow::LoadRawGame(QString  stringSudoku )
{
	QByteArray auxStringSudoku = stringSudoku.toLatin1();
	mSetBoard (mSudFrame->sudoku_board, auxStringSudoku.data() , 0,0);
}


/*************/
/* mSetBoard */
/*********************************************************************/
/*  This function load a Sudoku gived in a char array		     */
/*********************************************************************/
int MyMainWindow::mSetBoard (SudokuBoard * b,
              const char * str,
              const char * symbols,
              const char * empty)
{
        t_board_p pBoard = b->Board ();
        int cell, val, resul;

        resul = SetBoard (pBoard, str, symbols, empty);
        
        for (cell=0; cell<b->NumCells(); cell++)
        {
            val = GetSymbolOfCell (pBoard, cell);
            
            if (val != -1)
            {
#ifdef _DEBUG
	  	printf("valores de c= %d | j= %c \n", cell , b->pCl, symbols[val]);
#endif

		b->GCell(cell)->setFinalUserValue( (val ) ); /* TODO: Change to setFinalCompValue */
		b->GCell(cell)->canChange  =  false ;
		b->GCell(cell)->FvActivate  = true ;
            }
        }

        return resul;
}

/****************/
/* ShowSolution */
/********************************************************************************************/
/*  This function open a HTML dialog and show the solution of the sudoku if it is possible. */
/********************************************************************************************/
void MyMainWindow::ShowSolution()
{

#ifdef x_DEBUG
	printf ("MyMainWindow::ShowSolution 1\n");
#endif
	if(!mSudFrame->sudoku_board) return;


	if(mSudFrame->SudokuSolutionwasfound)
	{
		if (mSudFrame->ThisSudokuHasErrors)
		{
			QMessageBox A(tr("This Sudoku has errors"), tr("This Sudoku has errors\nI am going to show you the real solution\n Please, check it and study more next time\n"), QMessageBox::Information, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
			A.exec();
		}
		
			ShowSolutionDialog MyDialog;
			MyDialog.exec();
	}
	else if (mSudFrame->ThisSudokuHasErrors)
	{
		QMessageBox A(tr("This Sudoku has errors"), tr("This Sudoku has errors\n\n...mmm...\n\n Please check it."), QMessageBox::Information, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
		A.exec();
	}
	else
	{
		QMessageBox A(tr("I have not found the solution yet"), tr("I'm thinking.\n...mmm...\n\n I have not found the solution yet\n\nPlease, ask me about it later"), QMessageBox::Information, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
		A.exec();
	}

#ifdef x_DEBUG
	printf ("MyMainWindow::ShowSolution END\n");
#endif
}


/*************/
/* ShowAbout */
/****************************************************************************/
/*  This function open a HTML dialog and show information about the authors */
/****************************************************************************/
void MyMainWindow::ShowAbout()
{
	bool english= false;
	if (activelanguage.compare(chars_espanol) == 0)
	{
		english= false;
	}
	else if (activelanguage.compare(chars_english) == 0)
	{
		english= true;
	}
	ShowAboutDialog MyDialog(english);
	MyDialog.exec();
}

/**************/
/* ShowManual */
/************************************************************************/
/*  This function open a HTML dialog and show the manual of the program */
/************************************************************************/
void MyMainWindow::ShowManual()
{
	bool english= false;
	if (activelanguage.compare(chars_espanol) == 0)
	{
		english= false;
	}
	else if (activelanguage.compare(chars_english) == 0)
	{
		english= true;
	}
	ShowManualDialog MyDialog(english);
	MyDialog.exec();
}

/****************/
/* ShowTutorial */
/**************************************************************************/
/*  This function open a HTML dialog and show the tutorial of the program */
/**************************************************************************/
void MyMainWindow::ShowTutorial()
{
	bool english= false;
	if (activelanguage.compare(chars_espanol) == 0)
	{
		english= false;
	}
	else if (activelanguage.compare(chars_english) == 0)
	{
		english= true;
	}
	ShowTutorialDialog MyDialog(english);
	MyDialog.exec();
}

/*//////////////////////////////////////////////////////////// */
