/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: mainwindow.h  * Status: stable * Version: 2.00 * last changes in version: 0.50	*/
/************************************************************************************************/
/*  This file contains the definitions of MyMainWindow class					*/
/*												*/
/*  The dialog is defined in Dmainwindow.ui							*/
/*												*/
/*  The MyMainWindow class functions are defined in mainwindow.ccp				*/
/************************************************************************************************/



#ifndef _MAINWINDOW_H_
#define _MAINWINDOW_H_
#include <QMainWindow>
#include <QDialog>
#include <QTranslator>
#include <QString>
#include <QColor>
#include "SudokuView.h"
#include "ui_Dmainwindow.h"
#include "ui_Dloadlastsave.h"

#define MODE_HELP	  FlagHelpMode
#define MODE_NO_HELP 	 !FlagHelpMode
#define MODE_LITTLE_HELP (FlagHelpMode & !FlagExtraHelpMode)
#define MODE_SENSEI      (FlagHelpMode &  FlagExtraHelpMode)

class SudokuBoard;

class MyLoadLastSave : public QDialog, Ui_LoadLastSave
{
	Q_OBJECT
	public:
	MyLoadLastSave( QWidget * parent = 0, Qt::WindowFlags f = 0 ) : QDialog::QDialog( parent, f) { setupUi(this);}

};


class MyMainWindow : public QMainWindow, Ui_MainWindow
{
    Q_OBJECT

public:


/*  VARIABLES  */
    QApplication * app;                 /* Parent Application */
    QTranslator appTranslator;          /* Translator of Application */
    QString qmPath;

			/* Path of application */
					/* Important to find languages files */
    
    QActionGroup *languageActionGroup;  /* Group of languages actions      */
  			/* It cannot be done in QtDesigner */

  //QColorDialog ColorDialog;		

    SudokuView* Frame() const;

/* HELP MODE VARIABLES      ( FlagHelpMode ,  FlagExtraHelpMode )*/
/* NO HELP                      ( 0  ,   0 ) */
/* LITTLE HELP                  ( 1  ,   0 ) */
/* SENSEI HELP                  ( 1  ,   1 ) */
    bool FlagHelpMode;
    bool FlagExtraHelpMode;


/*  LANGUAGE VARIABLES */
    QString activelanguage;
    QString chars_espanol;
    QString chars_english;
//  QString <ADD MORE LANGUAGES HERE> 
// /* 1 of 3 : Changes to ADD MORE LANGUAGES */


/*  COLOR VARIABLES */

    QColor colorBackground;
    QColor colorGrid;
    QColor colorBox;
    QColor colorCell;
    QColor colorBigText;
    QColor colorSmallText;
    QColor colorCursor;

/*  COLOR HELP VARIABLES */
    QColor colorHelpBox1;
    QColor colorHelpBox2;
    QColor colorHelpBox3;

    QColor colorRule1color1;
    QColor colorRule1color2;
    QColor colorRule1color3;

    QColor colorRule2color1;
    QColor colorRule2color2;
    QColor colorRule2color3;

    QColor colorRule3color1;
    QColor colorRule3color2;
    QColor colorRule3color3;


/**************/
/*  FUNCTIONS */
/**************/

/* CONSTRUCTOR */
    MyMainWindow( QApplication *); /* Creates the object*/

    ~MyMainWindow(); /* Destroys the object*/

    void setupUi(QMainWindow*);    /* Configure all the aplication and */
                                   /* calls setupUi of Ui_MainWindow   */

/* That Function activate the diferents helps modes */
    void UpdateGameMode(bool flag1, bool flag2);
 //   void paintEvent(QPaintEvent *event);
    void resizeEvent(QResizeEvent *event);

    void readSettings();

    signals:
    void LanguageChanged();

public  slots:
    void writeSettings();
    void CreateNewGame();
    void CreateNewBoard();

    void switchLanguage(QAction *);

    void ShowSolution();

    void EnableGame();

    void Printing();

    void GenerateGame( short level );

/* Change Color Functions */
    void ChangeColorBackground();
    void ChangeColorGrid();
    void ChangeColorBox();
    void ChangeColorCell();
    void ChangeColorText();
    void ChangeSmallColorText();
    void ChangeColorCursor();
    void ChangeColorHelpBox1();
    void ChangeColorHelpBox2();
    void ChangeColorHelpBox3();
    void ChangeColorRule1color1();
    void ChangeColorRule1color2();
    void ChangeColorRule1color3();
    void ChangeColorRule2color1();
    void ChangeColorRule2color2();
    void ChangeColorRule2color3();
    void ChangeColorRule3color1();
    void ChangeColorRule3color2();
    void ChangeColorRule3color3();
/******************************/
    void SaveGameAs();
    void LoadGame();

    void Undo();
    void Redo();
    void SaveMovement();
    void ShowAbout();
    void ShowManual();
    void ShowTutorial();

    void EnableUndo        (bool set);
    void EnableRedo        (bool set);
    void EnableShowSolution(bool set);

    void clickEnlarge();
    void updateEnlarge(QAction * action);
    void keyPressEvent (QKeyEvent * event);
    //void Set_MinimumSize(int width, int height);
    int mReadBoard (SudokuBoard * b, FILE * pf, const char * symbols, const char * empty);

   
    void UpdateNoHelp()        {  UpdateGameMode(false, false );}
    void UpdateLittleHelp()    {  UpdateGameMode(true , false );}
    void UpdateHelpSensei()    {  UpdateGameMode(true , true  );}

protected:
	void closeEvent(QCloseEvent *event);
private:
   void SetAllSudokuColors();
   void LoadRawGame(QString  stringSudoku );
    int mSetBoard (SudokuBoard * b,
              const char * str,
              const char * symbols,
              const char * empty);

};
#endif
