/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: newgame.cpp    * Status: stable * Version: 2.0 * finished in version: 0.26 		*/
/************************************************************************************************/
/*  This file contains class functions of NewGame class                                  	*/
/*        									      		*/
/*   This class is defined in newgame.h						     		*/
/*												*/
/*   It's called by MyMainWindow class defined in mainwindow.h, mainwindow.cpp, Dmainwindow.ui	*/
/*												*/
/************************************************************************************************/

#include <QtWidgets>
#include "newgame.h"
#include <QProcess>
#include "dmem.h"

NewGame::NewGame(QWidget *parent)
		: QDialog(parent)
{
	/* Text of the GUI */
	min 	     = DM_NEW ( new QLabel(tr("min")) );
	medium       = DM_NEW ( new QLabel(tr("EASY")) );
	max 	     = DM_NEW ( new QLabel(tr("max")) );
	leveldif1    = DM_NEW ( new QLabel(tr("&Level of ")) );
	leveldif2    = DM_NEW ( new QLabel(tr("Difficulty")) );
	BoardEmpty   = DM_NEW ( new QRadioButton(tr("&Empty Board")) );
	BoardGenerate= DM_NEW ( new QRadioButton(tr("&Generate Problem")) );
	BoardWrited  = DM_NEW ( new QRadioButton(tr("&Import from Text")) );
	highdif      = DM_NEW ( new QCheckBox   (tr("&Difficulty")) );

	setOutput (-1);

	BoardGenerate->toggle();
	// Configuramos el slider
	level	     = DM_NEW (new QSlider(Qt::Horizontal) );
	level->setMinimum(DIF_MIN);
	level->setMaximum(DIF_MAX);
	level->setValue(DIF_DEFAULT);
	
	leveldif1->setBuddy(level);  // Select TABs connection 

	RadioGroup   = DM_NEW ( new QButtonGroup() );
	RadioGroup->addButton(BoardEmpty);
	RadioGroup->addButton(BoardGenerate);
	RadioGroup->addButton(BoardWrited);

	TextBoard = DM_NEW ( new QTextEdit() );

	//Buttons Acept y Close
	botAcept = DM_NEW ( new QPushButton(tr("&Ok")) );
	botClose = DM_NEW ( new QPushButton(tr("&Close")) );

	botAcept->setDefault(true);

	/* CONNECTIONS */

	// TODO: Need more connections
	connect(botClose, SIGNAL(clicked()), this, SLOT(close()));
	connect(botAcept, SIGNAL(clicked()), this, SLOT(aceptClicked()));

	connect(highdif, SIGNAL(clicked()), this, SLOT(changeEasy()));

	/********************/

	// Put buttons on layouts

	// Layout of slider's text
	LayoutTextSlider = DM_NEW ( new QHBoxLayout );
	LayoutTextSlider->addWidget(min);
	LayoutTextSlider->addStretch();
	LayoutTextSlider->addWidget(medium);
	LayoutTextSlider->addStretch();
	LayoutTextSlider->addWidget(max);
	
	//LayoutA: Layout of slider's text  +  slider
	LayoutSliderPlus = DM_NEW ( new QVBoxLayout );
	LayoutSliderPlus->addLayout(LayoutTextSlider);
	LayoutSliderPlus->addWidget(level);

	//LayoutB: Fusion of leveldif Labels
	LayoutLevelDif = DM_NEW ( new QVBoxLayout );
	LayoutLevelDif->addWidget(leveldif1);
	LayoutLevelDif->addWidget(leveldif2);

	// LayoutDIF = LayoutA  + LayoutB  + CheckBox(highdif)
	LayoutDIF = DM_NEW ( new QHBoxLayout );
	LayoutDIF->addLayout(LayoutLevelDif);
	LayoutDIF->addLayout(LayoutSliderPlus);
	LayoutDIF->addWidget(highdif);

	// Layout of Buttons
	LayoutButtons = DM_NEW ( new QHBoxLayout );
	LayoutButtons->addStretch();
	LayoutButtons->addWidget(botAcept);
	LayoutButtons->addWidget(botClose);

	// Final Layout 
	MainLayout = DM_NEW ( new QVBoxLayout );
	MainLayout->addWidget(BoardEmpty);
	MainLayout->addWidget(BoardGenerate);
	MainLayout->addLayout(LayoutDIF);
	MainLayout->addWidget(BoardWrited);
	MainLayout->addWidget(TextBoard);
	MainLayout->addLayout(LayoutButtons);

	// See the Final Layout:
	setLayout(MainLayout);

	setWindowTitle(tr("New Game"));

	//Ajust size
	//TODO: Review and change if is necessary
	//setFixedHeight(sizeHint().height());

}
void NewGame::aceptClicked()
{
	if (BoardEmpty->isChecked())
	{
		setOutput(-3);
	}
	else if (BoardWrited ->isChecked())
	{
		setOutput(-2);// TODO:  GenerateBoard(TextBoard);
	}
	else if (BoardGenerate->isChecked())
	{
		if (highdif->isChecked())
			setOutput(-1);
		else
		setOutput ( level->value() );
	}
	else 
	{
		setOutput (-4);/* ERROR ERROR */
		//Qprint (tr("Error in NewGame::aceptClicked\n"));
	}
	 QDialog::accept();
}

NewGame::~NewGame()
{
	delete DM_DEL (min	    );
	delete DM_DEL (medium       );
	delete DM_DEL (max 	    );
	delete DM_DEL (leveldif1    );
	delete DM_DEL (leveldif2    );
	delete DM_DEL (BoardEmpty   );
	delete DM_DEL (BoardGenerate);
	delete DM_DEL (BoardWrited  );
	delete DM_DEL (highdif      );

	delete DM_DEL (level        );
	delete DM_DEL (RadioGroup   );
	delete DM_DEL (TextBoard    );
	delete DM_DEL (botAcept     );
	delete DM_DEL (botClose     );

	delete DM_DEL (LayoutTextSlider );
	delete DM_DEL (LayoutSliderPlus );
	delete DM_DEL (LayoutLevelDif	);
	delete DM_DEL (LayoutDIF	);
	delete DM_DEL (LayoutButtons	);
	delete DM_DEL (MainLayout	);
}

void NewGame::changeEasy()
{
	if(highdif->isChecked())
	{
		level->setEnabled(false);
	}
	else 
	{
		level->setEnabled(true);
	}
}
