/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: newtable.cpp    * Status: stable * Version: 1.0 * finished in version: 0.29           */
/************************************************************************************************/
/*  This file contains functions of MyNewBoard class                                  		*/
/*        									      		*/
/*   This dialog is defined in Dnewtable.ui                                           		*/
/*   This class  is defined in newtable.h                                             		*/
/*												*/
/*   It's called by MyMainWindow class defined in mainwindow.h, mainwindow.cpp, Dmainwindow.ui	*/
/*												*/
/*   It calls MyEditor     class defined in editor.cpp, editor.h, Deditor.ui and                */
/*              SudokuWidget class defined in SudokuWidget.cpp, SudokuWidget.h                  */
/************************************************************************************************/
#include <QtWidgets>
#include "newtable.h"
#include "SudokuView.h"
#include "SudokuBoard.h"
#include "editor.h"
#include "dmem.h"

/***************/
/* Constructor */
/********************************************/
/* This function builds a MyNewBoard object */
/********************************************/
MyNewBoard::MyNewBoard(QWidget *parent): QDialog(parent)
{
	father = (MyMainWindow *) parent;
	setupUi(this);
}
/****************/
/*  setupUi     */
/****************************************************************************************/
/* This function creates and show the dialog, makes all connections and setup listView. */
/****************************************************************************************/

void MyNewBoard::setupUi(QDialog *NewBoard)
{
	
	/* Setup Dialog built with QT Designer */
	Ui_NewBoard::setupUi(NewBoard);
	mPreview->hide();
	/* CONNECTIONS */

	/* Accept Button */
	connect(ButAcept,SIGNAL(clicked()),this    , SLOT (CreateBoard()) );
	connect(this	,SIGNAL(AllOk())  ,parent(), SLOT (repaint()    ) );
//	connect(this	,SIGNAL(AllOk())  ,this    , SLOT (close()      ) );
	connect(this	,SIGNAL(AllOk())  ,this    , SLOT (accept()     ) );
	
	/* Buttons and ChangePreview */
	connect(rButBoxChildren, SIGNAL(toggled(bool)    ),this,SLOT (ChangePreview(bool)) );
	connect(rButBoxClassic , SIGNAL(toggled(bool)    ),this,SLOT (ChangePreview(bool)) );
	connect(rButBoxOther   , SIGNAL(toggled(bool)    ),this,SLOT (ChangePreview(bool)) );
	connect(spinBoxWidth   , SIGNAL(valueChanged(int)),this,SLOT (ChangePreview(int) ) );
	connect(spinBoxHeight  , SIGNAL(valueChanged(int)),this,SLOT (ChangePreview(int) ) );
	connect(rButGridOne    , SIGNAL(toggled(bool)    ),this,SLOT (ChangePreview(bool)) );
	connect(rButGridSamurai, SIGNAL(toggled(bool)    ),this,SLOT (ChangePreview(bool)) );
	connect(checkBoxPreview, SIGNAL(toggled(bool)    ),this,SLOT (ChangePreview(bool)) );

        connect(this           , SIGNAL(listChanged(bool)),this,SLOT (ChangePreview(bool)) );
	
	/* Connections Selection Board Files Buttons */
	
	connect(ButNewBoard  , SIGNAL(clicked()),this,SLOT (NewBoard()  ) );
	connect(ButEditBoard ,SIGNAL(clicked() ),this,SLOT (EditBoard() ) );
	connect(ButEraseBoard,SIGNAL(clicked() ),this,SLOT (EraseBoard()) );

	
	connect(listView     ,SIGNAL(clicked(const QModelIndex & )), this,SLOT( SelectFile(const QModelIndex &) ));

	/** Added in proyect 0.29 **/
	connect(listView     ,SIGNAL(selectChanged(const QModelIndex & )), this,SLOT( SelectFile(const QModelIndex &) ));

	/* Setup listView */

	/* Select only *.brd in current path */
	QStringList filterfiles;
	filterfiles << "*.brd";
	mDir = DM_NEW (new QDirModel (filterfiles, QDir::Files,QDir::Name) );
	/* Start with no selected file*/	
	isSelected = false;

	/* Setup listView internal parameters*/
	listView->setModel(mDir);
	listView->setRootIndex(mDir->index(QDir::currentPath()+"/board"));

	/* Setup listView visual parameters*/
	listView->header()->hide();
	listView->setSelectionMode (QAbstractItemView::SingleSelection);
	listView->setColumnHidden(1, true);
	listView->setColumnHidden(2, true);
	listView->setColumnHidden(3, true);
	listView->setColumnHidden(4, true);

	ChangePreview(1);
}


/****************/
/*  CreateBoard */
/**********************************************************************/
/* This function creates the final board and sends it to MyMainWindow */
/*					 			      */
/* NOTE: Creator Boards functions are in SudokuView class             */
/*       We need father pointer to call it                            */
/**********************************************************************/
void MyNewBoard::CreateBoard()
{
	#ifdef _DEBUG
		printf ("MyNewBoard::CreateBoard 1\n");
	#endif
	/* Tree with selections options */
	if ( rButBoxChildren->isChecked())
	{
		if ( rButGridOne->isChecked())
		{
			/* 1 Grid and Children box (2x3) selected */
			int  xg, yg;
			xg=0;
			yg=0;
			father->Frame()->CreateCustom( 1,&xg,&yg,2,3,0);
		}
		else if (rButGridSamurai->isChecked())
		{
			/* Samurai Grid and Children box (2x3) selected */
			father->Frame()->CreateSamurai(2,3,0);
		}
		else 
		{
			if (isSelected==true)
			{
				/* Custom Grid Selected */
				#ifdef _DEBUG
					printf ("MyNewBoard::CreateBoard A\n");
				#endif
				/* added proyect 0.51 check if there are less than 1000 cells */
				if (nmGrids * h * w * h * w > 1000)
				{
					QMessageBox A(tr("Board too big"), tr("The board wasn't be used because it is too big. \n Sudoku Sensei v2.0 only works with board with less than 1000 cells\n Sorry :,,-(\n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
					A.exec();
				 	return;
				}
				/**/
				father->Frame()->CreateCustom(nmGrids,GridsX,GridsY, h,w,0);
			}
		}
	
	}
	else if (rButBoxClassic->isChecked())
	{
		if (rButGridOne->isChecked())
		{
			/* 1 Grid and Classic box (3x3) selected */
			father->Frame()->CreateSimpleSudoku();
		}
		else if (rButGridSamurai->isChecked())
		{
			/*  Samurai Grid and Classic box (3x3) selected */
			father->Frame()->CreateSamurai(3,3,0);
		}
		
		else 
		{
			if (isSelected== false) return;
			/* Custom Grid Selected */
#ifdef _DEBUG
				printf ("MyNewBoard::CreateBoard B\n");
#endif
			/* added proyect 0.51 check if there are less than 1000 cells */
			if (nmGrids * h * w * h * w > 1000)
			{
				QMessageBox A(tr("Board too big"), tr("The board wasn't be used because it is too big. \n Sudoku Sensei v2.0 only works with board with less than 1000 cells\n Sorry :,,-(\n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
				A.exec();
			 	return;
			}
			/**/
			father->Frame()->CreateCustom(nmGrids,GridsX,GridsY, h,w,0);
			
		}
	}
	else
	{
		if (rButGridOne->isChecked())
		{
			/* 1 Grid and Custom box ( Width x Height) selected */
			int  xg, yg;
			xg=0;
			yg=0;
			/* added proyect 0.51 check if there are less than 1000 cells */
			if (spinBoxWidth->value() *  spinBoxHeight->value() > 30)
			{
				QMessageBox A(tr("Board too big"), tr("The board wasn't be used because it is too big. \n Sudoku Sensei v2.0 only works with board with less than 1000 cells\n Sorry :,,-(\n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
				A.exec();
			 	return;
			}
			/**/
			father->Frame()->CreateCustom(1,&xg,&yg, spinBoxWidth->value(), spinBoxHeight->value(),0);
		}
		else if (rButGridSamurai->isChecked())
		{
			/* added proyect 0.51 check if there are less than 1000 cells */
			if ( 5 * spinBoxWidth->value()  * spinBoxWidth->value() *  spinBoxHeight->value() *  spinBoxHeight->value() > 1000)
			{
				QMessageBox A(tr("Board too big"), tr("The board wasn't be used because it is too big. \n Sudoku Sensei v2.0 only works with board with less than 1000 cells\n Sorry :,,-(\n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
				A.exec();
			 	return;
			}
			/**/
			/* Samurai Grid and Custom box ( Width x Height) selected */
			father->Frame()->CreateSamurai(spinBoxWidth->value(), spinBoxHeight->value(),0);
		}
		else 
		{
			if (isSelected== false) return;
			/* Custom Grid Selected */
#ifdef _DEBUG
				printf ("MyNewBoard::CreateBoard C\n");
#endif
			/* added proyect 0.51 check if there are less than 1000 cells */
			if (nmGrids * h * w * h * w > 1000)
			{
				QMessageBox A(tr("Board too big"), tr("The board wasn't be used because it is too big. \n Sudoku Sensei v2.0 only works with board with less than 1000 cells\n Sorry :,,-(\n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
				A.exec();
			 	return;
			}
			/**/
			father->Frame()->CreateCustom(nmGrids,GridsX,GridsY, h,w,0);
		}
	}

	//TODO : delete this comment
	//CHECK FATHER SIZE IF IS IT SMALLER THAT MINIMUN SIZE PLEASE RESIZE 

	/* Set MyMainWindow minimum size to a better visualization */

#ifdef _DEBUG
	printf ("MyNewBoard::CreateBoard 2\n");
#endif
        t_board_p pBoard = father->Frame()->sudoku_board->Board ();
        int w = GetBoardWidth (pBoard);
        int h = GetBoardHeight (pBoard);
        
	father->setMinimumSize( 2 * (WIDTH_GRID_BORDER * (w + 1))+ _2WIDTH_W_BORDER , 
				2 * (WIDTH_GRID_BORDER * (h + 1)) + _2WIDTH_H_BORDER );
	father->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);

	/* Send signal which close all */
	emit AllOk();	

#ifdef _DEBUG
	printf ("MyNewBoard::CreateBoard end\n");
#endif
}

/******************/
/*  Destructor    */
/******************/
MyNewBoard::~MyNewBoard()
{
	delete DM_DEL (mDir);
}


/******************/
/*  ChangePreview */
/******************/

void MyNewBoard::ChangePreview( int i)
{
	if (i)
	ChangePreview( true);
}

/******************/
/*  ChangePreview */
/************************************************************************/
/* This function creates a board and sends it to mPreview which show it */
/* like preview                                                         */
/*                                                                      */
/* NOTE: These Creator Boards functions are in SudokuWidget class       */
/************************************************************************/

void MyNewBoard::ChangePreview( bool i)
{
	if  (checkBoxPreview->isChecked())   //Preview Enabled
	{
		/* Tree with selections options */
		if ( rButBoxChildren->isChecked()) 
		{
			if ( rButGridOne->isChecked())
			{
				/* 1 Grid and Children box ( 2 x 3) selected */
				int  xg, yg;
				xg=0;
				yg=0;
				mPreview->CreateCustom( 1,&xg,&yg,2,3,0);
			}
			else if (rButGridSamurai->isChecked())
			{
				/* Samurai Grid and Children box ( 2 x 3) selected */
				mPreview->CreateSamurai(2,3,0);
			}
			else      
			{
				 /* Custom Selected */
				if (isSelected== false) return;
				/* added proyect 0.51 check if there are less than 1000 cells */
				if (nmGrids * h * w * h * w > 1000)
				{
/*					QMessageBox A(tr("Board too big"), tr("The board wasn't be used because it is too big. \n Sudoku Sensei v2.0 only works with board with less than 1000 cells\n Sorry :,,-(\n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
					A.exec();*/
					mPreview->hide();
				 	return;
				}
				/**/
				mPreview->CreateCustom(nmGrids,GridsX,GridsY, h,w,0);
				
			}

		}
		else if (rButBoxClassic->isChecked())
		{
			if (rButGridOne->isChecked())
			{
				/* 1 Grid and Classic box ( 3 x 3) selected */
				mPreview->CreateSimpleSudoku();
			}
			else if (rButGridSamurai->isChecked())
			{
				/* Samurai Grid and Classic box ( 3 x 3) selected */
				mPreview->CreateSamurai(3,3,0);
			}
			else 
			{
				 /* Custom Selected */
				if (isSelected== false) return;
				/* added proyect 0.51 check if there are less than 1000 cells */
				if (nmGrids * h * w * h * w > 1000)
				{
/*					QMessageBox A(tr("Board too big"), tr("The board wasn't be used because it is too big. \n Sudoku Sensei v2.0 only works with board with less than 1000 cells\n Sorry :,,-(\n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
					A.exec();*/
					mPreview->hide();
				 	return;
				}
				/**/
				mPreview->CreateCustom(nmGrids,GridsX,GridsY, h,w,0);
				
			}

		}
		else //if (rButBoxOther->isChecked())
		{
			if (rButGridOne->isChecked())
			{
				/* 1 Grid and Custom box ( Width x Height) selected */
				int  xg, yg;
				xg=0;
				yg=0;
				/* added proyect 0.51 check if there are less than 1000 cells */
				if (spinBoxWidth->value() *  spinBoxHeight->value() > 30)
				{
/*					QMessageBox A(tr("Board too big"), tr("The board wasn't be used because it is too big. \n Sudoku Sensei v2.0 only works with board with less than 1000 cells\n Sorry :,,-(\n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
					A.exec();*/
					mPreview->hide();
				 	return;
				}
				/**/
				mPreview->CreateCustom(1,&xg,&yg, spinBoxWidth->value(), spinBoxHeight->value(),0);
			}
			else if (rButGridSamurai->isChecked())
			{
				/* added proyect 0.51 check if there are less than 1000 cells */
				if ( 5 * spinBoxWidth->value()  * spinBoxWidth->value() *  spinBoxHeight->value() *  spinBoxHeight->value() > 1000)
				{
/*					QMessageBox A(tr("Board too big"), tr("The board wasn't be used because it is too big. \n Sudoku Sensei v2.0 only works with board with less than 1000 cells\n Sorry :,,-(\n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
					A.exec();*/
					mPreview->hide();
				 	return;
				}
				/**/
				/* Samurai Grid and Custom box ( Width x Height) selected */
				mPreview->CreateSamurai(spinBoxWidth->value(), spinBoxHeight->value(),0);
			}
			else    
			{
				// Custom Selected
				if (isSelected== false) return;
				/* added proyect 0.51 check if there are less than 1000 cells */
				if (nmGrids * h * w * h * w > 1000)
				{
	/*				QMessageBox A(tr("Board too big"), tr("The board wasn't be used because it is too big. \n Sudoku Sensei v2.0 only works with board with less than 1000 cells\n Sorry :,,-(\n"), QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
					A.exec();*/
					mPreview->hide();
				 	return;
				}
				/**/
				mPreview->CreateCustom(nmGrids,GridsX,GridsY, h,w,0);
				
			}
		}
		
		mPreview->show();
	
		mPreview->repaint();
	}
	else
	{
		mPreview->hide();
	}
}

/*************/
/*  NewBoard */
/************************************************************************/
/* This function calls Board Editor and send it selected box parameters.*/
/* It updates listView When you finish with editor.                     */
/************************************************************************/
void MyNewBoard::NewBoard()
{
	/* Take selected width and heigt of box */
	if ( rButBoxChildren->isChecked())
	{
		w = 2;
		h = 3;
	}
	else if ( rButBoxClassic->isChecked())
	{
		w = 3; 
		h = 3;
	}
	else 
	{
		w= spinBoxWidth->value();
		h= spinBoxHeight->value();
	}


	/* Call Editor */
	MyEditor mEditor(this, w , h);
	mEditor.exec();


	// Refresh files in listView
	mDir->refresh();
	listView->setRootIndex(mDir->index(QDir::currentPath()+"/board"));
	isSelected = false;
}


/**************/
/*  EditBoard */
/************************************************************************/
/* This function calls Board Editor and send it selected box parameters */
/* and selected file. 						        */
/* It updates listView When you finish with editor.                     */
/************************************************************************/
void MyNewBoard::EditBoard()
{
	if (isSelected == true)
	{
		MyEditor mEditor(this, file);
		mEditor.exec();
	}
	// Refresh files.
	mDir->refresh();
	listView->setRootIndex(mDir->index(QDir::currentPath()+"/board"));
	isSelected = false;
}

/***************/
/*  SelectFile */
/**************************************************************************/
/* This function is called when you click on a listView file               */
/* This function save file name selected, load it to take board parameters */
/*  and emit listChanged signal which call ChangePreview                  */
/*                                                                        */
/* NOTE: LoadFile function is in MyEditor class                           */
/**************************************************************************/
void MyNewBoard::SelectFile(const QModelIndex & index)
{
	/* Save file name */
	file = mDir->fileName(index);
	isSelected=true;
	
	/* Load file and save board parameters */
	MyEditor mEditor(this, file);
	if ( !mEditor.LoadFile ( file, &nmGrids, GridsX, GridsY, &w, &h) ) return;


	// Preview con todo;
	emit listChanged(true);

}

/***************/
/*  EraseBoard */
/*****************************************/
/* This function erase the selected file */
/*****************************************/
void MyNewBoard::EraseBoard()
{
	// Remove file selected
	QDir dir(QDir::currentPath()+"/board");
	dir.remove(file);

	// Refresh files.
	mDir->refresh();
	listView->setRootIndex(mDir->index(QDir::currentPath()+"/board"));
	isSelected = false;
}
