/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: newtable.h    * Status: stable * Version: 1.0 * finished in version: 0.26 		*/
/************************************************************************************************/
/*  This file contains definition of MyNewBoard class                                  		*/
/*        									      		*/
/*   This dialog is defined in Dnewtable.ui                                           		*/
/*   The class functions are defined in newtable.cpp                                  		*/
/*												*/
/*   It's called by MyMainWindow class defined in mainwindow.h, mainwindow.cpp, Dmainwindow.ui	*/
/*												*/
/*   It calls MyEditor     class defined in editor.cpp, editor.h, Deditor.ui and                */
/*              SudokuWidget class defined in SudokuWidget.cpp, SudokuWidget.h                  */
/************************************************************************************************/

#ifndef NEWTABLE_H
#define NEWTABLE_H
#include <QDialog>
#include "ui_Dnewtable.h"
#include "mainwindow.h"
#include "SudokuWidget.h"

/********************************************************************************/
/* Class MyNewBoard                                                             */
/********************************************************************************/
/* This class creates and show a dialog when the user can select the game board */
/* The user can activate the Editor and manage board files                      */
/********************************************************************************/
class MyNewBoard : public QDialog, Ui_NewBoard
{
    Q_OBJECT

private:
	MyMainWindow * father; /* the parent MyMainWindow which creates this dialog */ 

	QDirModel * mDir;      /* Directory Model which find board files	*/
	QString file;          /* A String with file name selected		*/

	bool isSelected;	/* Flag. True when a file is selected in listView*/

	/* Board parameters */

	int w,h;		/* width and height of board's box	*/	

	int GridsX[15];	       /* Array with Position X of Grids        */
	int GridsY[15];        /* Array with Position Y of Grids        */
	int nmGrids;           /* Number of Grids which the board has		*/
			       /* It is number of elements of GridsX and GridsY */



public:
	
 	MyNewBoard(QWidget *);   /* Constructor */
	~MyNewBoard();

	void setupUi(QDialog *NewBoard); /* Setup Dialog and connections*/
					 /* Called by Constructor 	*/

signals:
	void listChanged(bool);/* Emited when listView has Changed. And must be updated	*/

	void AllOk();	       /* Emited when Accept Button is pressed and there isn't errors */

public slots:

	void CreateBoard();     /* Function which creates the final Board and sends it to MyMainWindow */

	/* Preview Functions */
	void ChangePreview(bool); /* Funcion which changes and update preview */
	void ChangePreview( int );/* Calls ChangePreview(bool). Created for compatibility whith signals*/

	/*  Functions which works with Board files and Board Editor */
	void NewBoard();        /* Calls Editor and order it to make a new board */
	void EditBoard();       /* Calls Editor and order it to edit the selected board in listView */
	void EraseBoard();	/* Erases selected Board in listView */

	void SelectFile(const QModelIndex & index); /* */

};
#endif
