/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: sudoku-creator.cpp  * Status: stable * Version: 2.00 * last changes in version: 0.50	*/
/************************************************************************************************/
/*  This file contains the functions which are used by CreatorThread				*/
/*												*/
/*  This functions are based in sudokusensei v1.04 functions					*/
/*												*/
/************************************************************************************************/
#include "sudoku-creator.h"
#include "thread-creator.h"
#include <QList>
#include "dmem.h"

inline bool checkCancel(CreatorThread * pThread)
{
  if (pThread)
    return pThread->flagCancel;
  else
    return false;
}

int sudokuCreator ( QStringList arguments,CreatorThread * pThread)
//int arguments.size(), char *argv[])
{
  t_params par;
  t_board_p b;
  int n=0, nCl=0, serial=0;
  int * cells=NULL, * symbols=NULL, * masks=NULL;
  char * given=NULL;
//  FILE * pf=NULL;

  srand ((unsigned)time(NULL));

  // PrintGPL (stdout);  /*Show License */

  if (ParseParams (&par, arguments) != 0)
  {
    SYSTEM_PAUSE;
    FreeParams (&par);
    return -1;
  }

  pThread->advance( 10);

  if (checkCancel(pThread) )
  {
    SYSTEM_PAUSE;
    FreeParams (&par);
    return -1;
  }
#ifdef _DEBUG
  printf ("Run \"demo5-inverse-creator -h\" for help.\n");
  printf ("Creating %d problems...\n", par.num);
#endif

  if (par.samurai)
    b = ConstructSamuraiBoard (par.blockWidth,
                               par.blockHeight,
                               par.diagonals);
  else
    b = ConstructCustomBoard (par.blockWidth,
                              par.blockHeight,
                              par.diagonals,
                              par.nGrids,
                              par.offsetsX,
                              par.offsetsY);

  pThread->advance( 25);

  if (checkCancel(pThread) )
  {
    SYSTEM_PAUSE;
    DestroyBoard(b);
    FreeParams (&par);
    return -1;
  }

  if (!b)
    fprintf (stderr, "ERROR: Not enough free memory available\n");
  else
  {
    nCl = GetNumCells (b);
    cells   = (int*) DM_NEW ( malloc (nCl*sizeof(int))      );
    symbols = (int*) DM_NEW ( malloc (nCl*sizeof(int))      );
    masks   = (int*) DM_NEW ( malloc (nCl*sizeof(int))      );
    given   = (char*)DM_NEW ( malloc ((nCl+1)*sizeof(char)) );

    if (!cells || !symbols || !masks || !given)
      fprintf (stderr, "ERROR: Not enough free memory available\n");
    else
    {
      serial = 0;

 /*     if ((pf = fopen ("serial.txt", "rt")))
      {
        fscanf (pf, "%d", &serial);
        fclose (pf);

        if (serial<0 || serial>=MAX_NUM)
          serial = 0;
      }

      if ((pf = fopen ("serial.txt", "wt")))
      {
        fprintf (pf, "%d\n", serial+par.num);
        fclose (pf);
      }
*/
      pThread->advance( 75);

      if (checkCancel(pThread) )
      {
        SYSTEM_PAUSE;
	free (DM_DEL (cells)  );
	free (DM_DEL (symbols));
	free (DM_DEL (masks)  );
	free (DM_DEL (given)  );
	FreeParams (&par);
	DestroyBoard(b);
        return -1;
      }

      bool aux= false;
      if (pThread)
	aux = pThread->highdificult;
      n = CreateProblems (b, &par,
                          cells, symbols, masks, given,
                          serial, pThread, aux);

      if (checkCancel(pThread) )
      {
        SYSTEM_PAUSE;
	free (DM_DEL (cells)  );
	free (DM_DEL (symbols));
	free (DM_DEL (masks)  );
	free (DM_DEL (given)  );
	FreeParams (&par);
	DestroyBoard(b);
        return -1;
      }
//      printf ("%d problems created.\n", n);
    }

    free (DM_DEL (cells)  );
    free (DM_DEL (symbols));
    free (DM_DEL (masks)  );
    free (DM_DEL (given)  );

    pThread->advance( 99);

    DestroyBoard (b);
  }

  FreeParams (&par);

//  SYSTEM_PAUSE;
  return 0;
}

/*////////////////////////////////////////////////////////////// */

int CreateProblems (t_board_p b, t_params * p,
                    int * cells, int * symbols, int * masks,
                    char * given,
                    int serial,
                    CreatorThread * pThread /*= NULL*/,
                    bool highdificult/*= false*/)
{
  int np=0, nSy=0, nCl=0;
  int i=0, j=0, k=0, n=0, c=0, s=0, r;
  int * index=NULL;
  char name[50], bRecycle=0;
  FILE * pf=NULL;

  index = masks;

  nSy = GetNumSymbols (b);
  nCl = GetNumCells (b);
  bRecycle = p->nGrids<=1 && !p->diagonals;

  for (np=0; np<p->num; np++)
  {
    if (!bRecycle)
    {
      r = FillBoard (b, cells, symbols, masks, pThread);
      
      if (r>0)
        return np;
      else if (r<0)
        return -1;
    }
    else if (LoadBoard(b,p)==0)
      ShuffleBoard (b, p, cells, symbols, index);
    else
    {
      r = FillBoard (b, cells, symbols, masks, pThread);

      if (r>0)
        return np;
      else if (r<0)
        return -1;


      if (checkCancel(pThread) )
      {
       return -1;
      }


      SaveBoard (b, p);
    }

     // Start to Solve the sudoku
    do
    {
	
	
	pThread->advance( 77);
	random_index (nCl, cells);
	
	for (i=0; i<nCl; i++)
	{
	symbols [i] = GetSymbolOfCell (b, cells[i]);
	given[i] = true;
	}
	
	
	
	
	if (checkCancel(pThread) )
	{
	return -1;
	}

				/* Now, try to remove */
	for (j=0, n=nCl; j<n;   )  /* unnecesary given values */
	{                          /* (No j++ here. See below) */
	CleanBoard (b);
	
	for (i=0; i<n; i++)        /* Set all but one */
		if (i!=j)
		SetSymbolInCell (b, symbols[i],
				cells[i]);
	
	Solve (b, 0, 0, NULL);     /* Try to solve */
	
	if (GetState(b)==solved)   /* If it works, then that */
	{                          /* given value can be removed */
		n --;
		s = 1;
	
		for (i=j; i<n; i++)
		{
		symbols[i] = symbols[i+1];
		cells[i] = cells[i+1];
		}
	}
	else
		j ++;
	
	if (checkCancel(pThread) )
	{
	return -1;
	}
	
	}
	
	pThread->advance( 85);
	CleanBoard (b);
	
	for (i=0; i<n; i++)
	SetSymbolInCell (b, symbols[i],
				cells[i]);
	

	if (checkCancel(pThread) )
	{
	return -1;
	}
	Solve (b, 0, 0, NULL); /* Try solving again */
                               /* If dificult is like the selected option, leave it */


    } while (!GetNumRulesNGt1(b) && highdificult || GetNumRulesNGt1(b) && !highdificult );
			       
    if (!GetNumRulesNGt1(b))   /* If it's not so difficult */
    {                          

 /*            New version by Victor Antolin                        */
      if (p->level == 0)
      {
	k = nCl/5 + my_random (
                    nCl/2-nCl/5 );
      }				     /* Choose a random number */
                                     /* of given values */
                                    /* 20%-50% of cells */

      else k= nCl * int ( p->level);

      if (checkCancel(pThread) )
      {
       return -1;
      }
/********************************************************************/
      for (i=0; i<nCl; i++)
        given[i] = 0;

      for (i=0; i<n; i++)
        given[cells[i]] = 1;

      if (n<k)                 /* Add given values up to */
        do                     /* the chosen number */
        {
          do
            c = my_random (nCl);
          while (given[c]);

          symbols[n] = GetSymbolOfCell (b, c);
          cells[n] = c;
          given[c] = 1;

          n ++;

		if (checkCancel(pThread) )
		{
			return -1;
		}

        }
        while (n<k);
    }

#ifdef _DEBUG
    if (GetNumRulesNGt1(b))  printf ("IT's a dificult game\n");	
#endif
    CleanBoard (b);

    for (i=0; i<n; i++)
      SetSymbolInCell (b, symbols[i],
                          cells[i]);

    Solve (b, 0, 0, NULL);



    if ( p->name == NULL)
    {
	if (GetNumRulesNGt2(b))  
	sprintf (name, "%sN3x%02d-N2x%02d-%03d-%05d.txt",
			p->diagonals ? "D-" : "",
			GetNumRulesNGt2(b),
			GetNumRulesN2(b),
			nCl-n, np+serial);
	else if (GetNumRulesN2(b))
	sprintf (name, "%sN2x%02d-%03d-%05d.txt",
			p->diagonals ? "D-" : "",
			GetNumRulesN2(b),
			nCl-n, np+serial);
	else
	sprintf (name, "%sN1-%03d-%05d.txt",
			p->diagonals ? "D-" : "",
			nCl-n, np+serial);
    }
    else 
         sprintf (name, "%s.txt",
			p->name);

    pf = fopen (name, "wt");

    if (!pf)
    {
      perror (name);
      break;
    }

      if (checkCancel(pThread) )
      {
       return -1;
      }


    CleanBoard (b);

    for (i=0; i<n; i++)
      SetSymbolInCell (b, symbols[i],
                          cells[i]);

    PrintBoard (b, pf, NULL, NULL);

    GetBoardRaw (b, given, NULL, NULL);

    fprintf (pf, "\n\n%d\n%s\n#\n%s\n",
                 n, name, given);

    for (i=0; i<40; i++)
      fputc ('\n', pf);


     if (checkCancel(pThread) )
     {
       return -1;
     }

    Solve (b, 0, 0, NULL);


     if (checkCancel(pThread) )
     {
      return -1;
     }

    PrintBoard (b, pf, NULL, NULL);

    GetBoardRaw (b, given, NULL, NULL);

    fprintf (pf, "\n@\n%s\n", given);

    fclose (pf);

    serial ++;

    if (serial>=MAX_NUM)
      serial = MAX_NUM;
  }

  return np;
}

/*////////////////////////////////////////////////////////////// */

void ShuffleBoard (t_board_p b, t_params * p,
                   int * A, int * B, int * index)
{
  int nSy, nCl, w, h, i, x, y;
  int * rows, * cols;

  nCl = GetNumCells (b);
  nSy = GetNumSymbols (b);
  w = p->blockWidth;
  h = p->blockHeight;

  rows = index;
  cols = index + nSy;

  for (i=0; i<nCl; i++)
    A[i] = GetSymbolOfCell (b, i);

  for (i=0; i<w; i++)
    random_index (h, rows+i*h);

  random_index (w, cols);

  for (i=0; i<nSy; i++)
    rows[i] += h * cols[i/h];

  for (y=0; y<nSy; y++)
    for (x=0; x<nSy; x++)
      B[y*nSy+x] = A[rows[y]*nSy+x];

  for (i=0; i<h; i++)
    random_index (w, cols+i*w);

  random_index (h, rows);

  for (i=0; i<nSy; i++)
    cols[i] += w * rows[i/w];

  for (y=0; y<nSy; y++)
    for (x=0; x<nSy; x++)
      A[y*nSy+x] = B[y*nSy+cols[x]];

  random_index (nSy, index);

  CleanBoard (b);

  for (i=0; i<nCl; i++)
    if (A[i]>=0 && A[i]<nSy)
      SetSymbolInCell (b, index[A[i]], i);
}

/*////////////////////////////////////////////////////////////// */

int FillBoard (t_board_p b,
               int * cells, int * symbols, int * masks,
               CreatorThread * pThread /*= NULL*/)
{
  int nSy, nCl, i, j, n, c, minPV, nPV, nGss;
  e_state s;

  CleanBoard (b);

  nCl = GetNumCells (b);
  nSy = GetNumSymbols (b);

  nGss = 0;
  cells[nGss] = 0;
  symbols[nGss] = my_random (nSy);
  SetSymbolInCell (b, symbols[nGss], cells[nGss]);
  nGss ++;

  for (;;)
  {
    if (checkCancel(pThread))
      return -1;
    
    Solve (b, 0, 0, NULL);
    s = GetState (b);

    if (s==solved)
      return 0;

    if (s==impossible)
    {
      while (nGss>1 && !masks[nGss-1])
        nGss --;

      if (nGss==1)
        return -1;

      CleanBoard (b);

      for (i=0; i<nGss-1; i++)
        SetSymbolInCell (b, symbols[i], cells[i]);
    }
    else
    {
      for (i=1, c=0, minPV=nSy+1; i<nCl; i++)
      {
        nPV = GetNumPossValuesOfCell (b, i);

        if (nPV==2)
        {
          minPV = nPV;
          c = i;
          break;
        }
        else if (nPV>2 && nPV<minPV)
        {
          minPV = nPV;
          c = i;
        }
      }

      masks[nGss] = GetPossValuesOfCell (b, c);
      cells[nGss] = c;
      nGss ++;
    }

    for (i=n=0; i<nSy; i++)
      if ((1<<i) & masks[nGss-1])
        n ++;

    j = my_random (n);

    for (i=n=0; 1; i++)
      if ((1<<i) & masks[nGss-1])
      {
        if (n==j)
        {
          symbols[nGss-1] = i;
          masks[nGss-1] &= ~ (1<<i);
          break;
        }

        n ++;
      }

    SetSymbolInCell (b, symbols[nGss-1],
                        cells[nGss-1]);
  }
}

/*////////////////////////////////////////////////////////////// */

int LoadBoard (t_board_p b, t_params * p)
{
  int w, h;
  char name[20];
  FILE * pf;

  w = p->blockWidth;
  h = p->blockHeight;

  sprintf (name, "%dx%d.txt", w, h);

  pf = fopen (name, "rt");

  if (!pf)
    return -1;

  ReadBoard (b, pf, NULL, NULL);
  fclose (pf);

  Solve (b, 0, 0, NULL);

  return GetState(b)==solved ? 0 : -1;
}

/*////////////////////////////////////////////////////////////// */

void SaveBoard (t_board_p b, t_params * p)
{
  int w, h;
  char name[20];
  FILE * pf;

  w = p->blockWidth;
  h = p->blockHeight;

  sprintf (name, "%dx%d.txt", w, h);

  pf = fopen (name, "wt");

  if (pf)
  {
    PrintBoard (b, pf, NULL, NULL);
    fclose (pf);
  }
}

/*////////////////////////////////////////////////////////////// */

int ParseParams (t_params * p, QStringList arg)
{
  int i, j;
  char ok, wrong, incompatible, nomem;



  QList<QByteArray> arguments;


  p->blockWidth = p->blockHeight = 3;   /* Default parameters */
  p->nGrids = 0;
  p->offsetsX = p->offsetsY = NULL;
  p->samurai = p->diagonals = false;
  p->num = 10;
  p->restrictinters = 0;
  p->name = NULL; // Add By Victor Antolin
  p->level = 0;

  ok = true;
  wrong = incompatible = nomem = false;
  QByteArray aux;
  for (i=0; i< arg.size() && ok; i++)  /* Compatibility Change QStingList -> List<QByteArray>. Because it's compatible with stdlib.h */
  {
	
	aux = arg[i].toLatin1();

	arguments.append(aux);
  }


  for (i=0; i< arg.size() && ok; i++)  /* Parse parameters */
  {
    
    if (!strcmp(arguments[i].data(),"-samurai"))  /* Samurai board */
    {
      if (p->nGrids)
      {
        ok = false;
        incompatible = true;
        break;
      }

      p->samurai = true;
    }
/***********ADDED BY VICTOR ANTOLI)N ************************/
    else if (!strcmp(arguments[i].data(),"-name"))  /* Name */
    {
      i ++;
      if (i+1>arg.size())
      {
        ok = false;
        wrong = true;
        break;
      }
      p->name = (char*) DM_NEW ( malloc ( sizeof(arguments[i])) );
      if (p->name==NULL)
      {
        ok = false;
        wrong = true;
        break;
      }
    
#ifdef _DEBUG
      printf("Name: %s \n", arguments[i].data());
#endif
      strcpy(p->name,arguments[i].data());
      //i ++;
    }


    else if (!strcmp(arguments[i].data(),"-level"))  /* Difficult level */
    {
      i ++;
      if (i+1>arg.size())
      {
        ok = false;
        wrong = true;
        break;
      }
      p->level = atof ( arguments[i].data());
      
      if ( p->level <= 0 || p->level >100 )
      {
        p->level=0;
        break;
      }
      else
      {
	p->level = 0.2 + 0.003 * (100 - p->level); 
      }
	
      //i ++;
    }
/***************************************************/
     else if (!strcmp(arguments[i].data(),"-grids"))  /* Custom board */
    {
      if (p->samurai)
      {
        ok = false;
        incompatible = true;
        break;
      }

      i ++;
      if (i>=arg.size())
      {
        ok = false;
        wrong = true;
        break;
      }

      p->nGrids = -1;
      p->nGrids = atoi (arguments[i].data());

      if (p->nGrids<2 || p->nGrids>MAX_GRIDS)
      {
        p->nGrids = 0;
        ok = false;
        wrong = true;
        break;
      }

      p->offsetsX = (int*) DM_NEW ( malloc (p->nGrids * sizeof(int)) );
      p->offsetsY = (int*) DM_NEW ( malloc (p->nGrids * sizeof(int)) );

      if (!p->offsetsX || !p->offsetsY)
      {
        ok = false;
        nomem = true;
        break;
      }

      for (j=0; j<p->nGrids; j++, i+=2)
      {
         if (i+2>=arg.size())
        {
          ok = false;
          wrong = true;
          break;
        }

        p->offsetsX[j] = p->offsetsY[j] = MAX_DIST+1;

	p->offsetsX[j] = atoi (arguments[i+1].data());
        p->offsetsY[j] = atoi (arguments[i+2].data());


        if (p->offsetsX[j]<-MAX_DIST ||
            p->offsetsX[j]>MAX_DIST ||
            p->offsetsY[j]<-MAX_DIST ||
            p->offsetsY[j]>MAX_DIST)
        {
          ok = false;
          wrong = true;
          break;
        }
      }

      if (!ok)
        break;
    }
   else if (!strcmp(arguments[i].data(),"-blocksize"))  /* Block size */
    {
      i ++;
      if (i+1>=arg.size())
      {
        ok = false;
        wrong = true;
        break;
      }

      p->blockWidth = p->blockHeight = -1;

      p->blockWidth = atoi (arguments[i].data());
      p->blockHeight = atoi (arguments[i+1].data());

      if (p->blockWidth < 2 ||
          p->blockHeight < 2 ||
          p->blockWidth*p->blockHeight > 30)
      {
        ok = false;
        wrong = true;
        break;
      }

      i ++;
    }
    else if (!strcmp(arguments[i].data(),"-num"))  /* Number of problems */
    {                                  /* to create */
      i ++;
      
      if (i>=arg.size())
      {
        ok = false;
        wrong = true;
        break;
      }

      p->num = -1;
      p->num = atoi (arguments[i].data());

      if (p->num < 1 ||
          p->num > MAX_NUM)
      {
        ok = false;
        wrong = true;
        break;
      }
    }
    else if (!strcmp(arguments[i].data(),"-restrictinters")) /* Maximum */
    {                                            /* number of */
      i ++;                                    /* intersected */
                                              /* groups in cells */
      if (i>=arg.size())                         /* with given values */
      {
        ok = false;
        wrong = true;
        break;
      }

      p->restrictinters = -1;
      p->restrictinters = atoi (arguments[i].data());

      if (p->restrictinters < 0)
      {
        ok = false;
        wrong = true;
        break;
      }
    }
    else if (!strcmp(arguments[i].data(),"-diagonals"))  /* Diagonals */
    {
      p->diagonals = true;
    }
    else if (!strcmp(arguments[i].data(),"-h"))  /* Help */
    {
      printf ("\n\tUSAGE:\n\tdemo5-inverse-creator "
              "[parameters]\n\n"
              "\tPOSSIBLE PARAMETERS:\n"
              "\t  -h\t(show this help)\n"
              "\t  -samurai\n"
              "\t  -grids n x1 y1 x2 y2... xn yn\n"
              "\t  -blocksize width height\n"
              "\t  -diagonals\n"
              "\t  -num n\n"
              "\t  -restrictinters numinters\n"
	      "\t  -name name_of_file (returns name_of_file.txt)\n"
	      "\t  -level xx  <level of difficult>|  0 < xx < 100\n"
	      "\t             if level=0 random level is selected\n"
              "\n");
      FreeParams (p);
      return -1;
    }
    else
    {
      ok = false;
      wrong = true;
    }
   }
  if (ok)
    return 0;

  if (wrong)
    fprintf (stderr, "ERROR: Wrong parameters\n");
  else if (incompatible)
    fprintf (stderr, "ERROR: Incompatible parameters\n");
  else if (nomem)
    fprintf (stderr, "ERROR: Not enough free memory available\n");

  FreeParams (p);
  return -1;
}

/*////////////////////////////////////////////////////////////// */

void FreeParams (t_params * p)
{
  if (p->offsetsX)
  {
    free (DM_DEL (p->offsetsX));
    p->offsetsX = NULL;
  }

  if (p->offsetsY)
  {
    free (DM_DEL (p->offsetsY));
    p->offsetsY = NULL;
  }
  if (p->name)
  {
	free (DM_DEL (p->name));
  }
}

/*////////////////////////////////////////////////////////////// */

int my_random (int num)
{
  double d;
  int r;

  d = rand () / (double) RAND_MAX;
  r = (int) (d*num);

  return r<0 ? 0 : r>=num ? num-1 : r;
}

/*////////////////////////////////////////////////////////////// */

void random_index (int num, int * index)
{
  int i, j, k, m;

  for (i=0; i<num; i++)
    index[i] = -1;

  for (i=0; num; i++, num--)
  {
    j = my_random (num);

    for (k=m=0; 1; k++)
      if (index[k]==-1)
      {
        if (m==j)
        {
          index[k] = i;
          break;
        }

        m ++;
      }
  }
}

