/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: thread-sensei.h    * Status: unstable * Version: 1.0 * finished in version: 0.45 	*/
/************************************************************************************************/
/*        									      		*/
/*  This file contains the definition of the CreatorThread class 				*/
/*        									      		*/
/************************************************************************************************/
#ifndef THREAD_CREATOR_H
#define THREAD_CREATOR_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <QtCore>
#include <QStringList>

#include "sudoku-pub.h"
#include "sudoku-creator.h"


#ifdef WINDOWS
#define SYSTEM_PAUSE system ("pause")
#else
#define SYSTEM_PAUSE
#endif

/*////////////////////////////////////////////////////////////// */


#define MAX_GRIDS 100
#define MAX_DIST 1000
#define MAX_NUM 10000
#define MAX_TRIALS 10


class CreatorThread;

int my_random (int num);
void random_index (int num, int * index);

/*////////////////////////////////////////////////////////////// */

class CreatorThread : public QThread
 {
	Q_OBJECT
 public:
	volatile bool flagCancel;
	bool highdificult;

 signals:
	void advanceSignal(int number); // Number = x% done, when x between (0,100)

 public slots:
	void setFlagCancel(bool flag) { flagCancel =flag;}
	void advance(int number){emit advanceSignal (number);}
 public:
     QStringList arguments;
  //   t_params par;

     void run()
     {
        threadState = running;
        
        if (sudokuCreator (arguments, this) == 0)
          flagCreated = true;
        
        threadState = not_running;
     }
     
 private:

        volatile enum { not_running, will_run, running } threadState;

 public:

        bool flagCreated;

        CreatorThread () : threadState(not_running) {}
        
        void Start_running ()  // Call this method instead of start()
        {
                threadState = will_run;
                flagCreated = false;
                flagCancel = false;
                start (QThread::LowestPriority);
        }

        void Wait_for_completion ()  // Call this method instead of wait()
        {
                if (threadState!=not_running)
                {
                        do
                                setPriority (QThread::NormalPriority);
                        while (!wait() &&
                               threadState!=not_running);
                }
        }
 };

#endif
