/*
-------------------------------------------------------------------
SUDOKU SENSEI 2.00: a Sudoku Explainer Application
Copyright (C) 2007  Victor Antolin Urtueta

This program is based on SUDOKU SENSEI 1.04, a Sudoku Explainer
Engine by Martin Knoblauch.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the authors: mentur.SudokuSensei AT gmail.com
                     comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------
*/
/************************************************************************************************/
/*  File: thread-sensei.h    * Status: unstable * Version: 1.0 * finished in version: 0.45 	*/
/************************************************************************************************/
/*        									      		*/
/*  This file contains the definition of the SenseiThread class and s_rule struct		*/
/*        									      		*/
/************************************************************************************************/

#ifndef THREAD_SENSEI_H
#define THREAD_SENSEI_H

#include <QtCore>
#include <QList>
#include <QMutex>

#include "sudoku-pub.h"

struct s_rule
{
  e_rule type;
  int level;
  int group;
  int inters_group;
  int cells_mask;
  int symbols_mask;
};

struct s_rule_node;

class SenseiThread : public QThread
{
	Q_OBJECT

  private:

	t_board_p work_board;
	bool flagOnlyCheckIntegrity;
	volatile char flagCancelOperations;
        volatile enum { not_running, will_run, running } threadState;

    s_rule_node * first, * last;
    QMutex prot_rules_mutex;

    void AppendRule (const s_rule & r);
    void Clear ();

    const SenseiThread & operator= (const SenseiThread & x)
    { return *this; }

  public:

    SenseiThread ()
      : work_board(NULL),
        flagCancelOperations(1),
        threadState(not_running),
        first(NULL)
    {}

    ~SenseiThread ()
    { Clear (); }

    int DumpRules ();
    QList<s_rule> rules;

    void Init (t_board_p pBoard, bool bOnlyIntegrity);
    void run();
    void Abort () { flagCancelOperations = 1; }

    void Start_running ()  // Call this method instead of start()
    {
      threadState = will_run;
      start (QThread::LowestPriority);
    }

    void Wait_for_completion ()  // Call this method instead of wait()
    {
      if (threadState!=not_running)
      {
        do
          setPriority (QThread::NormalPriority);
        while (!wait() &&
               threadState!=not_running);
      }
    }

  signals:	

	void NewRule();
    void Aborted();
	void IntegrityChecked (int result);
	void NoMoreRulestoFind();
	void Error(int reason);
	
  public slots:
};

#endif

